"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const node_crypto_1 = require("node:crypto");
const amqp_client_1 = require("@jfrog-pipelines/amqp-client");
const internal_clients_1 = require("@jfrog-pipelines/internal-clients");
const logger_1 = require("@jfrog-pipelines/logger");
const utility_1 = require("@jfrog-pipelines/utility");
const microservice_1 = require("@jfrog-pipelines/microservice");
const NodePoolWorker_1 = require("../NodePoolWorker");
const modules_1 = require("../modules");
const NodeStatsCleanupCommand_1 = require("../modules/nodeStats/NodeStatsCleanupCommand");
const NodeStatsCleanupHandler_1 = require("../modules/nodeStats/NodeStatsCleanupHandler");
const BuildPlaneMessageService_1 = require("../modules/buildPlaneMessage/BuildPlaneMessageService");
jest.mock('@jfrog/access-nodejs-client');
jest.mock('@jfrog-pipelines/amqp-client');
jest.mock('@jfrog-pipelines/configuration', () => {
    return {
        ConfigFactory: { config: { amqpExchange: '' } }
    };
});
jest.mock('@jfrog-pipelines/logger', () => {
    const actualTracker = jest.requireActual('@jfrog-pipelines/logger');
    return {
        ...actualTracker,
        LoggerFactory: {
            getLogger: jest.fn().mockReturnValue({
                info: jest.fn(),
                error: jest.fn(),
                debug: jest.fn(),
                trace: jest.fn(),
                warn: jest.fn()
            }),
            configure: jest.fn(),
            logEvent: jest.fn(),
            getEventLogger: jest.fn().mockReturnValue({
                info: jest.fn(),
                error: jest.fn(),
                debug: jest.fn(),
                trace: jest.fn(),
                warn: jest.fn()
            })
        }
    };
});
jest.spyOn(internal_clients_1.AccessClientFactory, 'getClient').mockReturnValue({
    getNodeAccessToken: jest.fn().mockReturnValue('token')
});
const client = new amqp_client_1.AmqpConnectionManagerProxy('testmsCode', 'testmsversion', { amqpUrl: '', heartbeatIntervalInSeconds: 30 }, logger_1.LoggerFactory.getLogger('NodePoolWorkerTest'));
const publisher = new amqp_client_1.AmqpPublisher(client.getConnectionManager(), 'testexchange', amqp_client_1.publisherOptions, logger_1.LoggerFactory.getLogger('NodePoolWorkerTest'));
const store = microservice_1.applicationContext;
const serviceEnum = 12;
const serviceDetails = utility_1.ServiceMeta[serviceEnum];
describe('Test NodePoolWorker #process', () => {
    it('should process message as node pool id in case of node pool is on demand', async () => {
        const nodeService = new modules_1.NodePoolService();
        const provisioner = new modules_1.NodeProvisioner(publisher);
        const collector = new modules_1.NodeCollector(publisher);
        const eventLoggerSpy = jest.spyOn(logger_1.LoggerFactory, 'logEvent');
        jest.spyOn(modules_1.NodeCollector.prototype, 'deProvision').mockImplementationOnce(() => Promise.resolve());
        jest.spyOn(BuildPlaneMessageService_1.BuildPlaneMessageService.prototype, 'getReadyMessageCount').mockResolvedValueOnce(2);
        jest.spyOn(modules_1.NodeProvisioner.prototype, 'provision').mockImplementationOnce(() => Promise.resolve());
        jest.spyOn(modules_1.NodePoolService.prototype, 'getNodePool').mockResolvedValue({ id: 1, isOnDemand: true });
        const lockResponse = { id: (0, node_crypto_1.randomUUID)() };
        jest.spyOn(modules_1.LockService.prototype, 'obtainLock').mockResolvedValue(lockResponse);
        jest.spyOn(modules_1.LockService.prototype, 'releaseLock').mockResolvedValue();
        store.set(microservice_1.SYSTEM_SERVICE_INSTANCE_INITIALIZER, { systemServiceInstancesId: 1 });
        const poolWorker = new NodePoolWorker_1.NodePoolWorker(serviceDetails, store);
        await poolWorker.process({ payload: { nodePool: { nodePoolId: 1 } } });
        expect(nodeService.getNodePool).toHaveBeenCalled();
        expect(provisioner.provision).toHaveBeenCalledTimes(1);
        expect(collector.deProvision).toHaveBeenCalledTimes(1);
        expect(eventLoggerSpy).toHaveBeenCalledWith(expect.objectContaining({ count: 2, domain: 'nodepool', eventType: 'onDemandNodes', nodePoolName: undefined }));
    });
    it('should not process message as node pool id in case of node pool is not on demand', async () => {
        const nodeService = new modules_1.NodePoolService();
        const provisioner = new modules_1.NodeProvisioner(publisher);
        const collector = new modules_1.NodeCollector(publisher);
        jest.spyOn(BuildPlaneMessageService_1.BuildPlaneMessageService.prototype, 'getReadyMessageCount').mockResolvedValueOnce(2);
        jest.spyOn(modules_1.NodeCollector.prototype, 'deProvision').mockImplementationOnce(() => Promise.resolve());
        jest.spyOn(modules_1.NodeProvisioner.prototype, 'provision').mockImplementationOnce(() => Promise.resolve());
        jest.spyOn(modules_1.NodePoolService.prototype, 'getNodePool').mockResolvedValue({ id: 1, isOnDemand: false });
        store.set(microservice_1.SYSTEM_SERVICE_INSTANCE_INITIALIZER, { systemServiceInstancesId: 1 });
        const poolWorker = new NodePoolWorker_1.NodePoolWorker(serviceDetails, store);
        await poolWorker.process({ payload: { nodePool: { nodePoolId: 1 } } });
        expect(nodeService.getNodePool).toHaveBeenCalled();
        expect(provisioner.provision).toHaveBeenCalledTimes(0);
        expect(collector.deProvision).toHaveBeenCalledTimes(0);
    });
    it('should not throw error in case of any errors', async () => {
        const provisioner = new modules_1.NodeProvisioner(publisher);
        const collector = new modules_1.NodeCollector(publisher);
        jest.spyOn(BuildPlaneMessageService_1.BuildPlaneMessageService.prototype, 'getReadyMessageCount').mockResolvedValueOnce(2);
        jest.spyOn(modules_1.NodeCollector.prototype, 'deProvision').mockRejectedValue({});
        jest.spyOn(modules_1.NodeProvisioner.prototype, 'provision').mockImplementationOnce(() => Promise.resolve());
        jest.spyOn(modules_1.NodePoolService.prototype, 'getNodePool').mockResolvedValue({ id: 1, isOnDemand: false });
        store.set(microservice_1.SYSTEM_SERVICE_INSTANCE_INITIALIZER, { systemServiceInstancesId: 1 });
        const poolWorker = new NodePoolWorker_1.NodePoolWorker(serviceDetails, store);
        await poolWorker.process({ payload: { nodePool: { nodePoolId: 1 } } });
        expect(provisioner.provision).toHaveBeenCalledTimes(0);
        expect(collector.deProvision).toHaveBeenCalledTimes(0);
    });
    it('should process command', async () => {
        const message = {
            payload: {
                command: {
                    actor: utility_1.Actor.nodeStats,
                    operation: utility_1.Operation.delete
                }
            }
        };
        store.set(microservice_1.SYSTEM_SERVICE_INSTANCE_INITIALIZER, { systemServiceInstancesId: 1 });
        const poolWorker = new NodePoolWorker_1.NodePoolWorker(serviceDetails, store);
        const command = new NodeStatsCleanupCommand_1.NodeStatsCleanupCommand(message.payload.command.operation);
        const nodeStatsCleanupSpy = jest.spyOn(NodeStatsCleanupHandler_1.NodeStatsCleanupHandler.prototype, 'handle').mockResolvedValueOnce();
        await poolWorker.process(message);
        expect(nodeStatsCleanupSpy).toHaveBeenCalledTimes(1);
        expect(nodeStatsCleanupSpy).toHaveBeenCalledWith(command);
    });
    it('should not process command', async () => {
        const logger = logger_1.LoggerFactory.getLogger();
        const message = {
            payload: {
                command: {
                    actor: utility_1.Actor.nodeStats,
                    operation: utility_1.Operation.delete
                }
            }
        };
        store.set(microservice_1.SYSTEM_SERVICE_INSTANCE_INITIALIZER, { systemServiceInstancesId: 1 });
        const poolWorker = new NodePoolWorker_1.NodePoolWorker(serviceDetails, store);
        const command = new NodeStatsCleanupCommand_1.NodeStatsCleanupCommand(message.payload.command.operation);
        const nodeStatsCleanupSpy = jest.spyOn(NodeStatsCleanupHandler_1.NodeStatsCleanupHandler.prototype, 'handle').mockRejectedValueOnce('error');
        await poolWorker.process(message);
        expect(nodeStatsCleanupSpy).toHaveBeenCalledTimes(1);
        expect(nodeStatsCleanupSpy).toHaveBeenCalledWith(command);
        expect(logger.error).toHaveBeenCalledTimes(1);
    });
});
//# sourceMappingURL=NodePoolWorker.test.js.map