'use strict';
var self = handler;
module.exports = self;
var fs = require('fs');
var path = require('path');
var AZAdapter = require('../interface/az/Adapter.js');
var ApiAdapter = require('../../Adapter.js');
var azMasterIntegrationName = 'azureKeys';
var nodeActions = require('../../nodeActions.js');
var nodeMetadataMap = require('../../nodeMetadataMap.js');
function handler(params, callback) {
    var bag = {
        action: params.action,
        nodeName: params.nodeName,
        nodeId: params.nodeId,
        operatingSystem: params.operatingSystem,
        instanceId: params.instanceId,
        architecture: params.architecture,
        startupScriptData: params.startupScriptData || {},
        providerMetadata: params.providerMetadata || {},
        nodeProviderMetadata: params.nodeProviderMetadata || {},
        appId: params.appId,
        tenant: params.tenant,
        password: params.password,
        region: params.region,
        imageId: params.imageId,
        imageMetadata: params.imageMetadata,
        consoleAdapter: params.consoleAdapter,
        projectId: params.projectId,
        nodePoolId: params.nodePoolId,
        isSuccess: true,
        customData: '',
        skipConsolePost: false,
        jFrogPipelinesSessionId: params.jFrogPipelinesSessionId,
        diskSizeinGB: params.diskSizeinGB
    };
    bag.who = util.format('az|%s|jFrogPipelinesSessionId:%s', self.name, bag.jFrogPipelinesSessionId);
    logger.debug(bag.who, 'Inside');
    if (_.has(bag.startupScriptData, 'insecureDockerRegistries')) {
        bag.startupScriptData.insecureDockerRegistries =
            JSON.stringify(bag.startupScriptData.insecureDockerRegistries);
    }
    else {
        bag.startupScriptData.insecureDockerRegistries = JSON.stringify([]);
    }
    async.series([
        _checkInputParams.bind(null, bag),
        _initializeAZAdapter.bind(null, bag),
        _loadTemplateFile.bind(null, bag),
        _renderTemplate.bind(null, bag),
        _encodeVMData.bind(null, bag),
        _createVM.bind(null, bag),
        _cacheVM.bind(null, bag),
        _startVM.bind(null, bag),
        _deleteVM.bind(null, bag)
    ], function (err) {
        var result = {};
        if (bag.consoleAdapter)
            bag.consoleAdapter.closeGrp(bag.isSuccess);
        if (err) {
            logger.warn(bag.who, 'reported an error', err);
        }
        else {
            result = {
                instance: bag.vm,
                isSuccess: bag.isSuccess,
                deletingVM: !!bag.deletingVM
            };
            logger.debug(bag.who, 'Successfully completed processing of az');
        }
        result.errorForMetric = bag.errorForMetric;
        if (bag.componentsCreated)
            result.componentsCreated = bag.componentsCreated;
        if (bag.tags)
            result.tags = bag.tags;
        return callback(err, result);
    });
}
function _checkInputParams(bag, next) {
    var who = bag.who + '|' + _checkInputParams.name;
    logger.trace(who, 'Inside');
    bag.suAdapter = new ApiAdapter(bag.jFrogPipelinesSessionId, global.config.apiToken);
    var consoleErrors = [];
    if (!bag.action)
        consoleErrors.push(who + ' is missing: action');
    var expectedParams = [
        'action',
        'nodeId',
        'appId',
        'tenant',
        'password',
        'consoleAdapter',
        'resourceGroupName',
        'subscriptionId',
        'nodeName'
    ];
    if (bag.action === nodeActions.CREATE) {
        _.extend(bag, bag.providerMetadata);
        expectedParams = expectedParams.concat([
            'providerMetadata',
            'startupScriptData',
            'imageId',
            'subnetName',
            'vnetName',
            'startupScriptData',
            'instanceSize',
            'imageMetadata',
            'nodePoolId',
            'projectId'
        ]);
        if (!bag.operatingSystem.startsWith('Windows'))
            expectedParams = expectedParams.concat([
                'publicSshKey'
            ]);
    }
    else {
        _.extend(bag, bag.nodeProviderMetadata);
    }
    _.each(expectedParams, function (expectedParam) {
        if (_.isNull(bag[expectedParam]) || _.isUndefined(bag[expectedParam]))
            consoleErrors.push(who + ' is missing ' + expectedParam);
    });
    if (bag.action === nodeActions.DELETE)
        bag.skipConsolePost = true;
    if (consoleErrors.length > 0) {
        _.each(consoleErrors, function (err) {
            logger.error(bag.who, err);
        });
        return next(true);
    }
    return next();
}
function _initializeAZAdapter(bag, next) {
    var who = bag.who + '|' + _initializeAZAdapter.name;
    logger.trace(who, 'Inside');
    bag.azAdapter = new AZAdapter(bag.appId, bag.tenant, bag.password, bag.subscriptionId);
    if (!bag.skipConsolePost) {
        var action;
        if (bag.action === nodeActions.CREATE)
            action = 'Creating';
        else if (bag.action === nodeActions.START)
            action = 'Starting';
        else if (bag.action === nodeActions.CACHE)
            action = 'Stopping';
        bag.consoleAdapter.openGrp(action + ' node: ' + bag.nodeId);
        bag.consoleAdapter.openCmd('Initializing AZ adapter');
    }
    bag.azAdapter.initialize(function (err) {
        if (err) {
            bag.errorForMetric = err.code;
            if (!bag.skipConsolePost) {
                bag.consoleAdapter.publishMsg('Failed to initialize az adapter');
                bag.consoleAdapter.closeCmd(false);
            }
            logger.warn(who, 'Failed to initialize AZ Adapter for Node: ' +
                bag.nodeId, err);
            bag.isSuccess = false;
            return next();
        }
        if (!bag.skipConsolePost) {
            bag.consoleAdapter.publishMsg('Successfully initialized az adapter');
            bag.consoleAdapter.closeCmd(true);
        }
        return next();
    });
}
function _loadTemplateFile(bag, next) {
    if (!bag.isSuccess)
        return next();
    if (bag.action !== nodeActions.CREATE)
        return next();
    var who = bag.who + '|' + _loadTemplateFile.name;
    logger.trace(who, 'Inside');
    var templateFile = util.format('templates/%s/%s/boot.%s', bag.architecture, bag.operatingSystem, nodeMetadataMap.getScriptFileExtension(bag.architecture, bag.operatingSystem));
    var templatePath = path.join(__dirname, templateFile);
    var templateString = fs.readFileSync(templatePath).toString();
    bag.template = _.template(templateString);
    return next();
}
function _renderTemplate(bag, next) {
    if (!bag.isSuccess)
        return next();
    if (bag.action !== nodeActions.CREATE)
        return next();
    var who = bag.who + '|' + _renderTemplate.name;
    logger.trace(who, 'Inside');
    bag.customData = bag.template(bag.startupScriptData);
    return next();
}
function _encodeVMData(bag, next) {
    if (!bag.isSuccess)
        return next();
    if (bag.action !== nodeActions.CREATE)
        return next();
    var who = bag.who + '|' + _encodeVMData.name;
    logger.trace(who, 'Inside');
    bag.consoleAdapter.openCmd('Generating intitialize script');
    bag.customData = Buffer.from(bag.customData).toString('base64');
    bag.consoleAdapter.publishMsg('Successfully generated intitialize script');
    bag.consoleAdapter.closeCmd(true);
    return next();
}
function _createVM(bag, next) {
    if (!bag.isSuccess)
        return next();
    if (bag.action !== nodeActions.CREATE)
        return next();
    var who = bag.who + '|' + _createVM.name;
    logger.trace(who, 'Inside');
    bag.consoleAdapter.openCmd('Creating vm');
    var innerBag = {
        who: who,
        subnetName: bag.subnetName,
        vnetName: bag.vnetName,
        securityGroupName: bag.securityGroupName,
        node: {},
        azAdapter: bag.azAdapter,
        publicSshKey: bag.publicSshKey,
        adminUsername: bag.adminUsername,
        adminPassword: bag.adminPassword,
        imageId: bag.imageId,
        imageType: bag.imageType,
        resourceGroupName: bag.resourceGroupName,
        region: bag.region,
        nodeName: bag.nodeName,
        instanceSize: bag.instanceSize,
        customData: bag.customData,
        componentsCreated: {},
        imageMetadata: bag.imageMetadata,
        architecture: bag.architecture,
        operatingSystem: bag.operatingSystem,
        projectId: bag.projectId,
        nodeId: bag.nodeId,
        nodePoolId: bag.nodePoolId,
        diskSizeinGB: bag.diskSizeinGB
    };
    async.series([
        __getSubnet.bind(null, innerBag),
        __getNetworkSecurityGroup.bind(null, innerBag),
        __createNIC.bind(null, innerBag),
        __createVM.bind(null, innerBag)
    ], function (err) {
        if (err) {
            bag.errorForMetric = err.code;
            bag.isSuccess = false;
            bag.consoleAdapter.publishMsg('Failed to create vm with error: ' +
                err.message);
            bag.consoleAdapter.closeCmd(false);
            logger.warn(who, 'Failed to create vm for node: ' + bag.nodeId +
                'with image: ' + bag.imageId, err);
        }
        else {
            bag.isSuccess = true;
            bag.consoleAdapter.publishMsg('VM requested');
            bag.consoleAdapter.publishMsg(util.inspect(innerBag.vm));
            bag.consoleAdapter.closeCmd(true);
        }
        if (innerBag.vm)
            bag.vm = innerBag.vm;
        if (innerBag.componentsCreated)
            bag.componentsCreated = innerBag.componentsCreated;
        if (innerBag.tags)
            bag.tags = innerBag.tags;
        return next();
    });
}
function __getSubnet(bag, done) {
    var who = bag.who + '|' + __getSubnet.name;
    logger.trace(who, 'Inside');
    bag.azAdapter.AZNetwork.getSubnet(bag.resourceGroupName, bag.vnetName, bag.subnetName, function (err, subnet) {
        if (err)
            return done(err);
        bag.subnet = subnet;
        return done();
    });
}
function __getNetworkSecurityGroup(bag, done) {
    if (_.isEmpty(bag.securityGroupName))
        return done();
    var who = bag.who + '|' + __getNetworkSecurityGroup.name;
    logger.trace(who, 'Inside');
    bag.azAdapter.AZNetwork.getNetworkSecurityGroup(bag.resourceGroupName, bag.securityGroupName, function (err, nsg) {
        if (err)
            return done(err);
        bag.networkSecurityGroup = nsg;
        return done();
    });
}
function __createNIC(bag, done) {
    var who = bag.who + '|' + __createNIC.name;
    logger.trace(who, 'Inside');
    var nicName = bag.nodeName + '-nic';
    bag.azAdapter.AZNetwork.createNIC(bag.resourceGroupName, bag.subnet, bag.networkSecurityGroup, bag.region, nicName, function (err, nic) {
        if (err)
            return done(err);
        bag.nic = nic;
        bag.componentsCreated.nicName = nicName;
        return done();
    });
}
function __createVM(bag, done) {
    var who = bag.who + '|' + __createVM.name;
    logger.trace(who, 'Inside');
    var vmName = bag.nodeName;
    var providerData = nodeMetadataMap.getProviderData(azMasterIntegrationName, bag.architecture, bag.operatingSystem) || {};
    var osType = providerData.osType;
    var osDiskSizeGB = bag.diskSizeinGB;
    var imageId;
    var imageUrn;
    var imagePlan;
    bag.tags = {
        'Project Id': bag.projectId.toString(),
        'Node Pool Id': bag.nodePoolId.toString(),
        'Node Id': bag.nodeId.toString(),
        'Node Name': bag.nodeName
    };
    if (bag.imageMetadata.type === 'managed') {
        imageId = bag.imageId;
    }
    else if (bag.imageMetadata.type === 'published') {
        imageUrn = bag.imageId;
        imagePlan = bag.imageMetadata.plan;
    }
    var vmParams = {
        resourceGroup: bag.resourceGroupName,
        location: bag.region,
        vmSize: bag.instanceSize,
        imageId: imageId,
        imageUrn: imageUrn,
        imagePlan: imagePlan,
        osType: osType,
        nicId: bag.nic.id,
        customData: bag.customData,
        publicSshKey: bag.publicSshKey,
        adminUsername: bag.adminUsername,
        adminPassword: bag.adminPassword,
        tags: bag.tags,
        vmName: vmName,
        osDiskSizeGB: osDiskSizeGB
    };
    bag.azAdapter.AZCompute.createVM(vmParams, function (err, vm) {
        if (err)
            return done(err);
        bag.vm = vm;
        bag.componentsCreated.vmName = vmName;
        return done();
    });
}
function _cacheVM(bag, next) {
    if (!bag.isSuccess)
        return next();
    if (bag.action !== nodeActions.CACHE)
        return next();
    var who = bag.who + '|' + _cacheVM.name;
    logger.trace(who, 'Inside');
    bag.consoleAdapter.openCmd('Stopping vm');
    bag.azAdapter.AZCompute.powerOffVM(bag.resourceGroupName, bag.nodeName, function (err, response) {
        if (err) {
            bag.errorForMetric = err.code;
            bag.consoleAdapter.publishMsg('Failed to power off vm with ' +
                'error: ' + err.message);
            bag.consoleAdapter.closeCmd(false);
            logger.warn(who, 'Failed to power off vm for node: ' + bag.nodeId, err);
            bag.isSuccess = false;
            return next();
        }
        bag.consoleAdapter.publishMsg('VM power off requested');
        bag.consoleAdapter.publishMsg(util.inspect(response));
        bag.consoleAdapter.closeCmd(true);
        return next();
    });
}
function __getVM(bag, callback) {
    var who = bag.who + '|' + __getVM.name;
    logger.trace(who, 'Inside');
    bag.azAdapter.AZCompute.getVM(bag.resourceGroupName, bag.nodeName, function (err, response) {
        return callback(err, response);
    });
}
function _startVM(bag, next) {
    if (!bag.isSuccess)
        return next();
    if (bag.action !== nodeActions.START)
        return next();
    var who = bag.who + '|' + _startVM.name;
    logger.trace(who, 'Inside');
    bag.consoleAdapter.openCmd('Starting vm');
    var innerBag = {
        who: who,
        azAdapter: bag.azAdapter,
        resourceGroupName: bag.resourceGroupName,
        nodeName: bag.nodeName
    };
    async.series([
        __checkIfVMIsStopped.bind(null, innerBag),
        __startVM.bind(null, innerBag)
    ], function (err) {
        if (err) {
            bag.errorForMetric = err.code;
            bag.consoleAdapter.publishMsg('Failed to start vm with error: ' +
                err.message);
            bag.consoleAdapter.closeCmd(false);
            logger.warn(who, 'Failed to start vm for node: ' + bag.nodeId, err);
            bag.isSuccess = false;
        }
        else {
            bag.isSuccess = true;
            bag.consoleAdapter.publishMsg('VM started');
            bag.consoleAdapter.closeCmd(true);
        }
        return next();
    });
}
function __checkIfVMIsStopped(bag, done) {
    var who = bag.who + '|' + __checkIfVMIsStopped.name;
    logger.trace(who, 'Inside');
    async.retry({
        times: 10,
        interval: 3000
    }, function (callback) {
        __getVM(bag, function (err, instanceView) {
            var canStartNode = false;
            if (err) {
                logger.warn(who, 'Failed to get instance for node: ' + bag.nodeId +
                    ' and name: ' + bag.nodeName, err);
                return callback(null, canStartNode);
            }
            logger.trace(who, 'Successfully fetched instance view of node: ' +
                bag.nodeId + ' instance name: ' + bag.nodeName);
            if (instanceView &&
                instanceView.statuses[1].code === 'PowerState/stopping')
                return callback(true, canStartNode);
            canStartNode = true;
            return callback(null, canStartNode);
        });
    }, function (err, canStartNode) {
        return done(!canStartNode);
    });
}
function __startVM(bag, done) {
    var who = bag.who + '|' + __startVM.name;
    logger.trace(who, 'Inside');
    bag.azAdapter.AZCompute.startVM(bag.resourceGroupName, bag.nodeName, function (err) {
        if (err)
            logger.warn(who, 'Failed to start vm for node: ' + bag.nodeId, err);
        return done(err);
    });
}
function _deleteVM(bag, next) {
    if (!bag.isSuccess)
        return next();
    if (bag.action !== nodeActions.DELETE)
        return next();
    var who = bag.who + '|' + _deleteVM.name;
    logger.trace(who, 'Inside');
    var innerBag = {
        who: who,
        azAdapter: bag.azAdapter,
        resourceGroupName: bag.resourceGroupName,
        vmName: bag.vmName,
        nodeName: bag.vmName,
        nicName: bag.nicName,
        vmDeleted: _.isEmpty(bag.vmName),
        nicDeleted: _.isEmpty(bag.nicName),
        nodeId: bag.nodeId,
        tags: bag.tags,
        consoleAdapter: bag.consoleAdapter
    };
    async.series([
        __checkVM.bind(null, innerBag),
        __deleteVM.bind(null, innerBag),
        __deleteDisks.bind(null, innerBag),
        __getNIC.bind(null, innerBag),
        __deleteNIC.bind(null, innerBag)
    ], function (err) {
        if (err) {
            bag.errorForMetric = err.code;
            logger.warn(who, 'Failed to delete vm id: ' + bag.instanceId + ' for ' +
                'node: ' + bag.nodeId, err);
            bag.isSuccess = false;
            return next();
        }
        else if (innerBag.vmDeleted && innerBag.nicDeleted) {
            bag.isSuccess = true;
        }
        bag.deletingVM = innerBag.deletingVM;
        return next();
    });
}
function __checkVM(bag, done) {
    if (!bag.vmName)
        return done();
    var who = bag.who + '|' + __checkVM.name;
    logger.trace(who, 'Inside');
    __getVM(bag, function (err, instanceView) {
        if (err) {
            if (err.code === 'ResourceNotFound') {
                bag.vmDeleted = true;
            }
            else {
                return done(err);
            }
        }
        if (instanceView &&
            instanceView.statuses[0].code === 'ProvisioningState/deleting')
            bag.deletingVM = true;
        return done();
    });
}
function __deleteVM(bag, done) {
    if (!bag.vmName)
        return done();
    if (bag.deletingVM)
        return done();
    if (bag.vmDeleted)
        return done();
    var who = bag.who + '|' + __deleteVM.name;
    logger.trace(who, 'Inside');
    bag.azAdapter.AZCompute.deleteVM(bag.resourceGroupName, bag.vmName, function (err) {
        if (err) {
            logger.warn(who, 'Failed to delete vm for node: ' + bag.nodeId, err);
        }
        else {
            bag.deletingVM = true;
            bag.consoleAdapter.openGrp('Deleting node: ' + bag.nodeId);
            bag.consoleAdapter.openCmd('Started deletion of vm: ' + bag.vmName);
            bag.consoleAdapter.publishMsg('This operation involves deleting the ' +
                'VM & the NIC associated with it. This may take a while...');
            bag.consoleAdapter.closeCmd(true);
        }
        return done(err);
    });
}
function __deleteDisks(bag, done) {
    if (bag.deletingVM)
        return done();
    if (_.isEmpty(bag.tags))
        return done();
    var who = bag.who + '|' + __deleteDisks.name;
    logger.trace(who, 'Inside');
    var innerBag = {
        who: who,
        nodeId: bag.nodeId,
        azAdapter: bag.azAdapter,
        resourceGroupName: bag.resourceGroupName,
        tags: bag.tags,
        tagName: 'Node Id'
    };
    async.series([
        ___getResources.bind(null, innerBag),
        ___verifyDisks.bind(null, innerBag),
        ___deleteDisks.bind(null, innerBag)
    ], function (err) {
        return done(err);
    });
}
function ___getResources(bag, done) {
    var who = bag.who + '|' + ___getResources.name;
    logger.trace(who, 'Inside');
    bag.azAdapter.AZResource.listResourcesByResourceGroupAndTag(bag.resourceGroupName, bag.tagName, bag.tags[bag.tagName], function (err, resources) {
        if (err) {
            logger.warn(who, 'Failed to list resources of resource group:' +
                bag.resourceGroupName + ' with tag ' + bag.tagName + ': ' +
                bag.tags[bag.tagName] + ' vm for node: ' + bag.nodeId, err);
            return done(err);
        }
        bag.diskNames = _.pluck(_.filter(resources, { type: 'Microsoft.Compute/disks' }), 'name');
        return done();
    });
}
function ___verifyDisks(bag, done) {
    if (_.isEmpty(bag.diskNames))
        return done();
    var who = bag.who + '|' + ___verifyDisks.name;
    logger.trace(who, 'Inside');
    var disks = [];
    async.eachLimit(bag.diskNames, 5, function (diskName, nextDisk) {
        bag.azAdapter.AZCompute.getDisk(bag.resourceGroupName, diskName, function (err, disk) {
            if (err) {
                logger.warn(who, 'Failed to get disk for diskName: ' + diskName +
                    ' vm for node: ' + bag.nodeId, err);
                return nextDisk(err);
            }
            disks.push(disk);
            return nextDisk();
        });
    }, function (err) {
        if (err)
            return done(err);
        bag.diskNames = _.pluck(_.filter(disks, function (disk) {
            return _.isEqual(disk.tags, bag.tags);
        }), 'name');
        return done();
    });
}
function ___deleteDisks(bag, done) {
    if (_.isEmpty(bag.diskNames))
        return done();
    var who = bag.who + '|' + ___deleteDisks.name;
    logger.trace(who, 'Inside');
    async.eachLimit(bag.diskNames, 5, function (diskName, nextDisk) {
        bag.azAdapter.AZCompute.deleteDisk(bag.resourceGroupName, diskName, function (err) {
            if (err)
                logger.warn(who, 'Failed to get delete disk with diskName: ' +
                    diskName + ' vm for node: ' + bag.nodeId, err);
            return nextDisk(err);
        });
    }, function (err) {
        return done(err);
    });
}
function __getNIC(bag, done) {
    if (!bag.nicName)
        return done();
    if (bag.deletingVM)
        return done();
    var who = bag.who + '|' + __getNIC.name;
    logger.trace(who, 'Inside');
    bag.azAdapter.AZNetwork.getNIC(bag.resourceGroupName, bag.nicName, function (err) {
        if (err && err.code === 'ResourceNotFound') {
            bag.nicDeleted = true;
            return done();
        }
        return done(err);
    });
}
function __deleteNIC(bag, done) {
    if (!bag.nicName)
        return done();
    if (bag.deletingVM)
        return done();
    if (bag.nicDeleted)
        return done();
    var who = bag.who + '|' + __deleteNIC.name;
    logger.trace(who, 'Inside');
    bag.azAdapter.AZNetwork.deleteNIC(bag.resourceGroupName, bag.nicName, function (err) {
        if (err)
            logger.warn(who, 'Failed to delete NIC for node: ' + bag.nodeId, err);
        else
            bag.nicDeleted = true;
        return done(err);
    });
}
//# sourceMappingURL=handler.js.map