'use strict';
var self = handler;
module.exports = self;
var fs = require('fs');
var path = require('path');
var AWSAdapter = require('../interface/aws/Adapter.js');
var ApiAdapter = require('../../Adapter.js');
var nodeActions = require('../../nodeActions.js');
var nodeMetadataMap = require('../../nodeMetadataMap.js');
function handler(params, callback) {
    var bag = {
        action: params.action,
        nodeName: params.nodeName,
        projectIntegrationId: params.projectIntegrationId,
        nodeId: params.nodeId,
        nodeStatus: params.nodeStatus,
        operatingSystem: params.operatingSystem,
        instanceId: params.instanceId,
        nodeSSHUser: params.nodeSSHUser,
        architecture: params.architecture,
        startupScriptData: params.startupScriptData || {},
        instanceSize: params.providerMetadata &&
            params.providerMetadata.instanceSize,
        accessKeyId: params.accessKeyId,
        secretAccessKey: params.secretAccessKey,
        region: params.region,
        securityGroupId: params.providerMetadata &&
            params.providerMetadata.securityGroupId || null,
        subnetId: params.providerMetadata && params.providerMetadata.subnetId ||
            null,
        keyPairName: params.providerMetadata && params.providerMetadata.keyPairName,
        clientToken: params.nodeProviderMetadata &&
            params.nodeProviderMetadata.uniqueToken,
        metadataOperatingSystem: params.nodeProviderMetadata &&
            params.nodeProviderMetadata.operatingSystem,
        imageId: params.imageId,
        consoleAdapter: params.consoleAdapter,
        projectId: params.projectId,
        projectName: params.projectName,
        nodePoolId: params.nodePoolId,
        nodePoolName: params.nodePoolName,
        isSuccess: true,
        instanceData: '',
        instanceTerminated: false,
        skipConsolePost: false,
        jFrogPipelinesSessionId: params.jFrogPipelinesSessionId,
        diskSizeinGB: params.diskSizeinGB,
        customTags: params.providerMetadata?.customTags || []
    };
    bag.who = util.format('ec2|%s|jFrogPipelinesSessionId:%s', self.name, bag.jFrogPipelinesSessionId);
    logger.debug(bag.who, 'Inside');
    if (_.has(bag.startupScriptData, 'insecureDockerRegistries')) {
        bag.startupScriptData.insecureDockerRegistries =
            JSON.stringify(bag.startupScriptData.insecureDockerRegistries);
    }
    else {
        bag.startupScriptData.insecureDockerRegistries = JSON.stringify([]);
    }
    async.series([
        _checkInputParams.bind(null, bag),
        _initializeAWSAdapter.bind(null, bag),
        _getAvailabilityZone.bind(null, bag),
        _allocateHost.bind(null, bag),
        _describeHost.bind(null, bag),
        _loadTemplateFile.bind(null, bag),
        _renderTemplate.bind(null, bag),
        _encodeInstanceData.bind(null, bag),
        _initializeInstance.bind(null, bag),
        _cacheInstance.bind(null, bag),
        _getInstanceStatus.bind(null, bag),
        _startInstance.bind(null, bag),
        _deleteInstance.bind(null, bag),
        _deallocateHost.bind(null, bag)
    ], function (err) {
        var result = {};
        if (bag.consoleAdapter)
            bag.consoleAdapter.closeGrp(bag.isSuccess);
        if (err) {
            logger.warn(bag.who, 'reported an error', err);
        }
        else {
            result = {
                instance: bag.instance,
                instanceTerminated: bag.instanceTerminated,
                host: bag.dedicatedHost,
                isSuccess: bag.isSuccess
            };
            logger.debug(bag.who, 'Successfully completed processing of ec2');
        }
        result.errorForMetric = bag.errorForMetric;
        return callback(err, result);
    });
}
function _checkInputParams(bag, next) {
    var who = bag.who + '|' + _checkInputParams.name;
    logger.trace(who, 'Inside');
    bag.suAdapter = new ApiAdapter(bag.jFrogPipelinesSessionId, global.config.apiToken);
    var consoleErrors = [];
    if (!bag.action)
        consoleErrors.push(util.format('%s is missing: action', who));
    if (bag.action === nodeActions.DELETE)
        bag.skipConsolePost = true;
    if (!bag.nodeId)
        consoleErrors.push(util.format('%s is missing: nodeId', who));
    if (_.isEmpty(bag.accessKeyId))
        consoleErrors.push(util.format('%s is missing: accessKeyId', who));
    if (_.isEmpty(bag.secretAccessKey))
        consoleErrors.push(util.format('%s is missing: secretAccessKey', who));
    if (_.isEmpty(bag.region))
        consoleErrors.push(util.format('%s is missing: region', who));
    if (!bag.consoleAdapter)
        consoleErrors.push(util.format('%s is missing: consoleAdapter', who));
    if (bag.action === nodeActions.CREATE) {
        if (_.isEmpty(bag.nodeName))
            consoleErrors.push(util.format('%s is missing: nodeName', who));
        if (_.isEmpty(bag.keyPairName))
            consoleErrors.push(util.format('%s is missing: keyPairName', who));
        if (_.isEmpty(bag.imageId))
            consoleErrors.push(util.format('%s is missing: imageId', who));
        if (_.isEmpty(bag.startupScriptData))
            consoleErrors.push(util.format('%s is missing: startupScriptData', who));
    }
    if (consoleErrors.length > 0) {
        _.each(consoleErrors, function (err) {
            logger.error(bag.who, err);
        });
        return next(true);
    }
    return next();
}
function _initializeAWSAdapter(bag, next) {
    var who = bag.who + '|' + _initializeAWSAdapter.name;
    logger.trace(who, 'Inside');
    bag.awsAdapter = new AWSAdapter(bag.accessKeyId, bag.secretAccessKey, bag.region);
    if (!bag.skipConsolePost) {
        var action;
        if (bag.action === nodeActions.CREATE)
            action = 'Creating';
        else if (bag.action === nodeActions.START)
            action = 'Starting';
        else if (bag.action === nodeActions.CACHE)
            action = 'Stopping';
        bag.consoleAdapter.openGrp(util.format('%s node: %s', action, bag.nodeId));
        bag.consoleAdapter.openCmd('Initializing AWS adapter');
    }
    bag.awsAdapter.initialize(function (err) {
        if (err) {
            if (!bag.skipConsolePost) {
                bag.consoleAdapter.publishMsg('Failed to initialize aws adapter');
                bag.consoleAdapter.closeCmd(false);
            }
            logger.warn(who, util.format('Failed to initialize AWS Adapter for Node: %s ', bag.nodeId), err);
            bag.isSuccess = false;
            return next();
        }
        if (!bag.skipConsolePost) {
            bag.consoleAdapter.publishMsg('Successfully initialized aws adapter');
            bag.consoleAdapter.closeCmd(true);
        }
        return next();
    });
}
function _getAvailabilityZone(bag, next) {
    if (!bag.isSuccess)
        return next();
    if (bag.metadataOperatingSystem !== 'MacOS_10.15')
        return next();
    if (bag.action !== nodeActions.CREATE)
        return next();
    if (bag.nodeStatus !== global.statusCodes.WAITING)
        return next();
    var who = bag.who + '|' + _getAvailabilityZone.name;
    logger.trace(who, 'Inside');
    bag.consoleAdapter.openCmd('Finding subnet');
    bag.awsAdapter.EC2.describeSubnets([bag.subnetId], function (err, subnets) {
        if (err) {
            bag.errorForMetric = err.code;
            bag.consoleAdapter.publishMsg('Failed to find subnet: ' + bag.subnetId);
            bag.consoleAdapter.closeCmd(false);
            logger.warn(who, util.format('Failed to find subnet for nodeId: %s', bag.nodeId), err);
            bag.isSuccess = false;
            return next();
        }
        bag.subnet = subnets[0];
        bag.consoleAdapter.publishMsg('Subnet found');
        bag.consoleAdapter.closeCmd(true);
        return next();
    });
}
function _allocateHost(bag, next) {
    if (!bag.isSuccess)
        return next();
    if (bag.metadataOperatingSystem !== 'MacOS_10.15')
        return next();
    if (bag.action !== nodeActions.CREATE)
        return next();
    if (bag.nodeStatus !== global.statusCodes.WAITING)
        return next();
    var who = bag.who + '|' + _allocateHost.name;
    logger.trace(who, 'Inside');
    bag.consoleAdapter.openCmd('Allocating host');
    var hostTags = [
        {
            Key: 'Project Id',
            Value: bag.projectId.toString()
        },
        {
            Key: 'Node Pool Id',
            Value: bag.nodePoolId.toString()
        },
        {
            Key: 'Node Name',
            Value: bag.nodeName
        },
        {
            Key: 'Node Id',
            Value: bag.nodeId.toString()
        }
    ];
    if (global.config.isSaas) {
        var saasTags = [
            {
                Key: 'Customer Name',
                Value: global.config.saasCustomerName
            },
            {
                Key: 'SaaS Environment',
                Value: global.config.saasEnvironment
            }
        ];
        hostTags = hostTags.concat(saasTags);
    }
    var tagSpecifications = [
        {
            ResourceType: 'dedicated-host',
            Tags: hostTags
        }
    ];
    bag.awsAdapter.EC2.allocateDedicatedHost(bag.subnet.AvailabilityZone, bag.instanceSize, bag.clientToken, tagSpecifications, function (err, hostId) {
        if (err) {
            bag.errorForMetric = err.code;
            bag.consoleAdapter.publishMsg('Failed to allocate host with error: ' +
                err.message);
            bag.consoleAdapter.closeCmd(false);
            logger.warn(who, util.format('Failed to allocate host for node: %s', bag.nodeId), err);
            bag.isSuccess = false;
            return next();
        }
        bag.hostId = hostId;
        bag.consoleAdapter.publishMsg('Host allocation requested');
        bag.consoleAdapter.publishMsg(util.inspect(hostId));
        bag.consoleAdapter.closeCmd(true);
        return next();
    });
}
function _describeHost(bag, next) {
    if (!bag.isSuccess)
        return next();
    if (bag.metadataOperatingSystem !== 'MacOS_10.15')
        return next();
    if (bag.action !== nodeActions.CREATE && bag.action !== nodeActions.DELETE)
        return next();
    var who = bag.who + '|' + _describeHost.name;
    logger.trace(who, 'Inside');
    if (!bag.skipConsolePost)
        bag.consoleAdapter.openCmd('Checking dedicated host');
    var filters = [
        {
            Name: 'client-token',
            Values: [bag.clientToken]
        }
    ];
    bag.awsAdapter.EC2.describeHosts(filters, null, null, null, function (err, hosts) {
        if (err) {
            bag.errorForMetric = err.code;
            if (!bag.skipConsolePost) {
                bag.consoleAdapter.publishMsg('Failed to find host with error: ' +
                    err.message);
                bag.consoleAdapter.closeCmd(false);
            }
            logger.warn(who, util.format('Failed to find allocated host for node: %s', bag.nodeId), err);
            bag.isSuccess = false;
            return next();
        }
        if (!_.isEmpty(hosts)) {
            bag.dedicatedHost = _.first(hosts);
            if (!bag.skipConsolePost) {
                bag.consoleAdapter.publishMsg('Host state: ' +
                    bag.dedicatedHost.State);
                bag.consoleAdapter.closeCmd(true);
            }
        }
        else if (bag.hostId) {
            bag.dedicatedHost = {
                HostId: bag.hostId
            };
            if (!bag.skipConsolePost)
                bag.consoleAdapter.closeCmd(true);
        }
        else if (bag.action !== nodeActions.DELETE) {
            if (!bag.skipConsolePost) {
                bag.consoleAdapter.publishMsg('Host not found');
                bag.consoleAdapter.closeCmd(false);
            }
            bag.isSuccess = false;
        }
        return next();
    });
}
function _loadTemplateFile(bag, next) {
    if (!bag.isSuccess)
        return next();
    if (bag.action !== nodeActions.CREATE)
        return next();
    if (bag.dedicatedHost && bag.dedicatedHost.State !== 'available')
        return next();
    var who = bag.who + '|' + _loadTemplateFile.name;
    logger.trace(who, 'Inside');
    var templateFile = util.format('templates/%s/%s/boot.%s', bag.architecture, bag.operatingSystem, nodeMetadataMap.getScriptFileExtension(bag.architecture, bag.operatingSystem));
    var templatePath = path.join(__dirname, templateFile);
    var templateString = fs.readFileSync(templatePath).toString();
    bag.template = _.template(templateString);
    return next();
}
function _renderTemplate(bag, next) {
    if (!bag.isSuccess)
        return next();
    if (bag.action !== nodeActions.CREATE)
        return next();
    if (bag.dedicatedHost && bag.dedicatedHost.State !== 'available')
        return next();
    var who = bag.who + '|' + _renderTemplate.name;
    logger.trace(who, 'Inside');
    bag.instanceData = bag.template(bag.startupScriptData);
    return next();
}
function _encodeInstanceData(bag, next) {
    if (!bag.isSuccess)
        return next();
    if (bag.action !== nodeActions.CREATE)
        return next();
    if (bag.dedicatedHost && bag.dedicatedHost.State !== 'available')
        return next();
    var who = bag.who + '|' + _encodeInstanceData.name;
    logger.trace(who, 'Inside');
    bag.consoleAdapter.openCmd('Generating intitialize script');
    bag.instanceData = Buffer.from(bag.instanceData).toString('base64');
    bag.consoleAdapter.publishMsg('Successfully generated intitialize script');
    bag.consoleAdapter.closeCmd(true);
    return next();
}
function _initializeInstance(bag, next) {
    if (!bag.isSuccess)
        return next();
    if (bag.action !== nodeActions.CREATE)
        return next();
    if (bag.dedicatedHost && bag.dedicatedHost.State !== 'available')
        return next();
    var who = bag.who + '|' + _initializeInstance.name;
    logger.trace(who, 'Inside');
    var subnetId = bag.subnetId;
    bag.consoleAdapter.openCmd('Creating instance');
    var instanceTags = [
        {
            Key: 'Project Id',
            Value: bag.projectId.toString()
        },
        {
            Key: 'Project',
            Value: bag.projectName
        },
        {
            Key: 'Node Pool Id',
            Value: bag.nodePoolId.toString()
        },
        {
            Key: 'NodePool',
            Value: bag.nodePoolName
        },
        {
            Key: 'Node Name',
            Value: bag.nodeName
        },
        {
            Key: 'Node Id',
            Value: bag.nodeId.toString()
        }
    ];
    var volumeTags = [];
    if (global.config.isSaas) {
        var saasTags = [
            {
                Key: 'Customer Name',
                Value: global.config.saasCustomerName
            },
            {
                Key: 'SaaS Environment',
                Value: global.config.saasEnvironment
            },
            {
                Key: 'Customer',
                Value: global.config.saasCustomerName
            },
        ];
        instanceTags = instanceTags.concat(saasTags);
        volumeTags = saasTags;
    }
    if (!_.isEmpty(bag.customTags) && Array.isArray(bag.customTags)) {
        instanceTags = instanceTags.concat(bag.customTags);
    }
    var tagSpecifications = [
        {
            ResourceType: 'instance',
            Tags: instanceTags
        }
    ];
    if (global.config.isSaas)
        tagSpecifications.push({
            ResourceType: 'volume',
            Tags: volumeTags
        });
    var hostId = (bag.dedicatedHost && bag.dedicatedHost.HostId) || null;
    var blockDeviceMappings = [];
    if (bag.diskSizeinGB) {
        blockDeviceMappings.push({
            DeviceName: '/dev/sda1',
            Ebs: {
                VolumeSize: bag.diskSizeinGB,
                VolumeType: 'gp2'
            }
        });
    }
    var runParams = {
        amiId: bag.imageId,
        subnetId: subnetId,
        instanceName: bag.nodeName,
        instanceSize: bag.instanceSize,
        securityGroupId: bag.securityGroupId,
        keyName: bag.keyPairName,
        userData: bag.instanceData,
        tagSpecifications: tagSpecifications,
        hostId: hostId,
        clientToken: bag.clientToken,
        blockDeviceMappings: blockDeviceMappings,
    };
    bag.awsAdapter.EC2.runInstance(runParams, function (err, instance) {
        if (err) {
            bag.errorForMetric = err.code;
            bag.consoleAdapter.publishMsg('Failed to create instance with error: ' +
                err.message);
            bag.consoleAdapter.closeCmd(false);
            logger.warn(who, util.format('Failed to run instance for node: %s with ami: ' +
                '%s', bag.nodeId, bag.imageId), err);
            bag.isSuccess = false;
            return next();
        }
        bag.instance = instance;
        bag.consoleAdapter.publishMsg('Instance requested');
        bag.consoleAdapter.publishMsg(util.inspect(instance));
        bag.consoleAdapter.closeCmd(true);
        return next();
    });
}
function _cacheInstance(bag, next) {
    if (!bag.isSuccess)
        return next();
    if (bag.action !== nodeActions.CACHE)
        return next();
    var who = bag.who + '|' + _cacheInstance.name;
    logger.trace(who, 'Inside');
    bag.consoleAdapter.openCmd('Stopping instance');
    bag.awsAdapter.EC2.stopInstance(bag.instanceId, function (err, response) {
        if (err) {
            bag.errorForMetric = err.code;
            bag.consoleAdapter.publishMsg('Failed to stop instance with error: ' +
                err.message);
            bag.consoleAdapter.closeCmd(false);
            logger.warn(who, util.format('Failed to stop instance for node: %s', bag.nodeId), err);
            bag.isSuccess = false;
            return next();
        }
        bag.consoleAdapter.publishMsg('Instance stop requested');
        bag.consoleAdapter.publishMsg(util.inspect(response));
        bag.consoleAdapter.closeCmd(true);
        return next();
    });
}
function _getInstanceStatus(bag, next) {
    if (!bag.isSuccess)
        return next();
    if (bag.action !== nodeActions.START)
        return next();
    logger.info(util.inspect(bag.instanceData));
    var who = bag.who + '|' + _getInstanceStatus.name;
    logger.trace(who, 'Inside');
    var retryOpts = {
        times: 10,
        interval: 3000
    };
    async.retry(retryOpts, function (callback) {
        bag.awsAdapter.EC2.describeInstanceStatus(bag.instanceId, function (err, instanceStatus) {
            var canStartNode = false;
            if (err) {
                bag.errorForMetric = err.code;
                logger.warn(who, util.format('Failed to get instance for node: %s' +
                    ' and instance id: %s', bag.nodeId, bag.instanceId), err.message);
                return callback(null, canStartNode);
            }
            logger.trace(who, util.format('Successfully described instance status of ' +
                'node: %s instance id: %s \n %s', bag.nodeId, bag.instanceId, JSON.stringify(instanceStatus)));
            if (instanceStatus && instanceStatus.Name === 'stopping')
                return callback(true, canStartNode);
            canStartNode = true;
            return callback(null, canStartNode);
        });
    }, function (err, canStartNode) {
        if (err || !canStartNode) {
            bag.consoleAdapter.openCmd('Getting instance status');
            let errMsg = 'Failed to get instance status';
            if (err && err.message) {
                errMsg = `${errMsg}: ${err.message}`;
            }
            bag.consoleAdapter.publishMsg(errMsg);
            bag.consoleAdapter.closeCmd(false);
            logger.warn(who, util.format('Retry failed to get instance status of node: %s and ' +
                'instance id: %s', bag.nodeId, bag.instanceId), err);
        }
        bag.isSuccess = canStartNode;
        return next();
    });
}
function _startInstance(bag, next) {
    if (!bag.isSuccess)
        return next();
    if (bag.action !== nodeActions.START)
        return next();
    var who = bag.who + '|' + _startInstance.name;
    logger.trace(who, 'Inside');
    bag.consoleAdapter.openCmd('Starting instance');
    bag.awsAdapter.EC2.startInstance(bag.instanceId, function (err, response) {
        if (err) {
            bag.errorForMetric = err.code;
            bag.consoleAdapter.publishMsg('Failed to start instance with error:' +
                err.message);
            bag.consoleAdapter.closeCmd(false);
            logger.warn(who, util.format('Failed to start instance for node: %s', bag.nodeId), err);
            bag.isSuccess = false;
            return next();
        }
        bag.consoleAdapter.publishMsg('Instance start requested');
        bag.consoleAdapter.publishMsg(util.inspect(response));
        bag.consoleAdapter.closeCmd(true);
        return next();
    });
}
function _deleteInstance(bag, next) {
    if (!bag.isSuccess)
        return next();
    if (bag.action !== nodeActions.DELETE)
        return next();
    if (_.isEmpty(bag.instanceId)) {
        logger.debug(util.format('Skipping termination of ec2 instance for nodeId:%s ' +
            'as instanceId is empty', bag.nodeId));
        return next();
    }
    var who = bag.who + '|' + _deleteInstance.name;
    logger.trace(who, 'Inside');
    bag.awsAdapter.EC2.terminateInstance(bag.instanceId, function (err) {
        if (err) {
            bag.errorForMetric = err.code;
            if (err.name === 'InvalidInstanceID.NotFound') {
                bag.instanceTerminated = true;
                logger.warn(who, util.format('Instance id: %s not found for node: %s, but ' +
                    'continuing to delete from pipelines', bag.instanceId, bag.nodeId), err);
                return next();
            }
            logger.warn(who, util.format('Failed to terminate instance id: %s for ' +
                'node: %s', bag.instanceId, bag.nodeId), err);
            bag.isSuccess = false;
            return next();
        }
        bag.instanceTerminated = true;
        logger.trace(who, util.format('Successfully terminated AWS instance id: %s for node: %s', bag.instanceId, bag.nodeId));
        return next();
    });
}
function _deallocateHost(bag, next) {
    if (!bag.isSuccess)
        return next();
    if (bag.action !== nodeActions.DELETE)
        return next();
    if (_.isEmpty(bag.dedicatedHost))
        return next();
    var who = bag.who + '|' + _deallocateHost.name;
    logger.trace(who, 'Inside');
    bag.awsAdapter.EC2.releaseHost(bag.dedicatedHost.HostId, function (err, data) {
        if (err) {
            bag.errorForMetric = err.code;
            if (_.contains(['InvalidHostId', 'InvalidHostID.NotFound',
                'InvalidHostId.NotFound'], err.name)) {
                logger.warn(who, util.format('Host id: %s not found for node: %s, but ' +
                    'continuing to delete from pipelines', bag.dedicatedHost.HostId, bag.nodeId), err);
                return next();
            }
            logger.warn(who, util.format('Failed to release dedicated host hostId: %s for ' +
                'node: %s', bag.dedicatedHost.HostId, bag.nodeId), err);
            bag.isSuccess = false;
            return next();
        }
        if (data && !_.isEmpty(data.Unsuccessful)) {
            logger.info(who, util.format('AWS Dedicated host: %s not yet released for node: %s with: %s', bag.dedicatedHost.HostId, bag.nodeId, util.inspect(_.first(data.Unsuccessful).Error)));
            bag.isSuccess = false;
            return next();
        }
        logger.trace(who, util.format('Successfully released AWS dedicated host: %s for node: %s', bag.dedicatedHost.HostId, bag.nodeId));
        return next();
    });
}
//# sourceMappingURL=handler.js.map