'use strict';
var self = handler;
module.exports = self;
var fs = require('fs');
var path = require('path');
var GCEAdapter = require('../interface/gce/Adapter.js');
var ApiAdapter = require('../../Adapter.js');
var nodeActions = require('../../nodeActions.js');
var nodeMetadataMap = require('../../nodeMetadataMap.js');
function handler(params, callback) {
    var bag = {
        action: params.action,
        nodeName: params.nodeName,
        projectIntegrationId: params.projectIntegrationId,
        nodeId: params.nodeId,
        instanceId: params.instanceId,
        operatingSystem: params.operatingSystem,
        architecture: params.architecture,
        startupScriptData: params.startupScriptData || {},
        instanceSize: params.providerMetadata &&
            params.providerMetadata.instanceSize,
        jsonKey: params.jsonKey,
        zone: (params.providerMetadata &&
            params.providerMetadata.zone) || (params.nodeProviderMetadata &&
            params.nodeProviderMetadata.zone),
        assignPublicIp: params.providerMetadata &&
            params.providerMetadata.assignPublicIp,
        subnetName: params.providerMetadata &&
            params.providerMetadata.subnetName,
        networkTags: params.providerMetadata &&
            params.providerMetadata.networkTags,
        imageName: params.buildPlaneImage &&
            params.buildPlaneImage.metadataPropertyBag &&
            params.buildPlaneImage.metadataPropertyBag.imageName,
        projectName: params.buildPlaneImage &&
            params.buildPlaneImage.metadataPropertyBag &&
            params.buildPlaneImage.metadataPropertyBag.projectName,
        region: params.buildPlaneImage &&
            params.buildPlaneImage.region,
        consoleAdapter: params.consoleAdapter,
        projectId: params.projectId,
        nodePoolId: params.nodePoolId,
        isSuccess: true,
        instanceData: '',
        skipConsolePost: false,
        jFrogPipelinesSessionId: params.jFrogPipelinesSessionId,
        diskSizeinGB: params.diskSizeinGB
    };
    bag.who = util.format('gce|%s|jFrogPipelinesSessionId:%s', self.name, bag.jFrogPipelinesSessionId);
    logger.debug(bag.who, 'Inside');
    if (_.has(bag.startupScriptData, 'insecureDockerRegistries')) {
        bag.startupScriptData.insecureDockerRegistries =
            JSON.stringify(bag.startupScriptData.insecureDockerRegistries);
    }
    else {
        bag.startupScriptData.insecureDockerRegistries = JSON.stringify([]);
    }
    async.series([
        _checkInputParams.bind(null, bag),
        _initializeGCEAdapter.bind(null, bag),
        _loadTemplateFile.bind(null, bag),
        _renderTemplate.bind(null, bag),
        _initializeInstance.bind(null, bag),
        _cacheInstance.bind(null, bag),
        _checkStoppedBeforeStarting.bind(null, bag),
        _setMetadata.bind(null, bag),
        _startInstance.bind(null, bag),
        _deleteInstance.bind(null, bag)
    ], function (err) {
        var result = {};
        if (bag.consoleAdapter)
            bag.consoleAdapter.closeGrp(bag.isSuccess);
        if (err) {
            logger.warn(bag.who, 'reported an error', err);
        }
        else {
            result = {
                instance: bag.instance,
                isSuccess: bag.isSuccess
            };
            logger.debug(bag.who, 'Successfully completed processing of gce');
        }
        result.errorForMetric = bag.errorForMetric;
        return callback(err, result);
    });
}
function _checkInputParams(bag, next) {
    var who = bag.who + '|' + _checkInputParams.name;
    logger.trace(who, 'Inside');
    bag.suAdapter = new ApiAdapter(bag.jFrogPipelinesSessionId, global.config.apiToken);
    var consoleErrors = [];
    var expectedParams = [
        'action',
        'nodeId',
        'jsonKey',
        'zone'
    ];
    if (bag.action === nodeActions.CREATE)
        expectedParams = expectedParams.concat([
            'nodeName',
            'imageName',
            'projectName',
            'instanceSize',
            'startupScriptData'
        ]);
    _.each(expectedParams, function (expectedParam) {
        if (_.isNull(bag[expectedParam]) || _.isUndefined(bag[expectedParam]))
            consoleErrors.push(util.format('%s is missing %s', who, expectedParam));
    });
    if (bag.action === nodeActions.DELETE)
        bag.skipConsolePost = true;
    if (consoleErrors.length > 0) {
        _.each(consoleErrors, function (err) {
            logger.error(bag.who, err);
        });
        return next(true);
    }
    return next();
}
function _initializeGCEAdapter(bag, next) {
    var who = bag.who + '|' + _initializeGCEAdapter.name;
    logger.trace(who, 'Inside');
    var jsonKey = bag.jsonKey;
    try {
        jsonKey = JSON.parse(jsonKey);
    }
    catch (err) {
        return next(util.format('%s: Failed to parse jsonKey: %s', who, err));
    }
    var config = {
        credentials: jsonKey,
        zone: bag.zone
    };
    bag.gceAdapter = new GCEAdapter(config);
    if (!bag.skipConsolePost) {
        var action;
        if (bag.action === nodeActions.CREATE)
            action = 'Creating';
        else if (bag.action === nodeActions.START)
            action = 'Starting';
        else if (bag.action === nodeActions.CACHE)
            action = 'Stopping';
        bag.consoleAdapter.openGrp(util.format('%s node: %s', action, bag.nodeId));
        bag.consoleAdapter.openCmd('Initializing GCE adapter');
        bag.consoleAdapter.closeCmd(true);
    }
    return next();
}
function _loadTemplateFile(bag, next) {
    if (!bag.isSuccess)
        return next();
    if (bag.action !== nodeActions.CREATE)
        return next();
    var who = bag.who + '|' + _loadTemplateFile.name;
    logger.trace(who, 'Inside');
    var templateFile = util.format('templates/%s/%s/boot.%s', bag.architecture, bag.operatingSystem, nodeMetadataMap.getScriptFileExtension(bag.architecture, bag.operatingSystem));
    var templatePath = path.join(__dirname, templateFile);
    var templateString = fs.readFileSync(templatePath).toString();
    bag.template = _.template(templateString);
    return next();
}
function _renderTemplate(bag, next) {
    if (!bag.isSuccess)
        return next();
    if (bag.action !== nodeActions.CREATE)
        return next();
    var who = bag.who + '|' + _renderTemplate.name;
    logger.trace(who, 'Inside');
    bag.instanceData = bag.template(bag.startupScriptData);
    return next();
}
function _initializeInstance(bag, next) {
    if (!bag.isSuccess)
        return next();
    if (bag.action !== nodeActions.CREATE)
        return next();
    var who = bag.who + '|' + _initializeInstance.name;
    logger.trace(who, 'Inside');
    var sourceImage = util.format('https://www.googleapis.com/compute/v1/projects/%s/global/images/%s', bag.projectName, bag.imageName);
    var config = {
        machineType: bag.instanceSize,
        disks: [
            {
                autoDelete: true,
                boot: true,
                initializeParams: {
                    sourceImage: sourceImage
                },
                diskSizeGb: bag.diskSizeinGB
            }
        ],
        metadata: {
            items: [{
                    value: bag.instanceData,
                    key: 'startup-script'
                }]
        },
        labels: {
            'project-id': bag.projectId,
            'nodepool-id': bag.nodePoolId,
            'nodepool-node-id': bag.nodeId,
            'node-name': bag.nodeName
        }
    };
    if (_.isBoolean(bag.assignPublicIp) && bag.assignPublicIp)
        config.networkInterfaces = [
            {
                accessConfigs: [
                    {
                        kind: 'compute#accessConfig'
                    }
                ]
            }
        ];
    if (bag.subnetName) {
        config.networkInterfaces = config.networkInterfaces || [];
        var subnetwork = util.format('regions/%s/subnetworks/%s', bag.region, bag.subnetName);
        var networkInterfaces;
        if (_.isEmpty(config.networkInterfaces))
            networkInterfaces = { 'subnetwork': subnetwork };
        else
            networkInterfaces = _.extend(_.first(config.networkInterfaces), { 'subnetwork': subnetwork });
        config.networkInterfaces = [networkInterfaces];
    }
    if (!_.isEmpty(bag.networkTags)) {
        config.tags = {
            items: bag.networkTags
        };
    }
    bag.consoleAdapter.openCmd('Creating instance');
    bag.gceAdapter.createInstance(bag.nodeName, config, function (err, instance) {
        if (err) {
            bag.errorForMetric = err.errors[0]?.reason;
            bag.consoleAdapter.publishMsg('Failed to create instance with error: ' + err.message);
            bag.consoleAdapter.closeCmd(false);
            logger.warn(who, util.format('Failed to run instance for node: %s with' +
                ' image %s', bag.nodeId, bag.imageId), err);
            bag.isSuccess = false;
            return next();
        }
        bag.instance = instance;
        bag.consoleAdapter.publishMsg('Instance requested');
        bag.consoleAdapter.publishMsg(util.inspect(instance));
        bag.consoleAdapter.closeCmd(true);
        return next();
    });
}
function _cacheInstance(bag, next) {
    if (!bag.isSuccess)
        return next();
    if (bag.action !== nodeActions.CACHE)
        return next();
    var who = bag.who + '|' + _cacheInstance.name;
    logger.trace(who, 'Inside');
    bag.consoleAdapter.openCmd('Stopping instance');
    bag.gceAdapter.stopInstance(bag.instanceId, function (err, response) {
        if (err) {
            bag.errorForMetric = err.errors[0]?.reason;
            bag.consoleAdapter.publishMsg('Failed to stop instance with error: ' + err.message);
            bag.consoleAdapter.closeCmd(false);
            logger.warn(who, util.format('Failed to stop instance for node: %s', bag.nodeId), err);
            bag.isSuccess = false;
            return next();
        }
        bag.consoleAdapter.publishMsg('Instance stop requested');
        bag.consoleAdapter.publishMsg(util.inspect(response));
        bag.consoleAdapter.closeCmd(true);
        return next();
    });
}
function _checkStoppedBeforeStarting(bag, next) {
    if (!bag.isSuccess)
        return next();
    if (bag.action !== nodeActions.START)
        return next();
    var who = bag.who + '|' + _checkStoppedBeforeStarting.name;
    logger.trace(who, 'Inside');
    var status = 'TERMINATED';
    var timeout = 300;
    bag.gceAdapter.waitForStatus(bag.instanceId, status, timeout, function (err, response) {
        if (err) {
            bag.errorForMetric = err.errors[0]?.reason;
            logger.warn(who, util.format('Failed to get instance status for node: %s' +
                ' with instance ID %s', bag.nodeId, bag.instanceId), err.message);
            let errMsg = 'Failed to get instance status';
            if (err && err.message) {
                errMsg = `${errMsg}: ${err.message}`;
            }
            bag.consoleAdapter.openCmd('Getting instance status');
            bag.consoleAdapter.publishMsg(errMsg);
            bag.consoleAdapter.closeCmd(false);
            bag.isSuccess = false;
            return next();
        }
        logger.debug(who, util.format('Successfully described instance status of ' +
            'node %s (instance ID %s): %s', bag.nodeId, bag.instanceId, response.status));
        return next();
    });
}
function _setMetadata(bag, next) {
    if (!bag.isSuccess)
        return next();
    if (bag.action !== nodeActions.START)
        return next();
    var who = bag.who + '|' + _setMetadata.name;
    logger.trace(who, 'Inside');
    var metadata = {
        'startup-script': '',
    };
    bag.gceAdapter.setMetadata(bag.instanceId, metadata, function (err) {
        if (err) {
            bag.errorForMetric = err.errors[0]?.reason;
            logger.warn(who, util.format('Failed to set metadata for node: %s' +
                ' with instance ID %s', bag.nodeId, bag.instanceId), err.message);
            let errMsg = 'Failed to get instance status';
            if (err && err.message) {
                errMsg = `${errMsg}: ${err.message}`;
            }
            bag.consoleAdapter.openCmd('Setting instance metadata');
            bag.consoleAdapter.publishMsg(errMsg);
            bag.consoleAdapter.closeCmd(false);
            bag.isSuccess = false;
        }
        return next();
    });
}
function _startInstance(bag, next) {
    if (!bag.isSuccess)
        return next();
    if (bag.action !== nodeActions.START)
        return next();
    var who = bag.who + '|' + _startInstance.name;
    logger.trace(who, 'Inside');
    bag.consoleAdapter.openCmd('Starting instance');
    bag.gceAdapter.startInstance(bag.instanceId, function (err, response) {
        if (err) {
            bag.errorForMetric = err.errors[0]?.reason;
            bag.consoleAdapter.publishMsg('Failed to start instance with error:' +
                err.message);
            bag.consoleAdapter.closeCmd(false);
            logger.warn(who, util.format('Failed to start instance for node: %s', bag.nodeId), err);
            bag.isSuccess = false;
            return next();
        }
        bag.consoleAdapter.publishMsg('Instance start requested');
        bag.consoleAdapter.publishMsg(util.inspect(response));
        bag.consoleAdapter.closeCmd(true);
        return next();
    });
}
function _deleteInstance(bag, next) {
    if (!bag.isSuccess)
        return next();
    if (bag.action !== nodeActions.DELETE)
        return next();
    if (_.isEmpty(bag.instanceId)) {
        logger.warn(util.format('Skipping termination of gce instance for nodeId:%s ' +
            'as instanceId is empty', bag.nodeId));
        return next();
    }
    var who = bag.who + '|' + _deleteInstance.name;
    logger.trace(who, 'Inside');
    bag.gceAdapter.deleteInstance(bag.instanceId, function (err, response) {
        if (err) {
            bag.errorForMetric = err.errors[0]?.reason;
            logger.warn(who, util.format('Failed to delete instance %s for node: %s', bag.instanceId, bag.nodeId), err);
            if (!response || !response.error || response.error.code !== 404)
                bag.isSuccess = false;
        }
        return next();
    });
}
//# sourceMappingURL=handler.js.map