'use strict';
var self = Adapter;
module.exports = self;
var AWS = require('aws-sdk');
var proxy = require('proxy-agent');
var async = require('async');
var _ = require('underscore');
var { EXPONENTIAL_RETRY_OPTIONS } = require('../../../constants.js');
function wrapWithRetry(ctx, func) {
    const options = {
        ...EXPONENTIAL_RETRY_OPTIONS,
        errorFilter: function (err) {
            return err.code === 'RequestLimitExceeded';
        }
    };
    return async.retryable(options, func.bind(ctx));
}
var EC2 = {};
var IAM = {};
var ECS = {};
var ELB = {};
var S3 = {};
var STS = {};
var ApplicationAutoScaling = {};
var CloudWatch = {};
function Adapter(accessKeyId, secretAccessKey, region, sessionToken) {
    this.config = {};
    this.aws_access_key_id = accessKeyId;
    this.aws_secret_access_key = secretAccessKey;
    this.region = region || 'us-east-1';
    this.sessionToken = sessionToken;
    this.apiVersions = '2015-02-01';
    this.initialized = false;
    var proxyAddress = process.env.https_proxy || process.env.http_proxy;
    if (proxyAddress)
        AWS.config.update({
            httpOptions: { agent: proxy(proxyAddress) }
        });
}
Adapter.prototype.initialize = function (callback) {
    var self = this;
    self.config = {
        accessKeyId: self.aws_access_key_id,
        secretAccessKey: self.aws_secret_access_key,
        region: self.region,
        sessionToken: self.sessionToken,
        apiVersions: self.apiVersions,
        httpOptions: {
            timeout: 5000
        }
    };
    self.ec2Config = {
        accessKeyId: self.aws_access_key_id,
        secretAccessKey: self.aws_secret_access_key,
        region: self.region,
        sessionToken: self.sessionToken,
        apiVersions: self.apiVersions,
        httpOptions: {
            timeout: 120000
        },
        maxRetries: 2
    };
    self.ecsConfig = {
        accessKeyId: self.aws_access_key_id,
        secretAccessKey: self.aws_secret_access_key,
        region: self.region,
        sessionToken: self.sessionToken,
        apiVersions: self.apiVersions,
        httpOptions: {
            timeout: 5000
        },
        retryDelayOptions: {
            base: 1000
        },
        maxRetries: 6
    };
    self.s3Config = {
        accessKeyId: self.aws_access_key_id,
        secretAccessKey: self.aws_secret_access_key,
        region: self.region,
        apiVersions: self.apiVersions,
        sessionToken: self.sessionToken,
        httpOptions: {
            timeout: 5000
        },
        signatureVersion: 'v4'
    };
    self.ec2 = new AWS.EC2(self.ec2Config);
    self.iam = new AWS.IAM(self.config);
    self.ecs = new AWS.ECS(self.ecsConfig);
    self.elb = new AWS.ELB(self.config);
    self.s3 = new AWS.S3(self.s3Config);
    self.sts = new AWS.STS(self.config);
    self.applicationautoscaling = new AWS.ApplicationAutoScaling(self.ec2Config);
    self.cloudwatch = new AWS.CloudWatch(self.ec2Config);
    self.EC2 = bindSubAdapter(self, EC2);
    self.IAM = bindSubAdapter(self, IAM);
    self.ECS = bindSubAdapter(self, ECS);
    self.ELB = bindSubAdapter(self, ELB);
    self.S3 = bindSubAdapter(self, S3);
    self.STS = bindSubAdapter(self, STS);
    self.ApplicationAutoScaling = bindSubAdapter(self, ApplicationAutoScaling);
    self.CloudWatch = bindSubAdapter(self, CloudWatch);
    self.initialized = true;
    return callback();
};
function bindSubAdapter(newThis, subAdapter) {
    var boundAdapter = {};
    _.each(Object.keys(subAdapter), function (item) {
        boundAdapter[item] = subAdapter[item].bind(newThis);
    });
    return boundAdapter;
}
Adapter.prototype.isValidRole = function (rolePolicy) {
    var isValidRole = false;
    var uriDecodedRole = '';
    var ecsRoleValue = 'ecs.amazonaws.com';
    try {
        uriDecodedRole = decodeURIComponent(rolePolicy);
        uriDecodedRole = uriDecodedRole.replace('\n', '');
        uriDecodedRole = uriDecodedRole.replace('\'', '');
    }
    catch (err) {
        logger.error(util.inspect(err));
        return isValidRole;
    }
    try {
        var role = JSON.parse(uriDecodedRole);
        var isECSRoleAvailable = false;
        var roleStatement = role.Statement;
        _.each(roleStatement, function (statement) {
            if (_.has(statement, 'Principal')) {
                if (_.has(statement.Principal, 'Service')) {
                    if (_.isString(statement.Principal.Service)) {
                        if (statement.Principal.Service === ecsRoleValue)
                            isECSRoleAvailable = true;
                    }
                    else if (_.isArray(statement.Principal.Service)) {
                        if (_.contains(statement.Principal.Service, ecsRoleValue))
                            isECSRoleAvailable = true;
                    }
                }
            }
        });
        isValidRole = isECSRoleAvailable;
    }
    catch (err) {
        logger.error('Cannot parse JSON from decoded role URI: ' +
            util.inspect(err));
        return isValidRole;
    }
    return isValidRole;
};
IAM.getRoles = function (callback) {
    var self = this;
    var roleList = [];
    listRoles();
    var listRolesWithRetry = wrapWithRetry(self.iam, self.iam.listRoles);
    function listRoles(marker) {
        var options = {};
        if (marker)
            options.Marker = marker;
        listRolesWithRetry(options, buildRoleList);
    }
    function buildRoleList(err, data) {
        if (err)
            return callback(err);
        roleList = roleList.concat(data.Roles);
        if (data.IsTruncated)
            return listRoles(data.Marker);
        return callback(null, roleList);
    }
};
EC2.getInstance =
    function (instanceId, callback) {
        var params = {
            InstanceIds: [instanceId]
        };
        var describeInstancesWithRetry = wrapWithRetry(this.ec2, this.ec2.describeInstances);
        describeInstancesWithRetry(params, function (err, data) {
            if (err)
                return callback(err);
            var instance;
            if (data.Reservations && data.Reservations.length) {
                if (data.Reservations[0].Instances &&
                    data.Reservations[0].Instances.length) {
                    instance = data.Reservations[0].Instances[0];
                }
            }
            return callback(null, instance);
        });
    };
EC2.describeInstanceStatus =
    function (instanceId, callback) {
        var params = {
            InstanceIds: [instanceId],
            IncludeAllInstances: true
        };
        var describeInstanceStatusWithRetry = wrapWithRetry(this.ec2, this.ec2.describeInstanceStatus);
        describeInstanceStatusWithRetry(params, function (err, data) {
            if (err)
                return callback(err);
            var instanceStatus;
            if (data.InstanceStatuses && data.InstanceStatuses.length)
                instanceStatus = data.InstanceStatuses[0].InstanceState;
            return callback(null, instanceStatus);
        });
    };
EC2.runInstance =
    function (params, callback) {
        var runParams = {
            ImageId: params.amiId,
            SubnetId: params.subnetId,
            MaxCount: 1,
            MinCount: 1,
            DryRun: false,
            Monitoring: {
                Enabled: false
            },
            InstanceType: params.instanceSize,
            InstanceInitiatedShutdownBehavior: 'stop',
            SecurityGroupIds: [params.securityGroupId],
            UserData: params.userData,
            KeyName: params.keyName,
            TagSpecifications: params.tagSpecifications
        };
        if (!_.isEmpty(params.blockDeviceMappings))
            runParams.BlockDeviceMappings = params.blockDeviceMappings;
        if (params.hostId)
            runParams.Placement = {
                HostId: params.hostId,
                Tenancy: 'host'
            };
        if (params.clientToken)
            runParams.ClientToken = params.clientToken;
        var runInstnacesWithRetry = wrapWithRetry(this.ec2, this.ec2.runInstances);
        runInstnacesWithRetry(runParams, function (err, data) {
            if (err)
                return callback(err);
            var instance;
            if (data.Instances && data.Instances.length) {
                instance = data.Instances[0];
                return callback(null, instance);
            }
            else {
                return callback('Could not bring up a new instance on EC2');
            }
        });
    };
EC2.stopInstance =
    function (instanceId, callback) {
        var stopParams = {
            InstanceIds: [instanceId],
            DryRun: false
        };
        var stopInstancesWithRetry = wrapWithRetry(this.ec2, this.ec2.stopInstances);
        stopInstancesWithRetry(stopParams, function (err, data) {
            if (err)
                return callback(err);
            var instance;
            if (data.StoppingInstances && data.StoppingInstances.length) {
                instance = data.StoppingInstances[0];
                return callback(null, instance);
            }
            else {
                return callback('Could not stop EC2 instance ' + instanceId);
            }
        });
    };
EC2.startInstance =
    function (instanceId, callback) {
        var startParams = {
            InstanceIds: [instanceId],
            DryRun: false
        };
        var startInstancesWithRetry = wrapWithRetry(this.ec2, this.ec2.startInstances);
        startInstancesWithRetry(startParams, function (err, data) {
            if (err)
                return callback(err);
            var instance;
            if (data.StartingInstances && data.StartingInstances.length) {
                instance = data.StartingInstances[0];
                return callback(null, instance);
            }
            else {
                return callback('Could not start EC2 instance ' + instanceId);
            }
        });
    };
EC2.createTags = function (instanceId, tags, callback) {
    var tagParams = {
        Resources: [instanceId],
        Tags: tags,
        DryRun: false,
    };
    var createTagsWithRetry = wrapWithRetry(this.ec2, this.ec2.createTags);
    createTagsWithRetry(tagParams, function (err) {
        if (err)
            return callback(err);
        return callback(null);
    });
};
EC2.terminateInstance = function (instanceId, callback) {
    var terminateParams = {
        InstanceIds: [instanceId],
        DryRun: false,
    };
    var terminateInstancesWithRetry = wrapWithRetry(this.ec2, this.ec2.terminateInstances);
    terminateInstancesWithRetry(terminateParams, function (err) {
        if (err)
            return callback(err);
        return callback(null);
    });
};
EC2.allocateDedicatedHost =
    function (availabilityZone, instanceSize, clientToken, tagSpecifications, callback) {
        var hostParams = {
            AvailabilityZone: availabilityZone,
            InstanceType: instanceSize,
            Quantity: 1,
            AutoPlacement: 'off',
            HostRecovery: 'off',
            ClientToken: clientToken,
            TagSpecifications: tagSpecifications
        };
        var allocateHostsWithRetry = wrapWithRetry(this.ec2, this.ec2.allocateHosts);
        allocateHostsWithRetry(hostParams, function (err, data) {
            if (err)
                return callback(err);
            var hostId;
            if (data.HostIds && data.HostIds.length) {
                hostId = data.HostIds[0];
                return callback(null, hostId);
            }
            else {
                return callback('Could not allocate new host on EC2');
            }
        });
    };
EC2.describeHosts =
    function (filters, hostIds, maxResults, nextToken, callback) {
        var hostParams = {};
        if (!_.isEmpty(filters))
            hostParams.Filter = filters;
        if (!_.isEmpty(hostIds))
            hostParams.HostIds = hostIds;
        else
            hostParams.MaxResults = maxResults || 100;
        if (nextToken)
            hostParams.NextToken = nextToken;
        var describeHostsWithRetry = wrapWithRetry(this.ec2, this.ec2.describeHosts);
        describeHostsWithRetry(hostParams, function (err, data) {
            if (err)
                return callback(err);
            if (data.Hosts)
                return callback(null, data.Hosts, data.NextToken);
            else
                return callback('Could not describe hosts on EC2');
        });
    };
EC2.releaseHost =
    function (hostId, callback) {
        var hostParams = {
            HostIds: [hostId]
        };
        var releaseHostsWithRetry = wrapWithRetry(this.ec2, this.ec2.releaseHosts);
        releaseHostsWithRetry(hostParams, function (err, data) {
            if (err)
                return callback(err, data);
            return callback(null, data);
        });
    };
EC2.describeSubnets =
    function (subnetIds, callback) {
        var subnetParams = {
            SubnetIds: subnetIds
        };
        var describeSubnetsWithRetry = wrapWithRetry(this.ec2, this.ec2.releaseHosts);
        describeSubnetsWithRetry(subnetParams, function (err, data) {
            if (err)
                return callback(err);
            if (data.Subnets && data.Subnets.length)
                return callback(null, data.Subnets);
            else
                return callback('Could not describe subnets on EC2');
        });
    };
ECS.getClusters = function (callback) {
    var self = this;
    var clusterList = [];
    const listClustersWithRetry = wrapWithRetry(self.ecs, self.ecs.listClusters);
    listClusters();
    function listClusters(nextToken) {
        var options = {};
        if (nextToken)
            options.nextToken = nextToken;
        listClustersWithRetry(options, buildClusterList);
    }
    function buildClusterList(err, data) {
        if (err && err.code === 'UnknownEndpoint') {
            logger.warn('ECS not available in : ' +
                self.region + ' ' + util.inspect(err.message));
            return callback(null, []);
        }
        else if (err && err.code === 'NetworkingError') {
            logger.warn('Could not connect to region : ' +
                self.region + ' ' + util.inspect(err.message));
            return callback(null, []);
        }
        else if (err) {
            return callback(err);
        }
        else {
            clusterList = clusterList.concat(data.clusterArns);
            if (data.nextToken)
                return listClusters(data.nextToken);
            return callback(null, clusterList);
        }
    }
};
ECS.getClusterByName = function (clusterName, callback) {
    var clusterObject = {
        clusters: [clusterName]
    };
    const describeClustersWithRetry = wrapWithRetry(this.ecs, this.ecs.describeClusters);
    describeClustersWithRetry(clusterObject, function (err, data) {
        if (err)
            return callback(err);
        var cluster;
        if (data.clusters.length > 0)
            cluster = data.clusters[0];
        return callback(null, cluster);
    });
};
ECS.getContainerInstancesByClusterName = function (clusterName, callback) {
    var self = this;
    var containerInstanceList = [];
    const listContainerInstancesWithRetry = wrapWithRetry(self.ecs, self.ecs.listContainerInstances);
    listContainerInstances();
    function listContainerInstances(nextToken) {
        var options = {
            cluster: clusterName
        };
        if (nextToken)
            options.nextToken = nextToken;
        listContainerInstancesWithRetry(options, buildContainerInstanceList);
    }
    function buildContainerInstanceList(err, data) {
        if (err)
            return callback(err);
        containerInstanceList =
            containerInstanceList.concat(data.containerInstanceArns);
        if (data.nextToken)
            return listContainerInstances(data.nextToken);
        return callback(null, containerInstanceList);
    }
};
ECS.getContainerInstance =
    function (clusterName, containerInstanceArn, callback) {
        var params = {
            containerInstances: [containerInstanceArn],
            cluster: clusterName
        };
        const describeContainerInstancesWithRetry = wrapWithRetry(this.ecs, this.ecs.describeContainerInstances);
        describeContainerInstancesWithRetry(params, function (err, data) {
            if (err)
                return callback(err);
            var containerInstance;
            if (data.containerInstances && data.containerInstances.length)
                containerInstance = data.containerInstances[0];
            return callback(null, containerInstance);
        });
    };
ECS.registerTaskDefinition = function (json, callback) {
    var self = this;
    const registerTaskDefinitionWithRetry = wrapWithRetry(self.ecs, self.ecs.registerTaskDefinition);
    registerTaskDefinitionWithRetry(json, function (err, data) {
        if (err)
            return callback(err);
        return callback(null, data);
    });
};
ECS.listTaskDefinitions = function (json, callback) {
    var self = this;
    const listTaskDefinitionswithRetry = wrapWithRetry(self.ecs, self.ecs.listTaskDefinitions);
    listTaskDefinitionswithRetry(json, function (err, data) {
        if (err)
            return callback(err);
        return callback(null, data);
    });
};
ECS.deregisterTaskDefinition = function (json, callback) {
    var self = this;
    const deregisterTaskDefinitionwithRetry = wrapWithRetry(self.ecs, self.ecs.deregisterTaskDefinition);
    deregisterTaskDefinitionwithRetry(json, function (err, data) {
        if (err)
            return callback(err);
        return callback(null, data);
    });
};
ECS.getService = function (clusterName, serviceName, callback) {
    var params = {
        cluster: clusterName,
        services: [serviceName]
    };
    const describeServiceswithRetry = wrapWithRetry(this.ecs, this.ecs.describeServices);
    describeServiceswithRetry(params, callback);
};
ECS.findService = function (params, callback) {
    var self = this;
    var foundService;
    var errors;
    async.retry({ times: 3, interval: 2000 }, function (done) {
        var identifiers = [];
        if (params.serviceArn)
            identifiers.push(params.serviceArn);
        if (params.serviceName)
            identifiers.push(params.serviceName);
        async.eachSeries(identifiers, function (identifier, next) {
            if (!!foundService)
                return next();
            tryToFindService(params.clusterName, identifier, function () {
                return next();
            });
        }, function (err) {
            if (err)
                return done(err);
            return done(!foundService);
        });
    }, function () {
        if (foundService)
            return callback(null, foundService);
        return callback(errors ||
            util.format('Unable to find service %s %s in cluster %s', params.serviceName, params.serviceArn, params.clusterName));
    });
    function tryToFindService(clusterName, identifier, done) {
        var params = {
            cluster: clusterName,
            services: [identifier]
        };
        self.ecs.describeServices(params, function (err, response) {
            if (err) {
                logger.warn(err);
                errors = err;
                return done(err);
            }
            if (response && !_.isEmpty(response.services))
                foundService = _.first(response.services);
            else if (response && !_.isEmpty(response.errors)) {
                errors = response.errors;
                logger.warn(errors);
            }
            return done();
        });
    }
};
ECS.createService = function (params, callback) {
    const createServicewithRetry = wrapWithRetry(this.ecs, this.ecs.createService);
    createServicewithRetry(params, callback);
};
ECS.deleteService = function (clusterName, serviceName, callback) {
    var params = {
        cluster: clusterName,
        service: serviceName
    };
    const deleteServiceWithRetry = wrapWithRetry(this.ecs, this.ecs.deleteService);
    deleteServiceWithRetry(params, callback);
};
ECS.updateService = function (params, callback) {
    const updateServicewithRetry = wrapWithRetry(this.ecs, this.ecs.updateService);
    updateServicewithRetry(params, callback);
};
Adapter.generateGetLoadBalancersHash = function (params) {
    return {
        LoadBalancerNames: [params.loadBalancerName]
    };
};
ELB.getLoadBalancers = function (params, callback) {
    var self = this;
    var loadBalancerList = [];
    const describeLoadBalancerswithRetry = wrapWithRetry(this.elb, this.elb.describeLoadBalancers);
    listLoadBalancers();
    function listLoadBalancers(nextToken) {
        if (nextToken)
            params.nextToken = nextToken;
        describeLoadBalancerswithRetry(params, buildLoadBalancerList);
    }
    function buildLoadBalancerList(err, data) {
        if (err)
            return callback(err);
        loadBalancerList = loadBalancerList.concat(data.LoadBalancerDescriptions);
        if (data.nextToken)
            return listLoadBalancers(data.NextMarker);
        return callback(null, loadBalancerList);
    }
};
Adapter.generateNewELBMultiListenerHash = function (params) {
    var listeners = [];
    _.each(params.listeners, function (listener) {
        listeners.push({
            InstancePort: listener.loadBalancerInternalPort,
            LoadBalancerPort: listener.loadBalancerExternalPort,
            Protocol: listener.loadBalancerExternalProtocol,
            InstanceProtocol: listener.loadBalancerInternalProtocol,
            SSLCertificateId: listener.certificateSourceId || ''
        });
    });
    return {
        Listeners: listeners,
        LoadBalancerName: params.loadBalancerName
    };
};
Adapter.generateNewELBListenerHash = function (params) {
    return {
        Listeners: [
            {
                InstancePort: params.loadBalancerInternalPort,
                LoadBalancerPort: params.loadBalancerExternalPort,
                Protocol: params.loadBalancerExternalProtocol,
                InstanceProtocol: params.loadBalancerInternalProtocol,
                SSLCertificateId: ''
            }
        ],
        LoadBalancerName: params.loadBalancerName
    };
};
ELB.createLoadBalancerListeners = function (params, callback) {
    const createLoadBalancerListenerswithRetry = wrapWithRetry(this.elb, this.elb.createLoadBalancerListeners);
    createLoadBalancerListenerswithRetry(params, callback);
};
Adapter.generateDeleteELBMultiListenerHash = function (params) {
    return {
        LoadBalancerPorts: params.externalPorts,
        LoadBalancerName: params.loadBalancerName
    };
};
Adapter.generateDeleteELBListenerHash = function (params) {
    return {
        LoadBalancerPorts: [params.loadBalancerExternalPort],
        LoadBalancerName: params.loadBalancerName
    };
};
ELB.deleteLoadBalancerListeners = function (params, callback) {
    const deleteLoadBalancerListenerswithRetry = wrapWithRetry(this.elb, this.elb.deleteLoadBalancerListeners);
    deleteLoadBalancerListenerswithRetry(params, callback);
};
Adapter.generateELBConfigureHealthCheckHash = function (params) {
    return {
        HealthCheck: {
            HealthyThreshold: 2,
            Interval: 10,
            Target: params.healthCheck.protocol + ':' +
                params.healthCheck.hostPort + params.healthCheck.path,
            Timeout: 3,
            UnhealthyThreshold: 2
        },
        LoadBalancerName: params.loadBalancerName
    };
};
ELB.configureHealthCheck = function (params, callback) {
    const configureHealthCheckwithRetry = wrapWithRetry(this.elb, this.elb.configureHealthCheck);
    configureHealthCheckwithRetry(params, callback);
};
S3.uploadFileToS3 = function (params, callback) {
    const uploadwithRetry = wrapWithRetry(this.s3, this.s3.upload);
    uploadwithRetry(params, callback);
};
STS.assumeRole = function (roleArn, externalId, callback) {
    var params = {
        RoleArn: roleArn,
        RoleSessionName: externalId,
        ExternalId: externalId
    };
    const assumeRoleWithRetry = wrapWithRetry(this.sts, this.sts.assumeRole);
    assumeRoleWithRetry(params, function (err, data) {
        if (err)
            return callback(err);
        callback(null, data.Credentials);
    });
};
S3.listObjects = function (bucket, prefix, callback) {
    var params = {
        Bucket: bucket,
        Prefix: prefix + '/'
    };
    const listObjectsWithRetry = wrapWithRetry(this.s3, this.s3.listObjects);
    listObjectsWithRetry(params, function (err, objects) {
        if (err)
            return callback(err);
        callback(null, objects);
    });
};
S3.listObjectsV2 = function (params, callback) {
    const listObjectsV2WithRetry = wrapWithRetry(this.s3, this.s3.listObjectsV2);
    listObjectsV2WithRetry(params, function (err, objects) {
        if (err)
            return callback(err);
        callback(null, objects);
    });
};
S3.generateDELETEUrl = function (bucket, key, expiration, callback) {
    var params = {
        Bucket: bucket,
        Key: key,
        Expires: expiration
    };
    const getSignedUrlWithRetry = wrapWithRetry(this.s3, this.s3.getSignedUrl);
    getSignedUrlWithRetry('deleteObject', params, function (err, data) {
        if (err)
            return callback(err);
        return callback(null, data);
    });
};
ApplicationAutoScaling.registerScalableTarget = function (params, callback) {
    var req = {
        MaxCapacity: params.max,
        MinCapacity: params.min,
        ResourceId: params.resourceId,
        ScalableDimension: params.scalableDimension,
        ServiceNamespace: params.namespace
    };
    if (params.role)
        req.RoleARN = params.role;
    const registerScalableTargetWithRetry = wrapWithRetry(this.applicationautoscaling, this.applicationautoscaling.registerScalableTarget);
    registerScalableTargetWithRetry(req, function (err, data) {
        if (err)
            return callback(err);
        return callback(null, data);
    });
};
ApplicationAutoScaling.deregisterScalableTarget = function (params, callback) {
    var req = {
        ResourceId: params.resourceId,
        ScalableDimension: params.scalableDimension,
        ServiceNamespace: params.namespace
    };
    const deregisterScalableTargetWithRetry = wrapWithRetry(this.applicationautoscaling, this.applicationautoscaling.deregisterScalableTarget);
    deregisterScalableTargetWithRetry(req, function (err, data) {
        if (err)
            return callback(err);
        return callback(null, data);
    });
};
ApplicationAutoScaling.putAutoScalingPolicy = function (params, callback) {
    const putScalingPolicyWithRetry = wrapWithRetry(this.applicationautoscaling, this.applicationautoscaling.putScalingPolicy);
    putScalingPolicyWithRetry(params, function (err, data) {
        if (err)
            return callback(err);
        return callback(null, data);
    });
};
ApplicationAutoScaling.deleteAutoScalingPolicy = function (params, callback) {
    const deleteScalingPolicyWithRetry = wrapWithRetry(this.applicationautoscaling, this.applicationautoscaling.deleteScalingPolicy);
    deleteScalingPolicyWithRetry(params, function (err, data) {
        if (err)
            return callback(err);
        return callback(null, data);
    });
};
CloudWatch.putMetricAlarm = function (params, callback) {
    const putMetricAlarmWithRetry = wrapWithRetry(this.cloudwatch, this.cloudwatch.putMetricAlarm);
    putMetricAlarmWithRetry(params, function (err, data) {
        if (err)
            return callback(err);
        return callback(null, data);
    });
};
CloudWatch.deleteAlarms = function (alarmNames, callback) {
    var params = {
        AlarmNames: alarmNames
    };
    const deleteMetricAlarmWithRetry = wrapWithRetry(this.cloudwatch, this.cloudwatch.deleteAlarms);
    deleteMetricAlarmWithRetry(params, function (err, data) {
        if (err)
            return callback(err);
        return callback(null, data);
    });
};
//# sourceMappingURL=Adapter.js.map