'use strict';
var self = Adapter;
module.exports = self;
var uuid = require('node-uuid');
var AzureCompute = require('@azure/arm-compute');
var AzureNetwork = require('@azure/arm-network');
var AzureResources = require('@azure/arm-resources');
var MsRestNodeAuth = require('@azure/ms-rest-nodeauth');
var async = require('async');
var _ = require('underscore');
var AZCompute = {};
var AZNetwork = {};
var AZResource = {};
var { EXPONENTIAL_RETRY_OPTIONS } = require('../../../constants.js');
function wrapWithRetry(ctx, func) {
    const options = {
        ...EXPONENTIAL_RETRY_OPTIONS,
        errorFilter: function (err) {
            return (err && err.body && err.body.innerError &&
                (err.body.innerError.internalErrorCode === 'TooManyRequestsReceived'));
        }
    };
    return async.retryable(options, func.bind(ctx));
}
function Adapter(appId, tenantId, clientSecret, subscriptionId) {
    this.config = {};
    this.appId = appId;
    this.tenantId = tenantId;
    this.clientSecret = clientSecret;
    this.subscriptionId = subscriptionId;
}
Adapter.prototype.initialize = function (callback) {
    var self = this;
    const loginWithServicePrincipalSecretWithRetry = wrapWithRetry(MsRestNodeAuth, MsRestNodeAuth.loginWithServicePrincipalSecret);
    loginWithServicePrincipalSecretWithRetry(self.appId, self.clientSecret, self.tenantId, function (err, credentials) {
        if (err)
            return callback(err);
        self.azComputeClient = new AzureCompute.ComputeManagementClient(credentials, self.subscriptionId);
        self.azNetworkClient = new AzureNetwork.NetworkManagementClient(credentials, self.subscriptionId);
        self.azResourceClient = new AzureResources.ResourceManagementClient(credentials, self.subscriptionId);
        self.AZCompute = bindSubAdapter(self, AZCompute);
        self.AZNetwork = bindSubAdapter(self, AZNetwork);
        self.AZResource = bindSubAdapter(self, AZResource);
        return callback();
    });
};
function bindSubAdapter(newThis, subAdapter) {
    var boundAdapter = {};
    _.each(Object.keys(subAdapter), function (item) {
        boundAdapter[item] = subAdapter[item].bind(newThis);
    });
    return boundAdapter;
}
AZNetwork.getSubnet = function (resourceGroup, vnetName, subnetName, callback) {
    var self = this;
    const getSubnetsWithRetry = wrapWithRetry(self.azNetworkClient.subnets, self.azNetworkClient.subnets.get);
    getSubnetsWithRetry(resourceGroup, vnetName, subnetName, function (err, subnet) {
        return callback(err, subnet);
    });
};
AZNetwork.getNetworkSecurityGroup = function (resourceGroup, nsgName, callback) {
    var self = this;
    const getNetworkSecurityGroupsWithRetry = wrapWithRetry(self.azNetworkClient.networkSecurityGroups, self.azNetworkClient.networkSecurityGroups.get);
    getNetworkSecurityGroupsWithRetry(resourceGroup, nsgName, function (err, networkSecurityGroup) {
        return callback(err, networkSecurityGroup);
    });
};
AZNetwork.createNIC = function (resourceGroup, subnet, networkSecurityGroup, location, nicName, callback) {
    var self = this;
    var nicParameters = {
        location: location,
        ipConfigurations: [
            {
                name: nicName,
                subnet: subnet,
                privateIPAllocationMethod: 'Dynamic'
            }
        ]
    };
    if (!!networkSecurityGroup)
        nicParameters.networkSecurityGroup = networkSecurityGroup;
    const createOrUpdateNICWithRetry = wrapWithRetry(self.azNetworkClient.networkInterfaces, self.azNetworkClient.networkInterfaces.createOrUpdate);
    createOrUpdateNICWithRetry(resourceGroup, nicName, nicParameters, function (err, nic) {
        return callback(err, nic);
    });
};
AZCompute.createVM = function (params, callback) {
    var self = this;
    var vmParameters = {
        location: params.location,
        osProfile: {
            computerName: params.vmName,
            adminUsername: params.vmName,
            customData: params.customData
        },
        hardwareProfile: {
            vmSize: params.vmSize
        },
        storageProfile: {
            osDisk: {
                osType: params.osType,
                createOption: 'fromImage'
            },
        },
        networkProfile: {
            networkInterfaces: [
                {
                    id: params.nicId,
                    primary: true
                }
            ]
        },
        tags: params.tags
    };
    if (params.osDiskSizeGB) {
        vmParameters.storageProfile.osDisk.diskSizeGB = params.osDiskSizeGB;
    }
    if (params.osType === 'Linux') {
        vmParameters.osProfile.linuxConfiguration = {
            disablePasswordAuthentication: true,
            ssh: {
                publicKeys: [{
                        keyData: params.publicSshKey,
                        path: '/home/' + params.vmName + '/.ssh/authorized_keys'
                    }]
            }
        };
    }
    else if (params.osType === 'Windows') {
        vmParameters.osProfile.adminUsername = params.adminUsername || params.vmName;
        vmParameters.osProfile.adminPassword = params.adminPassword || uuid.v4();
        vmParameters.osProfile.windowsConfiguration = {
            enableAutomaticUpdates: false,
        };
    }
    if (!!params.imageId) {
        vmParameters.storageProfile.imageReference = {
            id: params.imageId
        };
    }
    else if (!!params.imageUrn) {
        var imageData = params.imageUrn.split(':');
        vmParameters.storageProfile.imageReference = {
            publisher: imageData[0],
            offer: imageData[1],
            sku: imageData[2],
            version: imageData[3]
        };
    }
    if (!!params.imagePlan) {
        vmParameters.plan = params.imagePlan;
    }
    const beginCreateOrUpdateVMWithRetry = wrapWithRetry(self.azComputeClient.virtualMachines, self.azComputeClient.virtualMachines.beginCreateOrUpdate);
    beginCreateOrUpdateVMWithRetry(params.resourceGroup, params.vmName, vmParameters, function (err, response) {
        if (err)
            return callback(err);
        return callback(null, response);
    });
};
AZCompute.powerOffVM = function (resourceGroup, vmName, callback) {
    var self = this;
    const beginPowerOffVMWithRetry = wrapWithRetry(self.azComputeClient.virtualMachines, self.azComputeClient.virtualMachines.beginPowerOff);
    beginPowerOffVMWithRetry(resourceGroup, vmName, function (err, response) {
        if (err)
            return callback(err);
        return callback(null, response);
    });
};
AZCompute.getVM = function (resourceGroup, vmName, callback) {
    var self = this;
    const instanceViewWithRetry = wrapWithRetry(self.azComputeClient.virtualMachines, self.azComputeClient.virtualMachines.instanceView);
    instanceViewWithRetry(resourceGroup, vmName, function (err, instance) {
        callback(err, instance);
    });
};
AZCompute.startVM = function (resourceGroup, vmName, callback) {
    var self = this;
    const beginStartWithRetry = wrapWithRetry(self.azComputeClient.virtualMachines, self.azComputeClient.virtualMachines.beginStart);
    beginStartWithRetry(resourceGroup, vmName, function (err, response) {
        if (err)
            return callback(err);
        return callback(null, response);
    });
};
AZCompute.deleteVM = function (resourceGroup, vmName, callback) {
    var self = this;
    const beginDeleteMethodWithRetry = wrapWithRetry(self.azComputeClient.virtualMachines, self.azComputeClient.virtualMachines.beginDeleteMethod);
    beginDeleteMethodWithRetry(resourceGroup, vmName, function (err, response) {
        if (err)
            return callback(err);
        return callback(null, response);
    });
};
AZNetwork.getNIC = function (resourceGroup, nicName, callback) {
    var self = this;
    const getNICWithRetry = wrapWithRetry(self.azNetworkClient.networkInterfaces, self.azNetworkClient.networkInterfaces.get);
    getNICWithRetry(resourceGroup, nicName, function (err, nic) {
        callback(err, nic);
    });
};
AZNetwork.deleteNIC = function (resourceGroup, nicName, callback) {
    var self = this;
    const deleteNICWithRetry = wrapWithRetry(self.azNetworkClient.networkInterfaces, self.azNetworkClient.networkInterfaces.beginDeleteMethod);
    deleteNICWithRetry(resourceGroup, nicName, function (err, response) {
        if (err)
            return callback(err);
        return callback(null, response);
    });
};
AZResource.listResourcesByResourceGroupAndTag = function (resourceGroup, tagName, tagValue, callback) {
    var self = this;
    var options = {
        filter: 'tagName eq \'' + tagName + '\' and tagValue eq \'' +
            tagValue + '\''
    };
    const listByResourceGroupWithRetry = wrapWithRetry(self.azResourceClient.resources, self.azResourceClient.resources.listByResourceGroup);
    listByResourceGroupWithRetry(resourceGroup, options, function (err, resources) {
        callback(err, resources);
    });
};
AZCompute.getDisk = function (resourceGroup, diskName, callback) {
    var self = this;
    const getDisksWithRetry = wrapWithRetry(self.azComputeClient.disks, self.azComputeClient.disks.get);
    getDisksWithRetry(resourceGroup, diskName, function (err, disk) {
        callback(err, disk);
    });
};
AZCompute.deleteDisk = function (resourceGroup, diskName, callback) {
    var self = this;
    const beginDeleteDisksWithRetry = wrapWithRetry(self.azComputeClient.disks, self.azComputeClient.disks.beginDeleteMethod);
    beginDeleteDisksWithRetry(resourceGroup, diskName, function (err, response) {
        if (err)
            return callback(err);
        return callback(null, response);
    });
};
//# sourceMappingURL=Adapter.js.map