'use strict';
var self = Adapter;
module.exports = self;
var async = require('async');
var _ = require('underscore');
var util = require('util');
var fs = require('fs-extra');
var uuid = require('node-uuid');
var childProcess = require('child_process');
function Adapter(kubeconfig) {
    this.kubeconfig = kubeconfig;
    this.kubeConfigContent = '';
    this.sessionPath = util.format('/opt/jfrog/pipelines/var/tmp/%s', uuid.v4());
    this.kubeConfigFilePath = util.format('%s/kubeconfig', this.sessionPath);
}
Adapter.prototype.initialize = function (callback) {
    var bag = this;
    async.series([
        _validateAdapter.bind(null, bag),
        _initializeCommands.bind(null, bag),
        _initializeAdapterDir.bind(null, bag),
        _writeKubeConfigFile.bind(null, bag)
    ], function (err) {
        return callback(err);
    });
};
function _validateAdapter(bag, next) {
    if (_.isEmpty(bag.kubeconfig))
        return next('expects non-empty string: kubeconfig');
    bag.kubeConfigContent = bag.kubeconfig;
    return next();
}
function _initializeCommands(bag, next) {
    bag.commands = {
        mkdirp: fs.mkdirp,
        writeFile: fs.writeFile,
        execute: __execute,
        removeDirectoryWithContents: fs.remove
    };
    return next();
}
function _initializeAdapterDir(bag, next) {
    bag.commands.mkdirp(bag.sessionPath, function (err) {
        if (err)
            return next('Failed to create adapter directory: ' + err);
        return next();
    });
}
function _writeKubeConfigFile(bag, next) {
    bag.commands.writeFile(bag.kubeConfigFilePath, bag.kubeConfigContent, function (err) {
        if (err)
            return next('Failed to create kubeconfigfile ' + err);
        return next();
    });
}
Adapter.prototype.apply =
    function (namespaceName, fileName, fileContent, callback) {
        var self = this;
        var filePath = util.format('%s/%s.json', this.sessionPath, fileName);
        this.commands.writeFile(filePath, fileContent, function (err) {
            if (err)
                return callback(err);
            var command = 'kubectl';
            var opts = {
                args: [
                    'apply', '-f', filePath,
                    '--namespace', namespaceName,
                    '--kubeconfig', self.kubeConfigFilePath
                ]
            };
            self.commands.execute(command, opts, callback);
        });
    };
Adapter.prototype.rolloutStatus =
    function (objectName, namespaceName, callback) {
        var command = 'kubectl';
        var opts = {
            args: [
                'rollout', 'status', objectName,
                '--namespace', namespaceName,
                '--watch=false',
                '--kubeconfig', this.kubeConfigFilePath
            ]
        };
        this.commands.execute(command, opts, callback);
    };
Adapter.prototype.destroy = function (callback) {
    var bag = this;
    async.series([
        _removeSessionDir.bind(null, bag)
    ], function (err) {
        return callback(err);
    });
};
function _removeSessionDir(bag, next) {
    bag.commands.removeDirectoryWithContents(bag.sessionPath, function (err) {
        if (err)
            return next('Failed to remove session dir: ' + err);
        return next();
    });
}
Adapter.prototype.delete = function (objectName, namespaceName, callback) {
    var command = 'kubectl';
    var opts = {
        args: [
            'delete', objectName,
            '--namespace', namespaceName,
            '--kubeconfig', this.kubeConfigFilePath
        ]
    };
    this.commands.execute(command, opts, callback);
};
Adapter.prototype.get = function (objectName, namespaceName, outputFormat, labels, callback) {
    var command = 'kubectl';
    var opts = {
        args: [
            'get', objectName,
            '--namespace', namespaceName,
            '-o', outputFormat,
            '--kubeconfig', this.kubeConfigFilePath
        ],
        muteStdoutLogging: true
    };
    if (labels)
        opts.args.push('-l', labels);
    this.commands.execute(command, opts, callback);
};
Adapter.prototype.scale = function (req, namespaceName, callback) {
    var command = 'kubectl';
    var opts = {
        args: [
            'scale', req.name,
            '--replicas', req.replicas,
            '--namespace', namespaceName,
            '--kubeconfig', this.kubeConfigFilePath
        ]
    };
    this.commands.execute(command, opts, callback);
};
Adapter.prototype.createConfigMap =
    function (fileName, configMapContent, namespaceName, configMapName, callback) {
        var bag = {
            fileName: fileName,
            configMapContent: configMapContent,
            namespaceName: namespaceName,
            configMapName: configMapName,
            sessionPath: this.sessionPath,
            commands: this.commands,
            kubeConfigFilePath: this.kubeConfigFilePath,
            createStdOut: null,
            createErrOut: null
        };
        async.series([
            _writeConfigMapFile.bind(null, bag),
            _createConfigMap.bind(null, bag)
        ], function (err) {
            return callback(err, bag.createStdOut, bag.createErrOut);
        });
    };
function _writeConfigMapFile(bag, next) {
    bag.filePath = util.format('%s/%s', bag.sessionPath, bag.fileName);
    bag.commands.writeFile(bag.filePath, bag.configMapContent, 'utf8', function (err) {
        if (err)
            return next('Failed to create configmap file ' + err);
        return next();
    });
}
function _createConfigMap(bag, next) {
    bag.commands.execute('kubectl', {
        args: [
            'create', 'configmap', bag.configMapName,
            '--namespace', bag.namespaceName,
            '--from-file', bag.filePath,
            '--kubeconfig', bag.kubeConfigFilePath
        ]
    }, function (err, stdOut, errOut) {
        bag.createStdOut = stdOut;
        bag.createErrOut = errOut;
        return next(err);
    });
}
function __execute(command, options, callback) {
    var cmd;
    if (!_.isEmpty(options.timeout))
        cmd = childProcess.spawn('timeout', [
            options.timeout.duration,
            command
        ].concat(options.args));
    else
        cmd = childProcess.spawn(command, options.args);
    var stdout = '';
    var stderr = '';
    cmd.stdout.on('data', function (data) {
        if (!options.muteStdoutLogging)
            console.log(data.toString());
        stdout += data;
    });
    cmd.stderr.on('data', function (data) {
        console.log(data.toString());
        stderr += data;
    });
    cmd.on('error', function (err) {
        console.log(err.toString());
    });
    cmd.on('close', function (code) {
        var err;
        if (!_.isEmpty(options.timeout) && code === 124) {
            err = util.format('%s timeout out: %s elapsed', command, options.timeout.duration);
        }
        else if (code > 0) {
            err = util.format('%s failed and exited with error code %s', command, code);
        }
        return callback(err, stdout, stderr);
    });
}
Adapter.prototype.isAuthorizeToAccessResource =
    function (namespaceName, resource, callback) {
        var command = 'kubectl';
        var opts = {
            args: [
                'auth', 'can-i', 'create', resource,
                '--namespace', namespaceName,
                '--kubeconfig', this.kubeConfigFilePath
            ]
        };
        this.commands.execute(command, opts, callback);
    };
//# sourceMappingURL=Adapter.js.map