'use strict';
var self = handler;
module.exports = self;
var KubernetesAdapter = require('../interface/kubernetes/Adapter.js');
var ApiAdapter = require('../../Adapter.js');
var nodeActions = require('../../nodeActions.js');
var pvcSpec = require('./templates/pvcSpec.js');
var secretSpec = require('./templates/secretSpec.js');
var deploymentSpec = require('./templates/deploymentSpec.js');
var betaJobSpec = require('./templates/betaJobSpec.js');
var betaDockerRegistrySecretSpec = require('./templates/betaDockerRegistrySecretSpec.js');
const { TROUBLESHOOT_MSG } = require('../../constants.js');
function handler(params, callback) {
    var bag = {
        action: params.action,
        runtime: params.runtime,
        buildPlaneMessageId: params.buildPlaneMessageId,
        skipPVCDelete: params.skipPVCDelete,
        pvcName: params.pvcName || '',
        nodeName: params.nodeName,
        nodeId: params.nodeId,
        nodePool: params.nodePool,
        instanceId: params.instanceId,
        operatingSystem: params.operatingSystem,
        architecture: params.architecture,
        startupScriptData: params.startupScriptData || {},
        consoleAdapter: params.consoleAdapter,
        projectId: params.projectId,
        kubeconfig: params.kubeconfig,
        nodePoolId: params.nodePoolId,
        storageClass: params.providerMetadata &&
            params.providerMetadata.storageClass,
        namespace: (params.providerMetadata &&
            params.providerMetadata.namespace) || (params.nodeProviderMetadata &&
            params.nodeProviderMetadata.namespace) || 'default',
        labels: params.providerMetadata &&
            params.providerMetadata.labels,
        cpu: params.providerMetadata &&
            params.providerMetadata.cpu,
        memory: params.providerMetadata &&
            params.providerMetadata.memory,
        isSuccess: true,
        skipConsolePost: false,
        instance: {},
        deploymentDeleted: _.isEmpty(params.instanceId),
        secretDeleted: _.isEmpty(params.instanceId),
        pvcDeleted: _.isEmpty(params.instanceId),
        dockerDaemonConfigDeleted: _.isEmpty(params.instanceId),
        customCertsConfigDeleted: _.isEmpty(params.instanceId),
        k8sObjectDeleted: false,
        configMapName: '',
        customCertsConfigMapName: '',
        customCertsData: params.startupScriptData &&
            params.startupScriptData.customCertsData,
        customCertsEnabled: params.startupScriptData &&
            params.startupScriptData.customCertsEnabled,
        jFrogPipelinesSessionId: params.jFrogPipelinesSessionId,
        diskSizeinGB: params.diskSizeinGB,
        disableDind: params.disableDind || false,
        dockerRegistryIntegrationName: params.dockerRegistryIntegrationName,
        dockerRegistrySecretContent: ''
    };
    if (!bag.cpu && global.config && global.config.defaultK8sCPU) {
        bag.cpu = global.config.defaultK8sCPU;
    }
    if (!bag.memory && global.config && global.config.defaultK8sMemory) {
        bag.memory = global.config.defaultK8sMemory;
    }
    bag.who = util.format('k8s|%s|jFrogPipelinesSessionId:%s', self.name, bag.jFrogPipelinesSessionId);
    logger.debug(bag.who, 'Inside');
    async.series([
        _checkInputParams.bind(null, bag),
        _initializeKubernetesAdapter.bind(null, bag),
        _isAuthorizeToAccessResources.bind(null, bag),
        _getDockerRegistryIntegrationId.bind(null, bag),
        _createDockerRegistrySecretContent.bind(null, bag),
        _checkPVCCreation.bind(null, bag),
        _generatePVCSpec.bind(null, bag),
        _createPVC.bind(null, bag),
        _generateSecretSpec.bind(null, bag),
        _createSecret.bind(null, bag),
        _generateDockerRegistrySecretSpec.bind(null, bag),
        _createDockerRegistrySecret.bind(null, bag),
        _createDockerDaemonConfig.bind(null, bag),
        _createCustomCertsConfig.bind(null, bag),
        _generateDeploymentSpec.bind(null, bag),
        _createDeployment.bind(null, bag),
        _scaleDownDeployment.bind(null, bag),
        _checkIfDeploymentIsStopped.bind(null, bag),
        _scaleUpDeployment.bind(null, bag),
        _getDeployment.bind(null, bag),
        _deleteDeployment.bind(null, bag),
        _getSecret.bind(null, bag),
        _deleteSecret.bind(null, bag),
        _getDockerRegistrySecret.bind(null, bag),
        _deleteDockerRegistrySecret.bind(null, bag),
        _getPVC.bind(null, bag),
        _deletePVC.bind(null, bag),
        _getDockerDaemonConfig.bind(null, bag),
        _deleteDockerDaemonConfig.bind(null, bag),
        _getCustomCertsConfig.bind(null, bag),
        _deleteCustomCertsConfig.bind(null, bag),
        _removeTmpDirWithContents.bind(null, bag)
    ], function (err) {
        var result = {};
        if (bag.consoleAdapter)
            bag.consoleAdapter.closeGrp(bag.isSuccess);
        if (err) {
            logger.warn(bag.who, 'reported an error', err);
        }
        else {
            var k8sObjectDeleted;
            if (bag.action === nodeActions.DELETE && _.isEmpty(bag.instanceId))
                k8sObjectDeleted = true;
            else
                k8sObjectDeleted =
                    bag.pvcDeleted && bag.deploymentDeleted && bag.secretDeleted &&
                        bag.dockerDaemonConfigDeleted && bag.customCertsConfigDeleted;
            result = {
                instance: bag.instance,
                isSuccess: bag.isSuccess,
                k8sObjectsDeleted: k8sObjectDeleted
            };
            logger.debug(bag.who, 'Successfully completed processing of k8s');
        }
        result.errorForMetric = bag.errorForMetric;
        return callback(err, result);
    });
}
function _checkInputParams(bag, next) {
    var who = bag.who + '|' + _checkInputParams.name;
    logger.trace(who, 'Inside');
    bag.suAdapter = new ApiAdapter(bag.jFrogPipelinesSessionId, global.config.apiToken);
    var consoleErrors = [];
    var expectedParams = [
        'action',
        'nodeId'
    ];
    if (bag.action === nodeActions.CREATE)
        expectedParams = expectedParams.concat([
            'nodeName',
            'startupScriptData'
        ]);
    _.each(expectedParams, function (expectedParam) {
        if (_.isNull(bag[expectedParam]) || _.isUndefined(bag[expectedParam]))
            consoleErrors.push(util.format('%s is missing %s', who, expectedParam));
    });
    if (bag.action === nodeActions.DELETE)
        bag.skipConsolePost = true;
    if (consoleErrors.length > 0) {
        _.each(consoleErrors, function (err) {
            logger.error(bag.who, err);
        });
        return next(true);
    }
    return next();
}
function _initializeKubernetesAdapter(bag, next) {
    var who = bag.who + '|' + _initializeKubernetesAdapter.name;
    logger.trace(who, 'Inside');
    if (!bag.skipConsolePost) {
        var action;
        if (bag.action === nodeActions.CREATE)
            action = 'Creating';
        else if (bag.action === nodeActions.START)
            action = 'Starting';
        else if (bag.action === nodeActions.CACHE)
            action = 'Stopping';
        bag.consoleAdapter.openGrp(util.format('%s node: %s', action, bag.nodeId));
        bag.consoleAdapter.openCmd('Initializing Kubernetes adapter');
    }
    var kubeconfig = bag.kubeconfig;
    bag.kubernetesAdapter = new KubernetesAdapter(kubeconfig);
    bag.kubernetesAdapter.initialize(function (err) {
        if (err) {
            bag.errorForMetric = 'AdapaterInitializationFailed';
            bag.consoleAdapter.publishMsg('Failed to initialize Kubernetes ' +
                'Adapter');
            bag.closeCmd(false);
            logger.warn(who, 'Failed to initialize kubernetes adapter');
            bag.success = false;
        }
        if (!bag.skipConsolePost) {
            bag.consoleAdapter.publishMsg('Successfully initialized Kubernetes ' +
                'adapter');
            bag.consoleAdapter.closeCmd(true);
        }
        return next();
    });
}
function _isAuthorizeToAccessResources(bag, next) {
    if (!bag.isSuccess)
        return next();
    if (bag.action !== nodeActions.CREATE)
        return next();
    var who = self.name + '|' + _isAuthorizeToAccessResources.name;
    logger.debug(who, 'Inside');
    bag.consoleAdapter.openCmd('Checking whether user have access ' +
        'to create k8s resources');
    async.parallel([
        bag.kubernetesAdapter.isAuthorizeToAccessResource
            .bind(bag.kubernetesAdapter, bag.namespace, 'pvc'),
        bag.kubernetesAdapter.isAuthorizeToAccessResource
            .bind(bag.kubernetesAdapter, bag.namespace, 'secret'),
        bag.kubernetesAdapter.isAuthorizeToAccessResource
            .bind(bag.kubernetesAdapter, bag.namespace, 'configmap'),
        bag.kubernetesAdapter.isAuthorizeToAccessResource
            .bind(bag.kubernetesAdapter, bag.namespace, 'deployment')
    ], function (err) {
        if (err) {
            bag.errorForMetric = 'Unauthorized';
            var errMsg = 'Cannot connect to the provided kubernetes host or ' +
                'you do not have permissions to create resources such as PVC, ' +
                'secret, configmap, deployment.';
            bag.consoleAdapter.publishMsg(errMsg + TROUBLESHOOT_MSG);
            bag.consoleAdapter.closeCmd(false);
            logger.warn(who, errMsg, err);
            bag.isSuccess = false;
            return next();
        }
        bag.consoleAdapter.publishMsg('User have permissions to create' +
            ' resources.');
        bag.consoleAdapter.closeCmd(true);
        return next();
    });
}
function _getDockerRegistryIntegrationId(bag, next) {
    if (bag.nodePool && !bag.nodePool.isBeta)
        return next();
    if (!bag.isSuccess)
        return next();
    if (!bag.dockerRegistryIntegrationName)
        return next();
    if (bag.action !== nodeActions.CREATE)
        return next();
    var who = self.name + '|' + _getDockerRegistryIntegrationId.name;
    logger.debug(who, 'Inside');
    var query = 'sortOrder=1&sortBy=id&include=name,id';
    var callObjectLimit = 100;
    var fetchedAllProjectIntegrations = false;
    var skip = 0;
    var dockerRegistryIntegrationId = '';
    async.whilst(function () {
        return !fetchedAllProjectIntegrations;
    }, function (done) {
        var currentQuery = util.format('%s&limit=%s&skip=%s', query, callObjectLimit, skip);
        bag.suAdapter.getProjectIntegrations(currentQuery, function (err, projectIntegrations) {
            if (err) {
                logger.error(who, `getProjectIntegrations failed for query: ${query} with error:  ${err.message}`);
                bag.isSuccess = false;
                return next();
            }
            if (projectIntegrations.length === callObjectLimit)
                skip = skip + callObjectLimit;
            else
                fetchedAllProjectIntegrations = true;
            for (const integration of projectIntegrations) {
                if (integration.name === bag.dockerRegistryIntegrationName) {
                    bag.dockerRegistryIntegrationId = integration.id;
                    fetchedAllProjectIntegrations = true;
                    break;
                }
            }
            return done();
        });
    }, function (err) {
        if (err)
            return next(err);
        return next();
    });
}
function _createDockerRegistrySecretContent(bag, next) {
    if (bag.nodePool && !bag.nodePool.isBeta)
        return next();
    if (!bag.isSuccess)
        return next();
    if (!bag.dockerRegistryIntegrationId)
        return next();
    if (bag.action !== nodeActions.CREATE)
        return next();
    var who = self.name + '|' + _createDockerRegistrySecretContent.name;
    logger.debug(who, 'Inside');
    var integrationValues = {};
    var dockerRegistrySecret = {};
    bag.suAdapter.getProjectIntegrationById(bag.dockerRegistryIntegrationId, function (err, projectIntegration) {
        if (err) {
            logger.error(who, `getProjectIntegrationById failed for id: ${bag.dockerRegistryIntegrationId} with error:  ${err.message}`);
            bag.isSuccess = false;
            return next();
        }
        else {
            if (projectIntegration && projectIntegration.masterIntegrationName && projectIntegration.masterIntegrationName !== "dockerRegistryLogin") {
                bag.consoleAdapter.openCmd(`Found custom docker registry of type ${projectIntegration.masterIntegrationName}`);
                const message = 'Custom docker image registry login is supported only with integration type Docker Registry';
                bag.consoleAdapter.publishMsg(message);
                bag.consoleAdapter.closeCmd(false);
                logger.warn(who, message);
                bag.isSuccess = false;
                return next();
            }
            _.each(projectIntegration.formJSONValues, function (jsonValue) {
                if (jsonValue.label)
                    integrationValues[jsonValue.label] = jsonValue.value;
            });
            if (integrationValues?.url && integrationValues?.password && integrationValues?.username) {
                var auths = "auths";
                dockerRegistrySecret[auths] = {};
                var registryUrl = integrationValues.url;
                const protocolRegex = /^http(s)?:\/\//i;
                var registry = registryUrl.replace(protocolRegex, '').replace(/\/$/i, '');
                dockerRegistrySecret[auths][registry] = {};
                dockerRegistrySecret[auths][registry]["username"] = integrationValues.username;
                dockerRegistrySecret[auths][registry]["password"] = integrationValues.password;
                bag.dockerRegistrySecretContent = JSON.stringify(dockerRegistrySecret);
            }
        }
        return next();
    });
}
function _checkPVCCreation(bag, next) {
    if (bag.nodePool && !bag.nodePool.isBeta)
        return next();
    if (!bag.isSuccess)
        return next();
    if (bag.action !== nodeActions.CREATE)
        return next();
    var who = self.name + '|' + _checkPVCCreation.name;
    logger.debug(who, 'Inside');
    if (bag.pvcName) {
        bag.createPVC = false;
    }
    else {
        bag.createPVC = true;
    }
    return next();
}
function _generatePVCSpec(bag, next) {
    if (!bag.isSuccess)
        return next();
    if (bag.action !== nodeActions.CREATE)
        return next();
    var who = self.name + '|' + _generatePVCSpec.name;
    logger.debug(who, 'Inside');
    var innerBag = {
        storageClass: bag.storageClass,
        pvcName: bag.nodeName + '-pvc',
        diskSizeinGB: bag.diskSizeinGB
    };
    if (bag.nodePool.isBeta && bag.pvcName) {
        innerBag.pvcName = bag.pvcName;
    }
    bag.consoleAdapter.openCmd('Creating PVC ' + innerBag.pvcName);
    pvcSpec(innerBag, function (err, pvc) {
        if (err) {
            bag.errorForMetric = 'PVCGenerationFailed';
            bag.consoleAdapter.publishMsg('Failed to generate pvc spec with error: ' + err);
            bag.consoleAdapter.closeCmd(false);
            logger.warn(who, 'Failed to generate pvc spec', err);
            bag.isSuccess = false;
            return next();
        }
        bag.pvc = pvc;
        logger.trace(util.format('%s generated pvc spec', who));
        return next();
    });
}
function _createPVC(bag, next) {
    if (!bag.isSuccess)
        return next();
    if (bag.action !== nodeActions.CREATE)
        return next();
    var who = bag.who + '|' + _createPVC.name;
    logger.trace(who, 'Inside');
    if (bag.nodePool.isBeta && typeof bag.createPVC === 'boolean' && bag.createPVC === false) {
        var message = `Using existing pvc ${bag.pvcName}`;
        logger.debug(message);
        bag.consoleAdapter.publishMsg(message);
        bag.consoleAdapter.closeCmd(true);
        return next();
    }
    bag.kubernetesAdapter.apply(bag.namespace, bag.pvc.name, JSON.stringify(bag.pvc.spec), function (err, stdOut, errOut) {
        if (err) {
            bag.errorForMetric = 'PVCCreationFailed';
            const message = util.format('Failed to create pvc for node: %s - %s %s %s %s', bag.nodeId, bag.nodeName, err, stdOut, errOut);
            bag.consoleAdapter.publishMsg(message);
            bag.consoleAdapter.closeCmd(false);
            logger.warn(who, message);
            bag.isSuccess = false;
            return next();
        }
        bag.consoleAdapter.publishMsg('PVC ' + bag.pvc.name + ' created');
        bag.consoleAdapter.closeCmd(true);
        return next();
    });
}
function _generateSecretSpec(bag, next) {
    if (!bag.isSuccess)
        return next();
    if (bag.action !== nodeActions.CREATE)
        return next();
    var who = self.name + '|' + _generateSecretSpec.name;
    logger.debug(who, 'Inside');
    var innerBag = {
        secretName: bag.nodeName + '-secret',
        token: bag.startupScriptData.token
    };
    bag.consoleAdapter.openCmd('Creating Secret ' + innerBag.secretName);
    secretSpec(innerBag, function (err, secret) {
        if (err) {
            bag.errorForMetric = 'SecretGenerationFailed';
            bag.consoleAdapter.publishMsg('Failed to generate secret spec with error: ' + err);
            bag.consoleAdapter.closeCmd(false);
            logger.warn(who, 'Failed to generate secret spec', err);
            bag.isSuccess = false;
            return next();
        }
        bag.secret = secret;
        logger.trace(who, 'Generated secret spec');
        return next();
    });
}
function _createSecret(bag, next) {
    if (!bag.isSuccess)
        return next();
    if (bag.action !== nodeActions.CREATE)
        return next();
    var who = bag.who + '|' + _createSecret.name;
    logger.trace(who, 'Inside');
    bag.kubernetesAdapter.apply(bag.namespace, bag.secret.name, JSON.stringify(bag.secret.spec), function (err, stdOut, errOut) {
        if (err) {
            bag.errorForMetric = 'SecretCreationFailed';
            const message = util.format('Failed to create secret for node: %s - %s %s %s %s', bag.nodeId, bag.nodeName, err, stdOut, errOut);
            bag.consoleAdapter.publishMsg(message);
            bag.consoleAdapter.closeCmd(false);
            logger.warn(who, message);
            bag.isSuccess = false;
            return next();
        }
        bag.consoleAdapter.publishMsg('Secret created ' + bag.secret.name);
        bag.consoleAdapter.closeCmd(true);
        return next();
    });
}
function _generateDockerRegistrySecretSpec(bag, next) {
    if (!bag.isSuccess)
        return next();
    if (bag.action !== nodeActions.CREATE)
        return next();
    if (_.isEmpty(bag.dockerRegistrySecretContent))
        return next();
    var who = self.name + '|' + _generateDockerRegistrySecretSpec.name;
    logger.debug(who, 'Inside');
    var innerBag = {
        secretName: bag.nodeName + '-docker-reg-secret',
        dockerConfig: bag.dockerRegistrySecretContent,
        type: "kubernetes.io/dockerconfigjson"
    };
    bag.consoleAdapter.openCmd('Creating Docker Registry Secret ' + innerBag.secretName);
    betaDockerRegistrySecretSpec(innerBag, function (err, secret) {
        if (err) {
            bag.errorForMetric = 'SecretGenerationFailed';
            bag.consoleAdapter.publishMsg('Failed to generate docker registry secret spec with error: ' + err);
            bag.consoleAdapter.closeCmd(false);
            logger.warn(who, 'Failed to generate docker registry secret spec', err);
            bag.isSuccess = false;
            return next();
        }
        bag.dockerRegistrySecret = secret;
        logger.trace(who, 'Generated docker registry secret spec');
        return next();
    });
}
function _createDockerRegistrySecret(bag, next) {
    if (!bag.isSuccess)
        return next();
    if (bag.action !== nodeActions.CREATE)
        return next();
    if (_.isEmpty(bag.dockerRegistrySecretContent))
        return next();
    var who = bag.who + '|' + _createDockerRegistrySecret.name;
    logger.trace(who, 'Inside');
    bag.kubernetesAdapter.apply(bag.namespace, bag.dockerRegistrySecret.name, JSON.stringify(bag.dockerRegistrySecret.spec), function (err, stdOut, errOut) {
        if (err) {
            bag.errorForMetric = 'SecretCreationFailed';
            const message = util.format('Failed to create docker registry secret for node: %s - %s %s %s %s', bag.nodeId, bag.nodeName, err, stdOut, errOut);
            bag.consoleAdapter.publishMsg(message);
            bag.consoleAdapter.closeCmd(false);
            logger.warn(who, message);
            bag.isSuccess = false;
            return next();
        }
        bag.consoleAdapter.publishMsg('Docker Registry Secret created ' + bag.dockerRegistrySecret.name);
        bag.consoleAdapter.closeCmd(true);
        return next();
    });
}
function _createCustomCertsConfig(bag, next) {
    if (!bag.isSuccess)
        return next();
    if (bag.action !== nodeActions.CREATE)
        return next();
    if (!bag.customCertsEnabled)
        return next();
    const who = bag.who + '|' + _createCustomCertsConfig.name;
    logger.trace(who, 'Inside');
    bag.customCertsConfigMapName = bag.nodeName + '-custom-certs';
    const fileName = 'ca.crt';
    bag.consoleAdapter.openCmd('Creating Custom Certs Config');
    bag.kubernetesAdapter.createConfigMap(fileName, JSON.parse(bag.customCertsData), bag.namespace, bag.customCertsConfigMapName, function (err, stdOut, errOut) {
        if (err) {
            bag.errorForMetric = 'CustomCertsCreationFailed';
            const message = util.format('Failed to create custom certs configMap for node: %s - %s %s %s %s', bag.nodeId, bag.nodeName, err, stdOut, errOut);
            bag.consoleAdapter.publishMsg(message);
            bag.consoleAdapter.closeCmd(false);
            logger.warn(who, message);
            bag.isSuccess = false;
            return next();
        }
        bag.consoleAdapter.publishMsg('Custom Certs config created');
        bag.consoleAdapter.closeCmd(true);
        return next();
    });
}
function _generateDeploymentSpec(bag, next) {
    if (!bag.isSuccess)
        return next();
    if (bag.action !== nodeActions.CREATE)
        return next();
    var who = self.name + '|' + _generateDeploymentSpec.name;
    logger.debug(who, 'Inside');
    var dindMemory, dindCPU;
    var reqkickMemory, reqkickCPU;
    var nodeSelector;
    if (!_.isEmpty(bag.labels))
        nodeSelector = bag.labels;
    if (bag.memory) {
        if (bag.nodePool?.isBeta) {
            dindMemory = bag.memory * 0.75 + 'Mi';
            reqkickMemory = (typeof bag.disableDind !== 'undefined' && bag.disableDind === true) ? bag.memory * 1 + 'Mi' : bag.memory * 0.25 + 'Mi';
        }
        else {
            dindMemory = bag.memory * 0.75 + 'Mi';
            reqkickMemory = bag.memory * 0.25 + 'Mi';
        }
    }
    if (bag.cpu) {
        if (bag.nodePool?.isBeta) {
            dindCPU = bag.cpu * 0.75;
            reqkickCPU = (typeof bag.disableDind !== 'undefined' && bag.disableDind === true) ? bag.cpu * 1 : bag.cpu * 0.25;
        }
        else {
            dindCPU = bag.cpu * 0.75;
            reqkickCPU = bag.cpu * 0.25;
        }
    }
    var registryUrl;
    if (global.config.isSaas) {
        registryUrl = 'releases-docker.jfrog.io';
    }
    else {
        registryUrl = global.config.registryUrl;
    }
    var innerBag = {
        deploymentName: bag.nodeName + '-deployment',
        projectId: bag.projectId,
        nodePoolId: bag.nodePoolId,
        nodePool: bag.nodePool,
        nodeId: bag.nodeId,
        runtime: bag.runtime,
        buildPlaneMessageId: bag.buildPlaneMessageId,
        nodeName: bag.nodeName,
        nodeSelector: nodeSelector,
        pvcName: bag.pvc.name,
        secretName: bag.secret.name,
        dindParams: {
            image: util.format('%s/jfrog/pipelines-dind:%s', registryUrl, bag.startupScriptData.buildPlaneVersion),
            resources: {
                "requests": {
                    "memory": dindMemory,
                    "cpu": JSON.stringify(dindCPU)
                },
                "limits": {
                    "memory": dindMemory,
                    "cpu": JSON.stringify(dindCPU)
                }
            }
        },
        reqkickParams: {
            image: util.format('%s/jfrog/pipelines-reqkick:%s', registryUrl, bag.startupScriptData.buildPlaneVersion),
            resources: {
                "requests": {
                    "memory": reqkickMemory,
                    "cpu": JSON.stringify(reqkickCPU)
                },
                "limits": {
                    "memory": reqkickMemory,
                    "cpu": JSON.stringify(reqkickCPU)
                }
            }
        },
        listenQueue: bag.startupScriptData.listenQueue,
        runMode: bag.startupScriptData.runMode,
        architecture: bag.startupScriptData.architecture,
        operatingSystem: bag.startupScriptData.operatingSystem,
        buildPlaneVersion: bag.startupScriptData.buildPlaneVersion,
        apiExternalUrl: bag.startupScriptData.apiExternalUrl,
        wwwUrl: bag.startupScriptData.wwwUrl,
        baseUrlUI: bag.startupScriptData.baseUrlUI,
        token: bag.startupScriptData.token,
        nodePollerIntervalMS: bag.startupScriptData.nodePollerIntervalMS,
        customCertsEnabled: bag.customCertsEnabled,
        imagePullSecret: global.config.imageRegistrySecret,
        disableDind: bag.disableDind
    };
    if (!_.isEmpty(bag.dockerRegistrySecretContent) && bag.dockerRegistrySecret)
        innerBag.dockerRegistrySecretName = bag.dockerRegistrySecret.name;
    if (!_.isEmpty(bag.configMapName))
        innerBag.configMapName = bag.configMapName;
    if (bag.customCertsEnabled)
        innerBag.customCertsConfigMapName = bag.customCertsConfigMapName;
    if (bag.nodePool?.isBeta && innerBag.runtime) {
        bag.consoleAdapter.openCmd('Creating Beta job ' + innerBag.deploymentName);
        betaJobSpec(innerBag, function (err, deployment) {
            if (err) {
                bag.errorForMetric = 'JobGenerationError';
                bag.consoleAdapter.publishMsg('Failed to generate job spec with error: ' + err);
                bag.consoleAdapter.closeCmd(false);
                logger.warn(who, 'Failed to generate job spec', err);
                bag.isSuccess = false;
                return next();
            }
            bag.deployment = deployment;
            logger.trace(util.format('%s generated job spec', who));
            return next();
        });
    }
    else {
        bag.consoleAdapter.openCmd('Creating Deployment ' + innerBag.deploymentName);
        deploymentSpec(innerBag, function (err, deployment) {
            if (err) {
                bag.errorForMetric = 'DeploymentGenerationError';
                bag.consoleAdapter.publishMsg('Failed to generate deployment spec with error: ' + err);
                bag.consoleAdapter.closeCmd(false);
                logger.warn(who, 'Failed to generate deployment spec', err);
                bag.isSuccess = false;
                return next();
            }
            bag.deployment = deployment;
            logger.trace(util.format('%s generated deployment spec', who));
            return next();
        });
    }
}
function _createDeployment(bag, next) {
    if (!bag.isSuccess)
        return next();
    if (bag.action !== nodeActions.CREATE)
        return next();
    var who = bag.who + '|' + _createDeployment.name;
    logger.trace(who, 'Inside');
    bag.kubernetesAdapter.apply(bag.namespace, bag.deployment.name, JSON.stringify(bag.deployment.spec), function (err, stdOut, errOut) {
        if (err) {
            bag.errorForMetric = 'DeploymentCreationError';
            const message = util.format('Failed to create deployment for node: %s - %s %s %s %s', bag.nodeId, bag.nodeName, err, stdOut, errOut);
            bag.consoleAdapter.publishMsg(message);
            bag.consoleAdapter.closeCmd(false);
            logger.warn(who, message);
            bag.isSuccess = false;
            return next();
        }
        bag.instance.pvc = bag.pvc;
        bag.instance.deployment = bag.deployment;
        bag.consoleAdapter.publishMsg('Deployment created ' +
            bag.deployment.name);
        bag.consoleAdapter.closeCmd(true);
        return next();
    });
}
function _scaleDownDeployment(bag, next) {
    if (!bag.isSuccess)
        return next();
    if (bag.action !== nodeActions.CACHE)
        return next();
    var who = bag.who + '|' + _scaleDownDeployment.name;
    logger.trace(who, 'Inside');
    bag.consoleAdapter.openCmd('Stopping instance');
    var scaleObject = {
        name: util.format('deployment/%s-deployment', bag.instanceId),
        replicas: 0
    };
    bag.kubernetesAdapter.scale(scaleObject, bag.namespace, function (err, stdOut, errOut) {
        if (err) {
            bag.errorForMetric = 'ScaleDownError';
            const message = util.format('Failed to scale down deployment for node: %s - %s %s %s %s', bag.nodeId, bag.nodeName, err, stdOut, errOut);
            bag.consoleAdapter.publishMsg(message);
            bag.consoleAdapter.closeCmd(false);
            logger.warn(who, message);
            bag.isSuccess = false;
            return next();
        }
        bag.consoleAdapter.publishMsg('Instance stop requested');
        bag.consoleAdapter.closeCmd(true);
        return next();
    });
}
function _checkIfDeploymentIsStopped(bag, next) {
    if (!bag.isSuccess)
        return next();
    if (bag.action !== nodeActions.START)
        return next();
    var who = bag.who + '|' + _checkIfDeploymentIsStopped.name;
    logger.trace(who, 'Inside');
    var objectName = 'pods';
    var labels = util.format('node-name=%s', bag.nodeName);
    async.retry({
        times: 10,
        interval: 3000
    }, function (callback) {
        bag.kubernetesAdapter.get(objectName, bag.namespace, 'json', labels, function (err, stdout) {
            var canStartDeployment = false;
            if (err) {
                bag.errorForMetric = 'GetPodsError';
                logger.warn(who, 'Failed to get pods for node: ' + bag.nodeId +
                    ' and name: ' + bag.nodeName, err);
                return callback(null, canStartDeployment);
            }
            logger.trace(who, 'Successfully fetched pods for node: ' +
                bag.nodeId + ' node name: ' + bag.nodeName);
            var pods;
            try {
                pods = JSON.parse(stdout);
            }
            catch (e) {
                logger.trace(who, 'Unable to parse stdout', stdout, e);
                return callback(null, canStartDeployment);
            }
            if (!_.isEmpty(pods.items))
                return callback(true, canStartDeployment);
            canStartDeployment = true;
            return callback(null, canStartDeployment);
        });
    }, function (err, canStartDeployment) {
        if (err || !canStartDeployment) {
            bag.consoleAdapter.openCmd('Getting Pods status');
            if (err) {
                bag.errorForMetric = 'TerminatePodsError';
                bag.consoleAdapter.publishMsg('Failed to terminate pods for ' +
                    'nodeId: ' + bag.nodeId);
                logger.warn(who, util.format('Retry failed to terminate pods for node: %s ', bag.nodeId), err);
            }
            else if (!canStartDeployment) {
                bag.consoleAdapter.publishMsg('Failed to get pods status for ' +
                    'nodeId: ' + bag.nodeId);
                logger.warn(who, util.format('Retry failed to get pods status for node: %s ', bag.nodeId), err);
            }
            bag.consoleAdapter.closeCmd(false);
        }
        bag.isSuccess = canStartDeployment;
        return next();
    });
}
function _scaleUpDeployment(bag, next) {
    if (!bag.isSuccess)
        return next();
    if (bag.action !== nodeActions.START)
        return next();
    var who = bag.who + '|' + _scaleUpDeployment.name;
    logger.trace(who, 'Inside');
    bag.consoleAdapter.openCmd('Starting instance');
    var scaleObject = {
        name: util.format('deployment/%s-deployment', bag.instanceId),
        replicas: 1
    };
    bag.kubernetesAdapter.scale(scaleObject, bag.namespace, function (err, stdOut, errOut) {
        if (err) {
            bag.errorForMetric = 'ScaleUpError';
            const message = util.format('Failed to scale up deployment for node: %s - %s %s %s %s', bag.nodeId, bag.nodeName, err, stdOut, errOut);
            bag.consoleAdapter.publishMsg(message);
            bag.consoleAdapter.closeCmd(false);
            logger.warn(who, message);
            bag.isSuccess = false;
            return next();
        }
        bag.consoleAdapter.publishMsg('Instance start requested');
        bag.consoleAdapter.closeCmd(true);
        return next();
    });
}
function _getDeployment(bag, next) {
    if (!bag.isSuccess)
        return next();
    if (bag.action !== nodeActions.DELETE)
        return next();
    if (_.isEmpty(bag.instanceId))
        return next();
    var who = bag.who + '|' + _getDeployment.name;
    logger.trace(who, 'Inside');
    var objectName = util.format('deployment/%s-deployment', bag.instanceId);
    if (bag.nodePool?.isBeta) {
        objectName = util.format('job/%s-deployment', bag.instanceId);
    }
    bag.kubernetesAdapter.get(objectName, bag.namespace, 'json', '', function (err, stdout, stderr) {
        if (err) {
            bag.errorForMetric = 'DeploymentNotFound';
            bag.isSuccess = false;
            if (stderr.includes('Error from server (NotFound)')) {
                bag.deploymentDeleted = true;
                bag.isSuccess = true;
            }
        }
        return next();
    });
}
function _deleteDeployment(bag, next) {
    if (!bag.isSuccess)
        return next();
    if (bag.action !== nodeActions.DELETE)
        return next();
    if (bag.deploymentDeleted)
        return next();
    if (_.isEmpty(bag.instanceId)) {
        logger.warn(util.format('Skipping termination of kubernetes pods for nodeId:%s ' +
            'as instanceId is empty', bag.nodeId));
        return next();
    }
    var who = bag.who + '|' + _deleteDeployment.name;
    logger.trace(who, 'Inside');
    var objectName = util.format('deployment/%s-deployment', bag.instanceId);
    if (bag.nodePool?.isBeta) {
        objectName = util.format('job/%s-deployment', bag.instanceId);
    }
    bag.kubernetesAdapter.delete(objectName, bag.namespace, function (err, stdOut, stdErr) {
        if (err) {
            bag.errorForMetric = 'PodsDeletionError';
            logger.warn(who, util.format('Failed to delete kubernetes pods %s for node: %s - %s', bag.instanceId, bag.nodeId, bag.nodeName), err, stdOut, stdErr);
            bag.isSuccess = false;
        }
        else {
            bag.deploymentDeleted = true;
        }
        return next();
    });
}
function _createDockerDaemonConfig(bag, next) {
    if (!bag.isSuccess)
        return next();
    if (bag.action !== nodeActions.CREATE)
        return next();
    if (bag.startupScriptData &&
        _.isEmpty(bag.startupScriptData.insecureDockerRegistries))
        return next();
    const who = bag.who + '|' + _createDockerDaemonConfig.name;
    logger.trace(who, 'Inside');
    const daemonConfig = JSON.stringify({
        'insecure-registries': bag.startupScriptData.insecureDockerRegistries,
    });
    bag.configMapName = bag.nodeName + '-docker-config';
    const fileName = 'daemon.json';
    bag.consoleAdapter.openCmd('Creating Docker Daemon Config');
    bag.consoleAdapter.publishMsg(`daemon config: ${daemonConfig}`);
    bag.kubernetesAdapter.createConfigMap(fileName, daemonConfig, bag.namespace, bag.configMapName, function (err, stdOut, errOut) {
        if (err) {
            bag.errorForMetric = 'DockerCreationFailed';
            const message = util.format('Failed to create docker daemon configMap for node: %s - %s %s %s %s', bag.nodeId, bag.nodeName, err, stdOut, errOut);
            bag.consoleAdapter.publishMsg(message);
            bag.consoleAdapter.closeCmd(false);
            logger.warn(who, message);
            bag.isSuccess = false;
            return next();
        }
        bag.consoleAdapter.publishMsg('Docker daemon config created');
        bag.consoleAdapter.closeCmd(true);
        return next();
    });
}
function _getDockerDaemonConfig(bag, next) {
    if (!bag.isSuccess)
        return next();
    if (bag.action !== nodeActions.DELETE)
        return next();
    if (_.isEmpty(bag.instanceId))
        return next();
    const who = bag.who + '|' + _getDockerDaemonConfig.name;
    logger.trace(who, 'Inside');
    const configMapObjectName = util.format('configMap/%s-docker-config', bag.instanceId);
    bag.kubernetesAdapter.get(configMapObjectName, bag.namespace, 'json', '', function (err, stdout, stderr) {
        if (err) {
            bag.errorForMetric = 'DockerNotFound';
            bag.isSuccess = false;
            if (stderr.includes('Error from server (NotFound)')) {
                bag.dockerDaemonConfigDeleted = true;
                bag.isSuccess = true;
            }
        }
        return next();
    });
}
function _deleteDockerDaemonConfig(bag, next) {
    if (!bag.isSuccess)
        return next();
    if (bag.action !== nodeActions.DELETE)
        return next();
    if (bag.dockerDaemonConfigDeleted)
        return next();
    if (_.isEmpty(bag.instanceId))
        return next();
    const who = bag.who + '|' + _deleteDockerDaemonConfig.name;
    logger.trace(who, 'Inside');
    const configMapObjectName = util.format('configMap/%s-docker-config', bag.instanceId);
    bag.kubernetesAdapter.delete(configMapObjectName, bag.namespace, function (err, stdOut, errOut) {
        if (err) {
            bag.errorForMetric = 'DockerDaemonDeleteFailed';
            logger.warn(who, util.format('Failed to delete docker daemon configmap %s for node: %s - %s %s ' +
                '%s %s', bag.instanceId, bag.nodeId, bag.nodeName, err, stdOut, errOut));
            bag.isSuccess = false;
        }
        return next();
    });
}
function _getSecret(bag, next) {
    if (!bag.isSuccess)
        return next();
    if (bag.action !== nodeActions.DELETE)
        return next();
    if (_.isEmpty(bag.instanceId))
        return next();
    var who = bag.who + '|' + _getSecret.name;
    logger.trace(who, 'Inside');
    var objectName = util.format('secret/%s-secret', bag.instanceId);
    bag.kubernetesAdapter.get(objectName, bag.namespace, 'json', '', function (err, stdout, stderr) {
        if (err) {
            bag.errorForMetric = 'SecretNotFound';
            bag.isSuccess = false;
            if (stderr.includes('Error from server (NotFound)')) {
                bag.secretDeleted = true;
                bag.isSuccess = true;
            }
        }
        return next();
    });
}
function _deleteSecret(bag, next) {
    if (!bag.isSuccess)
        return next();
    if (bag.action !== nodeActions.DELETE)
        return next();
    if (bag.secretDeleted)
        return next();
    if (_.isEmpty(bag.instanceId))
        return next();
    var who = bag.who + '|' + _deleteSecret.name;
    logger.trace(who, 'Inside');
    var objectName = util.format('secret/%s-secret', bag.instanceId);
    bag.kubernetesAdapter.delete(objectName, bag.namespace, function (err, stdOut, errOut) {
        if (err) {
            bag.errorForMetric = 'KubernetesSecretDeletionFailed';
            logger.warn(who, util.format('Failed to delete kubernetes secret %s for node: %s - %s %s %s %s', bag.instanceId, bag.nodeId, bag.nodeName, err, stdOut, errOut));
            bag.isSuccess = false;
        }
        else {
            bag.secretDeleted = true;
        }
        return next();
    });
}
function _getDockerRegistrySecret(bag, next) {
    if (bag.nodePool && !bag.nodePool.isBeta)
        return next();
    if (!bag.isSuccess)
        return next();
    if (bag.action !== nodeActions.DELETE)
        return next();
    if (_.isEmpty(bag.instanceId))
        return next();
    var who = bag.who + '|' + _getSecret.name;
    logger.trace(who, 'Inside');
    var objectName = util.format('secret/%s-docker-reg-secret', bag.instanceId);
    bag.kubernetesAdapter.get(objectName, bag.namespace, 'json', '', function (err, stdout, stderr) {
        if (err) {
            bag.errorForMetric = 'SecretNotFound';
            if (stderr.includes('Error from server (NotFound)')) {
                bag.dockerRegistrySecretDeleted = true;
            }
        }
        return next();
    });
}
function _deleteDockerRegistrySecret(bag, next) {
    if (bag.nodePool && !bag.nodePool.isBeta)
        return next();
    if (!bag.isSuccess)
        return next();
    if (bag.action !== nodeActions.DELETE)
        return next();
    if (bag.dockerRegistrySecretDeleted)
        return next();
    if (_.isEmpty(bag.instanceId))
        return next();
    var who = bag.who + '|' + _deleteDockerRegistrySecret.name;
    logger.trace(who, 'Inside');
    var objectName = util.format('secret/%s-docker-reg-secret', bag.instanceId);
    bag.kubernetesAdapter.delete(objectName, bag.namespace, function (err, stdOut, errOut) {
        if (err) {
            bag.errorForMetric = 'KubernetesSecretDeletionFailed';
            logger.warn(who, util.format('Failed to delete kubernetes docker registry secret %s for node: %s - %s %s %s %s', bag.instanceId, bag.nodeId, bag.nodeName, err, stdOut, errOut));
        }
        return next();
    });
}
function _getPVC(bag, next) {
    if (!bag.isSuccess)
        return next();
    if (bag.action !== nodeActions.DELETE)
        return next();
    if (_.isEmpty(bag.instanceId))
        return next();
    if (bag.nodePool.isBeta && bag.skipPVCDelete === true)
        return next();
    var who = bag.who + '|' + _getPVC.name;
    logger.trace(who, 'Inside');
    if (bag.nodePool.isBeta && (bag.pvcName?.length > 0)) {
        var objectName = util.format('pvc/%s', bag.pvcName);
    }
    else {
        var objectName = util.format('pvc/%s-pvc', bag.instanceId);
    }
    bag.kubernetesAdapter.get(objectName, bag.namespace, 'json', '', function (err, stdout, stderr) {
        if (err) {
            bag.errorForMetric = 'PVCNotFound';
            bag.isSuccess = false;
            if (stderr.includes('Error from server (NotFound)')) {
                bag.pvcDeleted = true;
                bag.isSuccess = true;
            }
        }
        return next();
    });
}
function _deletePVC(bag, next) {
    if (!bag.isSuccess)
        return next();
    if (bag.action !== nodeActions.DELETE)
        return next();
    if (bag.pvcDeleted)
        return next();
    if (bag.nodePool.isBeta && bag.skipPVCDelete === true)
        return next();
    if (_.isEmpty(bag.instanceId)) {
        logger.warn(util.format('Skipping termination of kubernetes pods for nodeId:%s ' +
            'as instanceId is empty', bag.nodeId));
        return next();
    }
    var who = bag.who + '|' + _deletePVC.name;
    logger.trace(who, 'Inside');
    if (bag.nodePool.isBeta && (bag.pvcName?.length > 0)) {
        var objectName = util.format('pvc/%s', bag.pvcName);
    }
    else {
        var objectName = util.format('pvc/%s-pvc', bag.instanceId);
    }
    bag.kubernetesAdapter.delete(objectName, bag.namespace, function (err, stdOut, errOut) {
        if (err) {
            bag.errorForMetric = 'PVCPodDeletionError';
            logger.warn(who, util.format('Failed to delete kubernetes pods %s for node: %s - %s', bag.instanceId, bag.nodeId, bag.nodeName), err, stdOut, errOut);
            bag.isSuccess = false;
        }
        else {
            bag.pvcDeleted = true;
        }
        return next();
    });
}
function _getCustomCertsConfig(bag, next) {
    if (!bag.isSuccess)
        return next();
    if (bag.action !== nodeActions.DELETE)
        return next();
    if (_.isEmpty(bag.instanceId))
        return next();
    const who = bag.who + '|' + _getCustomCertsConfig.name;
    logger.trace(who, 'Inside');
    const configMapObjectName = util.format('configMap/%s-custom-certs', bag.instanceId);
    bag.kubernetesAdapter.get(configMapObjectName, bag.namespace, 'json', '', function (err, stdout, stderr) {
        if (err) {
            bag.errorForMetric = 'CustomCertsNotFound';
            bag.isSuccess = false;
            if (stderr.includes('Error from server (NotFound)')) {
                bag.customCertsConfigDeleted = true;
                bag.isSuccess = true;
            }
        }
        return next();
    });
}
function _deleteCustomCertsConfig(bag, next) {
    if (!bag.isSuccess)
        return next();
    if (bag.action !== nodeActions.DELETE)
        return next();
    if (bag.customCertsConfigDeleted)
        return next();
    const who = bag.who + '|' + _deleteCustomCertsConfig.name;
    logger.trace(who, 'Inside');
    const configMapObjectName = util.format('configMap/%s-custom-certs', bag.instanceId);
    bag.kubernetesAdapter.delete(configMapObjectName, bag.namespace, function (err, stdOut, errOut) {
        if (err) {
            bag.errorForMetric = 'CustomCertsDeletionFailed';
            logger.warn(who, util.format('Failed to delete custom certs configmap %s for node: %s - %s', bag.instanceId, bag.nodeId, bag.nodeName), err, stdOut, errOut);
            bag.isSuccess = false;
        }
        return next();
    });
}
function _removeTmpDirWithContents(bag, next) {
    if (!bag.isSuccess)
        return next();
    var who = bag.who + '|' + _removeTmpDirWithContents.name;
    logger.trace(who, 'Inside');
    bag.kubernetesAdapter.destroy(function (err) {
        if (err) {
            bag.errorForMetric = 'TmpDirectoryDeletionFailed';
            logger.warn(who, util.format('Failed to delete tmp directory %s for node: %s', bag.instanceId, bag.nodeId), err);
            bag.isSuccess = false;
        }
        return next();
    });
}
//# sourceMappingURL=handler.js.map