'use strict';
var self = microWorker;
module.exports = self;
var uuid = require('node-uuid');
var Adapter = require('./Adapter.js');
var nodeActions = require('./nodeActions.js');
var getValuesFromIntegrationJson = require('./getValuesFromIntJson.js');
var NodeConsoleAdapter = require('./NodeConsoleAdapter.js');
var updateUrlSettings = require('./updateUrlSettings.js');
var { LoggerFactory } = require('@jfrog-pipelines/logger');
const util = require('util');
var prepData = {};
prepData[nodeActions.CREATE] = require('./_common/createNode.js');
prepData[nodeActions.DELETE] = require('./_common/deleteNode.js');
prepData[nodeActions.CACHE] = require('./_common/cacheNode.js');
prepData[nodeActions.START] = require('./_common/startNode.js');
var providerMap = {
    'amazonKeys': require('./_common/ec2/handler.js'),
    'gcloudKey': require('./_common/gce/handler.js'),
    'azureKeys': require('./_common/az/handler.js'),
    'kubernetesConfig': require('./_common/k8s/handler.js')
};
const providerNameMap = {
    'amazonKeys': 'AWS',
    'gcloudKey': 'GoogleCloud',
    'azureKeys': 'Azure',
    'kubernetesConfig': 'K8s'
};
function microWorker(message, callback) {
    var bag = {
        inPayload: message.payload,
        runtime: message.payload.runtime,
        pvcName: message.payload.pvcName,
        skipPVCDelete: message.payload.skipPVCDelete,
        buildPlaneMessageId: message.payload.buildPlaneMessageId,
        jFrogPipelinesSessionId: message.payload &&
            message.payload.jFrogPipelinesSessionId || uuid.v4(),
        outPayload: {}
    };
    const startTime = Date.now();
    bag.who = util.format('nod|marshaller|%s|jFrogPipelinesSessionId:%s', self.name, bag.jFrogPipelinesSessionId);
    logger.info(bag.who, 'Started');
    async.series([
        _checkInputParams.bind(null, bag),
        _updateUrlSettings.bind(null, bag),
        _getNodeById.bind(null, bag),
        _getNodePool.bind(null, bag),
        _getProjectById.bind(null, bag),
        _getProjectIntegrationById.bind(null, bag),
        _getBuildPlaneImageById.bind(null, bag),
        _generateHandlerParams.bind(null, bag),
        _handleNodeRequest.bind(null, bag),
        _resetBuildPlaneMessage.bind(null, bag),
        _updateNodeStatus.bind(null, bag),
        _updateNodeInfo.bind(null, bag),
        _clearInstanceId.bind(null, bag),
        _deleteNode.bind(null, bag)
    ], function (err) {
        if (err)
            logger.warn(bag.who, util.format('Failed to marshall for action %s', bag.inPayload.action));
        else
            logger.debug(bag.who, util.format('Succeeded marshalling projectId: %s for action %s', bag.node.projectId, bag.inPayload.action));
        const stopTime = Date.now();
        const timeDiff = new Date(stopTime - startTime);
        const eventData = {
            timestamp: new Date().toISOString(),
            domain: providerNameMap[bag.handlerParams?.providerName] || undefined,
            status: bag.isSuccess ? 'success' : 'failed',
            durationMillis: timeDiff.valueOf(),
            pipelinesSessionId: bag.jFrogPipelinesSessionId,
            action: nodeActions.lookup(bag.inPayload?.action) || undefined,
            nodeId: bag.inPayload?.nodeId || undefined,
            nodeName: bag.node?.friendlyName || undefined,
            nodePool: bag.nodePool?.name || undefined,
            projectId: bag.project?.id || undefined,
            projectName: bag.project?.name || undefined,
            error: bag.errorForMetric || undefined
        };
        LoggerFactory.getEventLogger().info(eventData);
        callback();
    });
}
function _checkInputParams(bag, next) {
    var who = bag.who + '|' + _checkInputParams.name;
    logger.debug(who, 'Inside');
    var consoleErrors = [];
    bag.suAdapter = new Adapter(bag.jFrogPipelinesSessionId, global.config.apiToken);
    if (!bag.inPayload)
        consoleErrors.push(util.format('%s is missing: inPayload', who));
    if (!_.isNumber(bag.inPayload.action))
        consoleErrors.push(util.format('%s is missing: inPayload.action', who));
    if (!bag.inPayload.nodeId)
        consoleErrors.push(util.format('%s is missing: inPayload.nodeId', who));
    if (consoleErrors.length > 0) {
        _.each(consoleErrors, function (e) {
            logger.warn(bag.who, e);
        });
        return next(true);
    }
    else
        return next();
}
function _updateUrlSettings(bag, next) {
    var who = bag.who + '|' + _updateUrlSettings.name;
    logger.debug(who, 'Inside');
    updateUrlSettings(bag, function (err) {
        if (err) {
            logger.warn(who, 'Failed to update url settings.', err);
            return next(true);
        }
        return next();
    });
}
function _getNodeById(bag, next) {
    var who = bag.who + '|' + _getNodeById.name;
    logger.debug(who, 'Inside');
    bag.suAdapter.getNodeById(bag.inPayload.nodeId, function (err, node) {
        if (err) {
            logger.warn(who, 'Failed getNodeById for id: ' +
                bag.inPayload.nodeId, err);
            return next(true);
        }
        bag.node = node;
        logger.trace(util.format('%s got node with nodeId: %s', who, bag.node.id));
        bag.consoleAdapter = new NodeConsoleAdapter(bag.jFrogPipelinesSessionId, bag.node.id, bag.node.projectId);
        return next();
    });
}
function _getProjectById(bag, next) {
    var who = bag.who + '|' + _getProjectById.name;
    logger.debug(who, 'Inside');
    bag.suAdapter.getProjectById(bag.node.projectId, function (err, project) {
        if (err) {
            logger.warn(who, 'Failed getProjectById for id: ' +
                bag.node.projectId, err);
            return next(true);
        }
        bag.project = project;
        logger.trace(util.format('%s found project with projectId: %s', who, project.id));
        return next();
    });
}
function _getNodePool(bag, next) {
    var who = bag.who + '|' + _getNodePool.name;
    logger.debug(who, 'Inside');
    var query = util.format('nodePoolIds=%s', bag.node.nodePoolId);
    bag.suAdapter.getNodePools(query, function (err, nodePools) {
        if (err) {
            logger.warn(who, 'Failed getNodePools for id: ' +
                bag.node.nodePoolId, err);
            return next(true);
        }
        bag.nodePool = nodePools[0];
        if (!bag.nodePool)
            return next(new ActErr(who, ActErr.DataNotFound, 'NodePool not found for id: ' + bag.node.nodePoolId));
        return next();
    });
}
function _getProjectIntegrationById(bag, next) {
    var who = bag.who + '|' + _getProjectIntegrationById.name;
    logger.debug(who, 'Inside');
    bag.suAdapter.getProjectIntegrationById(bag.node.projectIntegrationId, function (err, projectIntegration) {
        if (err) {
            logger.warn(who, 'Failed to get projectIntegrationId for id: ' +
                bag.nodePool.projectIntegrationId, err);
            return next(true);
        }
        logger.trace(util.format('%s got projectIntegration with id: %s', who, projectIntegration.id));
        var integrationValues = getValuesFromIntegrationJson(projectIntegration.formJSONValues);
        if (projectIntegration.masterIntegrationName === 'amazonKeys' ||
            projectIntegration.masterIntegrationName === 'gcloudKey' ||
            projectIntegration.masterIntegrationName === 'azureKeys' ||
            projectIntegration.masterIntegrationName === 'kubernetesConfig') {
            bag.integrationValues = integrationValues;
        }
        else {
            logger.warn(who, 'Unsupported integration type ' +
                projectIntegration.masterIntegrationName);
            return next(true);
        }
        bag.providerName = projectIntegration.masterIntegrationName;
        return next();
    });
}
function _getBuildPlaneImageById(bag, next) {
    if (bag.inPayload.action !== nodeActions.CREATE)
        return next();
    if (bag.providerName === 'kubernetesConfig')
        return next();
    var who = bag.who + '|' + _getBuildPlaneImageById.name;
    logger.debug(who, 'Inside');
    var query = 'buildPlaneImageIds=' + bag.nodePool.buildPlaneImageId;
    bag.suAdapter.getBuildPlaneImages(query, function (err, bpis) {
        if (err) {
            logger.warn(who, 'Failed buildPlaneImageId for id: ' +
                bag.node.buildPlaneImageId, err);
            return next(true);
        }
        bag.buildPlaneImage = bpis[0];
        if (!bag.buildPlaneImage)
            return next(new ActErr(who, ActErr.DataNotFound, 'Build plane image not found for id: ' +
                bag.nodePool.buildPlaneImageId));
        logger.trace(util.format('%s got buildPlaneImage with id: %s', who, bag.buildPlaneImage.id));
        return next();
    });
}
function _generateHandlerParams(bag, next) {
    var who = self.name + '|' + _generateHandlerParams.name;
    logger.debug(who, 'Inside');
    var action = bag.inPayload.action;
    var genHandlerParam = prepData[action];
    if (!genHandlerParam) {
        logger.warn(who, 'No strategy found for action', action);
        return next(true);
    }
    genHandlerParam(bag, function (err, handlerParams) {
        if (err) {
            logger.warn(who, 'Failed to process action', action, 'for project', bag.node.projectId, err);
            return next(true);
        }
        bag.handlerParams = handlerParams;
        bag.handlerParams.nodePool = bag.nodePool;
        bag.handlerParams.jFrogPipelinesSessionId = bag.jFrogPipelinesSessionId;
        logger.trace(util.format('%s created outPayload object for action: %s', who, handlerParams.action));
        return next();
    });
}
function _handleNodeRequest(bag, next) {
    var who = self.name + '|' + _handleNodeRequest.name;
    logger.debug(who, 'Inside');
    if (bag.inPayload.action === nodeActions.DELETE) {
        var skipHandler = false;
        if (bag.providerName === 'azureKeys') {
            var pb = bag.node.providerMetadataPropertyBag || {};
            var vmName = pb.vmName;
            var nicName = pb.nicName;
            if (_.isEmpty(bag.node.instanceInfo) && !vmName && !nicName)
                skipHandler = true;
        }
        else if (bag.providerName === 'kubernetesConfig') {
            if (_.isEmpty(bag.node.instanceInfo) &&
                _.isEmpty(bag.node.providerMetadataPropertyBag))
                skipHandler = true;
        }
        else if (bag.providerName === 'amazonKeys' &&
            bag.node.providerMetadataPropertyBag &&
            bag.node.providerMetadataPropertyBag.operatingSystem === 'MacOS_10.15') {
            if (_.isEmpty(bag.node.instanceInfo))
                logger.trace(who, 'Not skipping cloud provider:', bag.providerName, 'handler to check for dedicated host on the cloud provider');
        }
        else {
            if (_.isEmpty(bag.node.instanceInfo))
                skipHandler = true;
        }
        if (skipHandler) {
            logger.trace(who, 'Skipping cloud provider: ' + bag.providerName +
                ' handler since no resources were created on the cloud provider');
            bag.isSuccess = true;
            return next();
        }
    }
    if (bag.providerName === 'kubernetesConfig') {
        bag.handlerParams.pvcName = bag.pvcName;
        bag.handlerParams.skipPVCDelete = bag.skipPVCDelete;
    }
    var handler = providerMap[bag.handlerParams.providerName];
    handler(bag.handlerParams, function (err, result) {
        if (err)
            logger.error(who, 'error in handler ' +
                bag.handlerParams.providerName, err);
        bag = _.extend(bag, result);
        return next();
    });
}
function _resetBuildPlaneMessage(bag, next) {
    if (bag.inPayload.action !== nodeActions.DELETE
        && bag.inPayload.action !== nodeActions.CACHE)
        return next();
    var who = self.name + '|' + _resetBuildPlaneMessage.name;
    logger.debug(who, 'Inside');
    const innerBag = {
        nodeId: bag.inPayload.nodeId,
        suAdapter: bag.suAdapter
    };
    async.series([
        __getBuildPlaneMessageByNodeId.bind(null, innerBag),
        __resetBuildPlaneMessageByNodeId.bind(null, innerBag)
    ], function (err) {
        return next(err);
    });
}
function __getBuildPlaneMessageByNodeId(bag, next) {
    var who = self.name + '|' + __getBuildPlaneMessageByNodeId.name;
    logger.debug(who, 'Inside');
    const buildPlaneMessageStatusCodes = [global.statusCodes.READY, global.statusCodes.QUEUED];
    const query = `nodeIds=${bag.nodeId}&statusCodes=${buildPlaneMessageStatusCodes.join(',')}`;
    bag.suAdapter.getBuildPlaneMessages(query, function (err, bpms) {
        if (err) {
            logger.warn(who, util.format('Failed to get buildplane messages status for :id %s', bag.nodeId), err);
            return next(true);
        }
        if (_.isEmpty(bpms)) {
            return next();
        }
        bag.buildPlaneMessage = bpms[0];
        return next();
    });
}
function __resetBuildPlaneMessageByNodeId(bag, next) {
    if (_.isEmpty(bag.buildPlaneMessage))
        return next();
    var who = self.name + '|' + __resetBuildPlaneMessageByNodeId.name;
    logger.debug(who, 'Inside');
    const update = {
        resetBuildPlaneMessage: true,
        nodeId: bag.nodeId
    };
    bag.suAdapter.putBuildPlaneMessageById(bag.buildPlaneMessage.id, update, function (err) {
        if (err) {
            logger.warn(who, util.format('Failed to reset buildplane messages for :id %s', bag.buildPlaneMessage.id), err);
            return next(true);
        }
        logger.warn(`Successfully reseted buildPlaneMessage with id:${bag.buildPlaneMessage.id}`);
        return next();
    });
}
function _updateNodeStatus(bag, next) {
    if (bag.providerName === 'azureKeys' &&
        bag.inPayload.action === nodeActions.DELETE &&
        bag.deletingVM)
        return next();
    if (bag.providerName === 'kubernetesConfig' &&
        bag.inPayload.action === nodeActions.DELETE &&
        !bag.k8sObjectsDeleted)
        return next();
    var who = bag.who + '|' + _updateNodeStatus.name;
    logger.trace(who, 'Inside');
    var statusBody = {
        statusCode: null
    };
    if (bag.isSuccess && bag.inPayload.action === nodeActions.DELETE)
        statusBody.statusCode = global.statusCodes.DELETED;
    else if (bag.isSuccess && bag.inPayload.action === nodeActions.CACHE)
        statusBody.statusCode = global.statusCodes.CACHED;
    else if (bag.isSuccess && bag.inPayload.action === nodeActions.START)
        statusBody = {};
    else if (bag.host && bag.inPayload.action === nodeActions.CREATE)
        statusBody.statusCode = global.statusCodes.QUEUED;
    else if (bag.isSuccess)
        statusBody.statusCode = global.statusCodes.WAITING;
    else
        statusBody.statusCode = global.statusCodes.FAILED;
    bag.suAdapter.putNodeById(bag.inPayload.nodeId, statusBody, function (err, node) {
        if (err) {
            logger.warn(who, util.format('Failed to update node status for :id %s', bag.inPayload.nodeId), err);
            return next(true);
        }
        logger.trace(who, util.format('Successfully updated node status for :id %s', bag.inPayload.nodeId));
        bag.node = node;
        return next();
    });
}
function _updateNodeInfo(bag, next) {
    if (bag.inPayload.action !== nodeActions.CREATE)
        return next();
    var who = bag.who + '|' + _updateNodeInfo.name;
    logger.trace(who, 'Inside');
    var providerMetadataPropertyBag = bag.node.providerMetadataPropertyBag || {};
    var update = {};
    if (bag.nodePool.diskSizeinGB)
        providerMetadataPropertyBag.diskSizeinGB = bag.nodePool.diskSizeinGB;
    if (bag.providerName === 'amazonKeys') {
        providerMetadataPropertyBag.region = bag.buildPlaneImage.region;
        providerMetadataPropertyBag.imageId = bag.buildPlaneImage.imageId;
        if (bag.instance) {
            update.instanceInfo = bag.instance;
            if (bag.inPayload.action === nodeActions.CREATE &&
                _.isEmpty(bag.instance.InstanceId))
                logger.warn(util.format('No instanceId found while initializing nodeId:%s', bag.inPayload.nodeId));
            if (bag.instance.InstanceId)
                update.instanceId = bag.instance.InstanceId;
            update.provisionedAt = Date.now();
        }
    }
    else if (bag.providerName === 'gcloudKey') {
        var nodePoolProviderMetadata = bag.nodePool.providerMetadataPropertyBag;
        providerMetadataPropertyBag.imageMetadata = {};
        providerMetadataPropertyBag.imageMetadata.imageName =
            bag.buildPlaneImage.metadataPropertyBag &&
                bag.buildPlaneImage.metadataPropertyBag.imageName;
        providerMetadataPropertyBag.imageMetadata.projectName =
            bag.buildPlaneImage.metadataPropertyBag &&
                bag.buildPlaneImage.metadataPropertyBag.projectName;
        providerMetadataPropertyBag.zone = nodePoolProviderMetadata.zone;
        if (bag.instance) {
            update.instanceInfo = {
                imageName: bag.buildPlaneImage.metadataPropertyBag.imageName,
                projectName: bag.buildPlaneImage.metadataPropertyBag.projectName,
                subnetName: nodePoolProviderMetadata.subnetName,
                networkTags: nodePoolProviderMetadata.networkTags,
                assignPublicIp: nodePoolProviderMetadata.assignPublicIp
            };
            update.instanceId = bag.node.friendlyName;
            update.provisionedAt = Date.now();
        }
    }
    else if (bag.providerName === 'azureKeys') {
        providerMetadataPropertyBag.subscriptionId =
            bag.nodePool.providerMetadataPropertyBag &&
                bag.nodePool.providerMetadataPropertyBag.subscriptionId;
        providerMetadataPropertyBag.resourceGroupName =
            bag.nodePool.providerMetadataPropertyBag &&
                bag.nodePool.providerMetadataPropertyBag.resourceGroupName;
        if (bag.instance) {
            update.instanceInfo = bag.instance;
            update.instanceId = bag.instance.vmId;
            update.provisionedAt = Date.now();
            _.extend(providerMetadataPropertyBag, {
                subnetName: bag.nodePool.providerMetadataPropertyBag.subnetName,
                securityGroupName: bag.nodePool.providerMetadataPropertyBag.securityGroupName,
                vnetName: bag.nodePool.providerMetadataPropertyBag.vnetName,
                publicSshKey: bag.nodePool.providerMetadataPropertyBag.publicSshKey,
                adminUsername: bag.nodePool.providerMetadataPropertyBag.adminUsername,
                adminPassword: bag.nodePool.providerMetadataPropertyBag.adminPassword,
                imageId: bag.buildPlaneImage.imageId,
                imageMetadata: bag.buildPlaneImage.metadataPropertyBag
            });
        }
        if (bag.componentsCreated) {
            providerMetadataPropertyBag = _.extend(providerMetadataPropertyBag, {
                vmName: bag.componentsCreated.vmName,
                nicName: bag.componentsCreated.nicName
            });
        }
        if (bag.tags)
            providerMetadataPropertyBag = _.extend(providerMetadataPropertyBag, { tags: bag.tags });
    }
    else if (bag.providerName === 'kubernetesConfig') {
        providerMetadataPropertyBag.namespace =
            bag.nodePool.providerMetadataPropertyBag &&
                bag.nodePool.providerMetadataPropertyBag.namespace;
        providerMetadataPropertyBag.storageClass =
            bag.nodePool.providerMetadataPropertyBag &&
                bag.nodePool.providerMetadataPropertyBag.storageClass;
        providerMetadataPropertyBag.cpu =
            bag.nodePool.providerMetadataPropertyBag &&
                bag.nodePool.providerMetadataPropertyBag.cpu;
        providerMetadataPropertyBag.memory =
            bag.nodePool.providerMetadataPropertyBag &&
                bag.nodePool.providerMetadataPropertyBag.memory;
        providerMetadataPropertyBag.labels =
            bag.nodePool.providerMetadataPropertyBag &&
                bag.nodePool.providerMetadataPropertyBag.labels;
        providerMetadataPropertyBag.operatingSystem = bag.nodePool.operatingSystem;
        if (!_.isEmpty(bag.instance)) {
            update.instanceInfo = bag.instance;
            update.instanceId = bag.node.friendlyName;
            update.provisionedAt = Date.now();
        }
    }
    update.providerMetadataPropertyBag = providerMetadataPropertyBag;
    bag.suAdapter.putNodeById(bag.node.id, update, function (err) {
        if (err) {
            logger.warn(who, util.format('Failed to update node for :id %s', bag.node.id), err);
            return next(true);
        }
        logger.trace(who, util.format('Successfully updated node for :id %s', bag.node.id));
        return next();
    });
}
function _clearInstanceId(bag, next) {
    if (!bag.instanceTerminated)
        return next();
    if (bag.node.statusCode === global.statusCodes.DELETED)
        return next();
    var who = bag.who + '|' + _clearInstanceId.name;
    logger.trace(who, 'Inside');
    var update = {
        instanceId: null
    };
    bag.suAdapter.putNodeById(bag.node.id, update, function (err) {
        if (err) {
            logger.warn(who, util.format('Failed to update node for :id %s', bag.node.id), err);
            return next(true);
        }
        logger.trace(who, util.format('Successfully updated node for :id %s', bag.node.id));
        return next();
    });
}
function _deleteNode(bag, next) {
    if (bag.node.statusCode !== global.statusCodes.DELETED)
        return next();
    var who = bag.who + '|' + _deleteNode.name;
    logger.trace(who, 'Inside');
    bag.suAdapter.deleteNodeById(bag.node.id, function (err) {
        if (err) {
            logger.warn(who, util.format('Failed to delete node for :id %s', bag.node.id), err);
            return next(true);
        }
        logger.trace(who, util.format('Successfully deleted node for :id %s', bag.node.id));
        return next();
    });
}
//# sourceMappingURL=microWorker.js.map