"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.NodeCollector = void 0;
const node_util_1 = require("node:util");
const utility_1 = require("@jfrog-pipelines/utility");
const logger_1 = require("@jfrog-pipelines/logger");
const statusCodes_1 = require("../marshaller/statusCodes");
const NodeService_1 = require("./node/NodeService");
const NODE_TIMEOUT = 15;
const ZERO = 0;
const ONE_MINUTE_MILLISECONDS = 60000;
const ONE_DAY_MILLISECONDS = 86400000;
class NodeCollector {
    constructor(publisher) {
        this.logger = logger_1.LoggerFactory.getLogger('NodeCollector');
        this.nodeService = new NodeService_1.NodeService(publisher);
    }
    async deProvision(nodePool, requiredNodes) {
        this.logger = logger_1.LoggerFactory.getLogger(`NodeCollector:${nodePool.id}${nodePool.name}`);
        try {
            const nodeDeProvisionStartTime = Date.now();
            const nodes = await this.getNodes(nodePool.id);
            this.logger.trace(`Active nodes are ${nodes.length}`);
            const idleNodes = this.getIdleNodes(nodePool, nodes);
            this.logger.trace(`idleNodes nodes are ${idleNodes.length}`);
            let nodesToDelete = this.computeDeleteState(nodePool, idleNodes);
            this.logger.trace(`Nodes to be deleted are ${nodesToDelete.length}`);
            nodesToDelete = this.filterRequiredNodes(nodesToDelete, requiredNodes);
            if (nodePool.isCacheEnabled === true) {
                const nodesToCache = this.computeCachedState(idleNodes, nodesToDelete);
                const cachedNodes = await this.applyCachedStateToNodes(nodesToCache);
                this.logger.info(`Cached ${cachedNodes.length} nodes`);
            }
            const deletedNodes = await this.applyDeleteStateToNodes(nodesToDelete);
            this.logger.info(`Deleted ${deletedNodes.length} nodes`);
            const nodeDeProvisionEndTime = Date.now();
            const nodeDeProvisionTimeDiff = new Date(nodeDeProvisionEndTime - nodeDeProvisionStartTime);
            const deProvisionedNodeData = {
                domain: 'nodepool',
                eventType: 'onNodeReclaimed',
                count: nodesToDelete.length,
                durationMillis: nodeDeProvisionTimeDiff.valueOf(),
                nodePoolName: nodePool.name
            };
            logger_1.LoggerFactory.logEvent(deProvisionedNodeData);
        }
        catch (error) {
            this.logger.warn(`Failed to deProvision nodes for node pool ${nodePool.id} due to err ${(0, node_util_1.inspect)(error)}`);
        }
        finally {
            this.logger.trace(`Node collection completed`);
        }
    }
    async collectJunkNodes() {
        try {
            const orphanNodes = await this.nodeService.getOrphanNodes();
            const collectedNodes = await this.applyDeleteStateToNodes(orphanNodes);
            this.logger.info(`Collected ${collectedNodes.length} orphan nodes.`);
            return collectedNodes.length;
        }
        catch (error) {
            this.logger.warn(`Failed to collectOrphan nodes due to err ${(0, node_util_1.inspect)(error)}`);
        }
        return ZERO;
    }
    filterIdleNodes(nodePool, nodes) {
        return nodes.filter((node) => {
            return this.isNodeIdle(nodePool, node);
        });
    }
    isNodeIdle(nodePool, node) {
        if (node.stopRequested || this.hasOlderBuildPlaneVersion(node)) {
            return true;
        }
        if (NodeService_1.unhealthyNodeStatuses.includes(node.statusCode)) {
            return true;
        }
        const nodeLastUpdatedAt = new Date(node.statusLastUpdatedAt).getTime();
        return (NodeService_1.idleNodeStatuses.includes(node.statusCode) &&
            Date.now() - nodeLastUpdatedAt >= nodePool.nodeIdleIntervalInMins * ONE_MINUTE_MILLISECONDS);
    }
    async getNodes(poolID) {
        return this.nodeService.getAllNodes(poolID);
    }
    filterRequiredNodes(nodes, requiredNodes) {
        const remainingNodes = [];
        let requiredNodeIndex = requiredNodes;
        for (const node of nodes) {
            if (node.statusCode === statusCodes_1.SUCCESS && requiredNodeIndex > 0) {
                requiredNodeIndex = requiredNodeIndex - 1;
                continue;
            }
            remainingNodes.push(node);
        }
        return remainingNodes;
    }
    getIdleNodes(nodePool, nodes) {
        return this.filterIdleNodes(nodePool, nodes);
    }
    computeDeleteState(nodePool, nodes) {
        return nodes.filter((node) => {
            if (node.statusCode === NodeService_1.NodeStatus.processing) {
                return false;
            }
            const shouldDelete = this.getNodeDeleteCondition(nodePool, node);
            return shouldDelete;
        });
    }
    getNodeDeleteCondition(nodePool, node) {
        if (this.isNodeInactive(node)) {
            return true;
        }
        if (this.hasNodeExpired(nodePool, node)) {
            return true;
        }
        if (this.hasCachedNodeExpired(nodePool, node)) {
            return true;
        }
        return false;
    }
    hasNodeExpired(nodePool, node) {
        if (this.hasOlderBuildPlaneVersion(node)) {
            return true;
        }
        const currentDateInMS = Date.now();
        const statusLastUpdatedAt = new Date(node.statusLastUpdatedAt).getTime();
        const isNodeCached = node.statusCode === NodeService_1.NodeStatus.cached;
        const isNodeExpired = this.isSuccessNodePassedIdleInterval(nodePool, node, currentDateInMS, statusLastUpdatedAt);
        const isNodePoolCached = this.isCachedEnabled(nodePool);
        if (!isNodePoolCached && (isNodeCached || isNodeExpired)) {
            return true;
        }
        return false;
    }
    hasCachedNodeExpired(nodePool, node) {
        const isNodePoolCached = this.isCachedEnabled(nodePool);
        if (isNodePoolCached && this.isCachedNodeInactive(nodePool, node)) {
            return true;
        }
        return false;
    }
    isNodeInactive(node) {
        if (node.stopRequested) {
            return true;
        }
        const CUTOFF_TIME = NODE_TIMEOUT * ONE_MINUTE_MILLISECONDS;
        const cutOffTimeForWaitingAt = Date.now() - CUTOFF_TIME;
        const statusLastUpdatedAt = new Date(node.statusLastUpdatedAt).getTime();
        if (node.statusCode === NodeService_1.NodeStatus.waiting && statusLastUpdatedAt < cutOffTimeForWaitingAt) {
            return true;
        }
        return [NodeService_1.NodeStatus.stopped, NodeService_1.NodeStatus.failure, NodeService_1.NodeStatus.deleted].includes(node.statusCode);
    }
    hasOlderBuildPlaneVersion(node) {
        return (!(0, utility_1.isEmpty)(node.currentBuildPlaneVersion) &&
            !(0, utility_1.isEmpty)(node.intendedBuildPlaneVersion) &&
            node.currentBuildPlaneVersion !== node.intendedBuildPlaneVersion);
    }
    isCachedNodeInactive(nodePool, node) {
        const dayOfWeekMap = {
            sunday: 0,
            monday: 1,
            tuesday: 2,
            wednesday: 3,
            thursday: 4,
            friday: 5,
            saturday: 6
        };
        const [currentDateInMS, currentDateOfMonth, currentMonthOfYear, currentYear, currentDayOfWeek] = this.getDateTuple();
        const nodeProvisionedAt = new Date(node.provisionedAt);
        const [provisionedAt, nodeProvisionedAtDate, nodeProvisionedAtMonth, nodeProvisionedAtYear] = this.getDateTuple(nodeProvisionedAt.toString());
        const daysPassedSinceProvision = (currentDateInMS - provisionedAt) / ONE_DAY_MILLISECONDS;
        const isProvisionedToday = this.checkIfNodeProvisionedToday(currentDateOfMonth, nodeProvisionedAtDate, currentMonthOfYear, nodeProvisionedAtMonth, currentYear, nodeProvisionedAtYear);
        if (!isProvisionedToday) {
            return true;
        }
        if (this.isNodeCachedForStopIntervalDays(nodePool, daysPassedSinceProvision)) {
            return true;
        }
        const nodeStopDayOfWeek = nodePool.cacheSettingsPropertyBag.nodeStopDayOfWeek;
        if (currentDayOfWeek === dayOfWeekMap[nodeStopDayOfWeek]) {
            return true;
        }
        return false;
    }
    checkIfNodeProvisionedToday(currentDateOfMonth, nodeProvisionedAtDate, currentMonthOfYear, nodeProvisionedAtMonth, currentYear, nodeProvisionedAtYear) {
        return (currentDateOfMonth === nodeProvisionedAtDate &&
            currentMonthOfYear === nodeProvisionedAtMonth &&
            currentYear === nodeProvisionedAtYear);
    }
    isCachedEnabled(nodePool) {
        return nodePool.isCacheEnabled === true && !(0, utility_1.isEmpty)(nodePool.cacheSettingsPropertyBag);
    }
    isNodeCachedForStopIntervalDays(nodePool, daysPassedSinceProvision) {
        return (nodePool.cacheSettingsPropertyBag.nodeStopSetting === `nodeStopIntervalDays` &&
            daysPassedSinceProvision >= nodePool.cacheSettingsPropertyBag.nodeStopIntervalDays);
    }
    isSuccessNodePassedIdleInterval(nodePool, node, currentDateInMS, statusLastUpdatedAt) {
        return (nodePool.isCacheEnabled === false &&
            node.statusCode === NodeService_1.NodeStatus.success &&
            currentDateInMS - statusLastUpdatedAt >= nodePool.nodeIdleIntervalInMins * ONE_MINUTE_MILLISECONDS);
    }
    computeCachedState(idleNodes, deletedNodes) {
        const ONE = 1;
        const nodes = new Map();
        for (let index = 0; index <= idleNodes.length - ONE; index++) {
            nodes.set(idleNodes[index].id, idleNodes[index]);
        }
        for (let index = 0; index <= deletedNodes.length - ONE; index++) {
            nodes.delete(idleNodes[index].id);
        }
        const tobeCachedNodes = [...nodes.values()];
        return tobeCachedNodes.filter((node) => {
            return node.statusCode === statusCodes_1.SUCCESS;
        });
    }
    async applyDeleteStateToNodes(nodes) {
        this.logger.trace(`Trying to deprovision ${nodes.length} nodes`);
        const deleteStatePromises = nodes.map((node) => {
            return this.nodeService.applyNodeDeletedState(node);
        });
        return Promise.all(deleteStatePromises);
    }
    async applyCachedStateToNodes(nodes) {
        this.logger.trace(`Trying to cache ${nodes.length} nodes`);
        const cachedStatePromises = nodes.map((node) => {
            return this.nodeService.applyNodeToCachedState(node);
        });
        return Promise.all(cachedStatePromises);
    }
    getDateTuple(inputDate) {
        const currentDate = inputDate ? new Date(inputDate) : new Date();
        const time = currentDate.getTime();
        const date = currentDate.getDate();
        const month = currentDate.getMonth();
        const year = currentDate.getFullYear();
        const week = currentDate.getDay();
        return [time, date, month, year, week];
    }
}
exports.NodeCollector = NodeCollector;
//# sourceMappingURL=NodeCollector.js.map