"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const uuid_1 = require("uuid");
const LockDao_1 = require("../LockDao");
const LockService_1 = require("../LockService");
jest.useFakeTimers();
jest.spyOn(global, 'setTimeout');
jest.mock('@jfrog-pipelines/logger', () => {
    const actualTracker = jest.requireActual('@jfrog-pipelines/logger');
    return {
        ...actualTracker,
        LoggerFactory: {
            getLogger: jest.fn().mockReturnValue({
                info: jest.fn(),
                error: jest.fn(),
                debug: jest.fn(),
                trace: jest.fn(),
                warn: jest.fn()
            }),
            configure: jest.fn()
        }
    };
});
const systemInstanceID = 1;
describe('Test LockService #obtainLock', () => {
    const TIMEOUT_DURATION = 15 * 60000;
    const lockTimeOutMs = Date.now() + TIMEOUT_DURATION;
    it('should throw an error when lock is already present and has not been timeout out', async () => {
        const lockService = new LockService_1.LockService(systemInstanceID);
        const dao = new LockDao_1.LockDao();
        const lockAttributes = {
            name: 'test',
            lockTimeoutInMs: lockTimeOutMs
        };
        const lock = { name: lockAttributes.name, timeoutAt: new Date(lockTimeOutMs) };
        const expectedError = new Error(`Lock already exists with name ${lockAttributes.name}`);
        const spyFindOne = jest.spyOn(LockDao_1.LockDao.prototype, 'findOne').mockResolvedValueOnce(lock);
        await expect(lockService.obtainLock(lockAttributes)).rejects.toThrow(expectedError);
        expect(dao.findOne).toHaveBeenCalled();
        spyFindOne.mockClear();
    });
    it('should get lock response back when lock creation is successful', async () => {
        const lockService = new LockService_1.LockService(systemInstanceID);
        const dao = new LockDao_1.LockDao();
        const lockAttributes = {
            name: 'test',
            lockTimeoutInMs: lockTimeOutMs
        };
        const expectedValue = { name: 'test', timeoutAt: new Date(lockTimeOutMs), systemServiceInstanceId: 12 };
        const spyFindOne = jest.spyOn(LockDao_1.LockDao.prototype, 'findOne').mockResolvedValueOnce(null);
        const spyCreate = jest.spyOn(LockDao_1.LockDao.prototype, 'create').mockResolvedValueOnce(expectedValue);
        const actual = await lockService.obtainLock(lockAttributes);
        expect(setTimeout).toHaveBeenCalled();
        expect(dao.findOne).toHaveBeenCalledTimes(1);
        expect(dao.create).toHaveBeenCalledTimes(1);
        expect(actual).toBe(expectedValue);
        spyFindOne.mockClear();
        spyCreate.mockClear();
    });
    it('should get lock response back when lock creation is successful when lock timeout is not provided', async () => {
        const lockService = new LockService_1.LockService(systemInstanceID);
        const dao = new LockDao_1.LockDao();
        const lockAttributes = {
            name: 'test'
        };
        const expectedValue = { name: 'test', timeoutAt: new Date(LockService_1.LockTimeoutInMs), systemServiceInstanceId: 12 };
        const spyFindOne = jest.spyOn(LockDao_1.LockDao.prototype, 'findOne').mockResolvedValueOnce(null);
        const spyCreate = jest.spyOn(LockDao_1.LockDao.prototype, 'create').mockResolvedValueOnce(expectedValue);
        const actual = await lockService.obtainLock(lockAttributes);
        expect(setTimeout).toHaveBeenCalled();
        expect(dao.findOne).toHaveBeenCalledTimes(1);
        expect(dao.create).toHaveBeenCalledTimes(1);
        expect(actual).toBe(expectedValue);
        spyFindOne.mockClear();
        spyCreate.mockClear();
    });
    it('should throw error when lock creation is failed', async () => {
        const lockService = new LockService_1.LockService(systemInstanceID);
        const dao = new LockDao_1.LockDao();
        const lockAttributes = {
            name: 'test',
            lockTimeoutInMs: lockTimeOutMs
        };
        const spyFindOne = jest.spyOn(LockDao_1.LockDao.prototype, 'findOne').mockResolvedValueOnce(null);
        const spyCreate = jest.spyOn(LockDao_1.LockDao.prototype, 'create').mockRejectedValue(null);
        await expect(lockService.obtainLock(lockAttributes)).rejects.toThrowError();
        expect(setTimeout).toHaveBeenCalledTimes(0);
        expect(dao.findOne).toHaveBeenCalledTimes(1);
        expect(dao.create).toHaveBeenCalledTimes(1);
        spyFindOne.mockClear();
        spyCreate.mockClear();
    });
});
describe('Test LockService #releaseLock', () => {
    it('should throw an error is lock is not found', async () => {
        const lockService = new LockService_1.LockService(systemInstanceID);
        const dao = new LockDao_1.LockDao();
        const lockID = (0, uuid_1.v4)();
        const spyFindById = jest.spyOn(LockDao_1.LockDao.prototype, 'findOne').mockResolvedValueOnce(null);
        await lockService.releaseLock(lockID);
        expect(dao.findOne).toHaveBeenCalledTimes(1);
        spyFindById.mockClear();
    });
    it('should return success when lock has been removed', async () => {
        const lockService = new LockService_1.LockService(systemInstanceID);
        const lockID = (0, uuid_1.v4)();
        const dao = new LockDao_1.LockDao();
        const lock = { id: lockID };
        const spyFindById = jest.spyOn(LockDao_1.LockDao.prototype, 'findOne').mockResolvedValueOnce(lock);
        const spyDelete = jest.spyOn(LockDao_1.LockDao.prototype, 'delete').mockResolvedValueOnce(1);
        await lockService.releaseLock(lockID);
        expect(dao.findOne).toHaveBeenCalledTimes(1);
        expect(dao.delete).toHaveBeenCalledTimes(1);
        spyFindById.mockClear();
        spyDelete.mockClear();
    });
});
//# sourceMappingURL=LockService.test.js.map