"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.NodeQueryApi = void 0;
const node_path_1 = __importDefault(require("node:path"));
const node_fs_1 = require("node:fs");
const data_access_layer_1 = require("@jfrog-pipelines/data-access-layer");
const configuration_1 = require("@jfrog-pipelines/configuration");
const metadata_1 = require("@jfrog-pipelines/metadata");
const microservice_1 = require("@jfrog-pipelines/microservice");
const utility_1 = require("@jfrog-pipelines/utility");
const errors_1 = require("@jfrog-pipelines/errors");
const acl_1 = require("../../acl");
const NodePoolService_1 = require("../nodePool/NodePoolService");
const constant_1 = require("../../shared/constant");
const NodeDao_1 = require("./NodeDao");
class NodeQueryApi {
    constructor() {
        this.userService = new acl_1.UserRepository();
        this.nodeDao = new NodeDao_1.NodeDao();
        this.nodePoolService = new NodePoolService_1.NodePoolService();
        this.properties = configuration_1.ConfigFactory.config;
    }
    async getNodeInitScript(nodeId, nodePoolIds) {
        const unifiedUrl = await this.userService.getUnifiedUiAndWwwUrl();
        const rtUiUrl = unifiedUrl.rtUiUrl;
        const node = await this.getNode(nodeId, nodePoolIds);
        if (!node) {
            throw new errors_1.NotFoundError(`Node node found for id: ${nodeId} or you don't have sufficient permission`);
        }
        const nodePool = await this.nodePoolService.getNodePool(node.nodePoolId);
        if (!nodePool) {
            throw new errors_1.NotFoundError(`NodePool not found for nodePoold: ${node.nodePoolId}`);
        }
        const scriptFileExtension = metadata_1.NodeMetadata.getScriptFileExtension(nodePool.architecture, nodePool.operatingSystem);
        const buildPlaneArtifactExtension = metadata_1.NodeMetadata.getBuildPlaneArtifactExtension(nodePool.architecture, nodePool.operatingSystem);
        this.validateSupport(nodePool, buildPlaneArtifactExtension, scriptFileExtension);
        return this.generateInitScript(node, nodePool, rtUiUrl, buildPlaneArtifactExtension, scriptFileExtension);
    }
    getNode(nodeId, nodePoolIds) {
        const query = new data_access_layer_1.SequelizeQueryBuilder({}).where('id', nodeId);
        if (nodePoolIds) {
            query.where('nodePoolId', nodePoolIds);
        }
        return this.nodeDao.findOne(query.build());
    }
    generateInitScript(node, nodePool, rtUiUrl, buildPlaneArtifactExtension, scriptFileExtension) {
        const scriptPath = `./marshaller/templates/static/${nodePool.architecture}/${nodePool.operatingSystem}/boot.${scriptFileExtension}`;
        const filePath = node_path_1.default.join(__dirname, scriptPath);
        const fileContent = (0, node_fs_1.readFileSync)(filePath).toString();
        const scriptData = this.prepareInitScriptData(node, nodePool, rtUiUrl, buildPlaneArtifactExtension);
        const templateData = (0, utility_1.template)(fileContent)(scriptData);
        const fileName = `node_init_${node.id}.${scriptFileExtension}`;
        return {
            scriptData: templateData,
            scriptName: fileName
        };
    }
    prepareInitScriptData(node, nodePool, rtUiUrl, buildPlaneArtifactExtension) {
        const externalApiUrl = this.properties.externalApiUrl;
        const parameters = {
            nodeId: node.id,
            projectId: node.projectId,
            buildPlaneVersion: node.intendedBuildPlaneVersion,
            apiUrl: externalApiUrl,
            baseUrlUI: rtUiUrl,
            runMode: this.properties.runMode,
            architecture: nodePool.architecture,
            operatingSystem: nodePool.operatingSystem,
            buildPlaneArtifactDownloadUrl: `${externalApiUrl}/passthrough/artifacts/buildPlane-${nodePool.architecture}-${nodePool.operatingSystem}.${buildPlaneArtifactExtension}`,
            configureNode: false,
            installPrereqs: true,
            isSwapEnabled: false,
            noVerifySsl: this.properties.noVerifySsl,
            customCertsData: this.getCustomCertData(),
            customCertsEnabled: this.properties.customCertsEnabled,
            httpProxy: process.env.http_proxy ?? '',
            httpsProxy: process.env.https_proxy ?? '',
            noProxy: process.env.no_proxy ?? '',
            token: node.systemPropertyBag?.token,
            nodePollerIntervalMS: this.properties.nodePollerIntervalMS,
            rootUserInit: !nodePool.initAsNonRoot,
            insecureDockerRegistries: this.getInsecureDockerRegistryParams(node)
        };
        this.setStaticNodeProperties(parameters, node);
        return parameters;
    }
    getCustomCertData() {
        const certInitialzer = microservice_1.applicationContext.get(microservice_1.CERTIFICATE_INITIALIZER);
        return certInitialzer?.customCertsData;
    }
    getInsecureDockerRegistryParams(node) {
        return JSON.stringify(node.initPropertyBag?.insecureDockerRegistries ?? []);
    }
    setStaticNodeProperties(parameters, node) {
        if (!node.isOnDemand) {
            parameters.installPrereqs = true;
            this.setSwapProperties(parameters, node);
            this.setProxyProperties(parameters);
        }
    }
    setSwapProperties(parameters, node) {
        if (node.isSwapEnabled) {
            parameters.configureNode = true;
            parameters.isSwapEnabled = true;
        }
    }
    setProxyProperties(parameters) {
        if ((0, utility_1.isEmpty)(parameters.httpProxy) || (0, utility_1.isEmpty)(parameters.httpsProxy) || (0, utility_1.isEmpty)(parameters.noProxy)) {
            parameters.configureNode = true;
        }
    }
    validateSupport(nodePool, buildPlaneArtifactExtension, scriptFileExtension) {
        if (!metadata_1.NodeMetadata.isSupportedOperatingSystem(nodePool.architecture, nodePool.operatingSystem)) {
            throw new errors_1.BadRequestError(`${nodePool.operatingSystem} has reached end of support. ` +
                `Please change the OS to another supported version. ${constant_1.TROUBLESHOOT_MSG}`);
        }
        if ((0, utility_1.isEmpty)(buildPlaneArtifactExtension) || (0, utility_1.isEmpty)(scriptFileExtension)) {
            throw new errors_1.BadRequestError(`node init files not found for architecture: ${nodePool.architecture} & operating system: ${nodePool.operatingSystem}`);
        }
    }
}
exports.NodeQueryApi = NodeQueryApi;
//# sourceMappingURL=NodeApi.js.map