"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.NodeService = exports.NodeActions = exports.nodeAccessorID = exports.idleNodeStatuses = exports.unhealthyNodeStatuses = exports.activeNodeStatus = exports.NodeStatus = exports.DEFAULT_FETCH_LIMIT = void 0;
const node_util_1 = require("node:util");
const data_access_layer_1 = require("@jfrog-pipelines/data-access-layer");
const internal_clients_1 = require("@jfrog-pipelines/internal-clients");
const logger_1 = require("@jfrog-pipelines/logger");
const utility_1 = require("@jfrog-pipelines/utility");
const metadata_1 = require("@jfrog-pipelines/metadata");
const NodeConsoleService_1 = require("../nodeConsole/NodeConsoleService");
const NodeStatsService_1 = require("../nodeStats/NodeStatsService");
const BuildPlaneMessageService_1 = require("../buildPlaneMessage/BuildPlaneMessageService");
const UserRepository_1 = require("../../acl/userService/UserRepository");
const NodeDao_1 = require("./NodeDao");
const Marshaller_1 = require("./Marshaller");
const Marshaller_2 = require("./marshaller/Marshaller");
exports.DEFAULT_FETCH_LIMIT = 100;
var NodeStatus;
(function (NodeStatus) {
    NodeStatus[NodeStatus["queued"] = 4000] = "queued";
    NodeStatus[NodeStatus["processing"] = 4001] = "processing";
    NodeStatus[NodeStatus["success"] = 4002] = "success";
    NodeStatus[NodeStatus["failure"] = 4003] = "failure";
    NodeStatus[NodeStatus["waiting"] = 4005] = "waiting";
    NodeStatus[NodeStatus["cancelled"] = 4006] = "cancelled";
    NodeStatus[NodeStatus["unstable"] = 4007] = "unstable";
    NodeStatus[NodeStatus["skipped"] = 4008] = "skipped";
    NodeStatus[NodeStatus["timeout"] = 4009] = "timeout";
    NodeStatus[NodeStatus["stopped"] = 4010] = "stopped";
    NodeStatus[NodeStatus["deleted"] = 4011] = "deleted";
    NodeStatus[NodeStatus["cached"] = 4012] = "cached";
    NodeStatus[NodeStatus["cancelling"] = 4013] = "cancelling";
    NodeStatus[NodeStatus["timingout"] = 4014] = "timingout";
})(NodeStatus = exports.NodeStatus || (exports.NodeStatus = {}));
exports.activeNodeStatus = [
    NodeStatus.success,
    NodeStatus.waiting,
    NodeStatus.stopped,
    NodeStatus.failure,
    NodeStatus.queued,
    NodeStatus.processing,
    NodeStatus.cached
];
exports.unhealthyNodeStatuses = [NodeStatus.failure, NodeStatus.deleted, NodeStatus.stopped];
exports.idleNodeStatuses = [NodeStatus.waiting, NodeStatus.success, NodeStatus.cached];
exports.nodeAccessorID = '1';
var NodeActions;
(function (NodeActions) {
    NodeActions[NodeActions["CREATE"] = 10] = "CREATE";
    NodeActions[NodeActions["CACHE"] = 20] = "CACHE";
    NodeActions[NodeActions["START"] = 30] = "START";
    NodeActions[NodeActions["DELETE"] = 40] = "DELETE";
})(NodeActions = exports.NodeActions || (exports.NodeActions = {}));
class NodeService {
    constructor(publisher) {
        this.publisher = publisher;
        this.logger = logger_1.LoggerFactory.getLogger('NodeService');
        this.nodeDao = new NodeDao_1.NodeDao();
        this.accessClient = internal_clients_1.AccessClientFactory.getClient();
        this.nodeConsoleAdapter = new NodeConsoleService_1.NodeConsoleService();
        this.nodeStatsService = new NodeStatsService_1.NodeStatsService();
        this.cloudMarshaller = new Marshaller_2.Marshaller(this.nodeConsoleAdapter);
        this.userRepository = new UserRepository_1.UserRepository();
        this.buildPlaneMessageService = new BuildPlaneMessageService_1.BuildPlaneMessageService();
    }
    async createNode(attribute) {
        try {
            let node = await this.nodeDao.create(attribute);
            const nodeToken = await this.getNodeAccessToken(node);
            this.logger.trace(`Successfully created nodeToken for ${node.friendlyName}`);
            node = await this.updateNodeBootStrapToken(node, nodeToken);
            const masterIntegration = await this.userRepository.getProjectIntegrationById(node.projectIntegrationId);
            if (masterIntegration.masterIntegrationName !== `amazonKeys` &&
                masterIntegration.masterIntegrationName !== `gcloudKey`) {
                await Marshaller_1.Marshaller.callMarshaller({ action: NodeActions.CREATE, nodeId: node.id });
                this.logger.trace(`Succesfully published message to marshaller to provision node for ${node.friendlyName}`);
                await this.sendWWWSignal(node);
                return node;
            }
            await this.createNodeInstance(node);
            await this.sendWWWSignal(node);
            return node;
        }
        catch (error) {
            this.logger.error(`Failed to create node due to error : ${(0, node_util_1.inspect)(error)}`);
            throw error;
        }
    }
    async createBetaNode(nodeCreateAttributes, runtime) {
        const node = await this.nodeDao.create(nodeCreateAttributes);
        this.logger.debug(`Node with node pool id ${nodeCreateAttributes.nodePoolId} created successfully with name ${node.friendlyName}`);
        const nodeToken = await this.getNodeAccessToken(node);
        this.logger.trace(`Succesfully created nodeToken for ${node.friendlyName}`);
        await this.updateNodeBootStrapToken(node, nodeToken);
        await Marshaller_1.Marshaller.callMarshaller({
            action: NodeActions.CREATE,
            nodeId: node.id,
            runtime: (0, utility_1.getRuntimeFromString)(runtime)
        });
        this.logger.trace(`Succesfully provisioned node for ${node.friendlyName}`);
        await this.sendWWWSignal(node);
        return node;
    }
    async getRunningNodes(poolID) {
        const queryBuilder = new data_access_layer_1.SequelizeQueryBuilder({})
            .where(`nodePoolId`, poolID)
            .where('statusCode', exports.activeNodeStatus)
            .where('stopRequested', 'false');
        return this.nodeDao.findAll(queryBuilder.build());
    }
    async getWaitingNodes(poolID) {
        const queryBuilder = new data_access_layer_1.SequelizeQueryBuilder({})
            .where(`nodePoolId`, poolID)
            .where('statusCode', NodeStatus.waiting)
            .where('stopRequested', 'false');
        return this.nodeDao.findAll(queryBuilder.build());
    }
    async getAllNodes(poolID) {
        const queryBuilder = new data_access_layer_1.SequelizeQueryBuilder({}).where(`nodePoolId`, poolID);
        return this.nodeDao.findAll(queryBuilder.build());
    }
    async getOrphanNodes() {
        const queryBuilder = new data_access_layer_1.SequelizeQueryBuilder({}).where(`nodePoolId`, null);
        return this.nodeDao.findAll(queryBuilder.build());
    }
    async applyNodeDeletedState(node) {
        const masterIntegration = await this.userRepository.getProjectIntegrationById(node.projectIntegrationId);
        if (masterIntegration.masterIntegrationName !== `amazonKeys` &&
            masterIntegration.masterIntegrationName !== `gcloudKey`) {
            await Marshaller_1.Marshaller.callMarshaller({ action: NodeActions.DELETE, nodeId: node.id });
            await this.applyNodeState(node, NodeStatus.stopped);
            return 0;
        }
        try {
            await this.stopNode(node);
            await this.resetBuildPlaneMessage(node);
            if (!(0, utility_1.isEmpty)(node.instanceId)) {
                await this.cloudMarshaller.deleteInstance(node);
                await this.nodeDao.updateByInstance(node, {
                    instanceInfo: {},
                    instanceId: null,
                    providerMetadataPropertyBag: {}
                });
            }
            await this.nodeStatsService.deleteByNodeId(node.id);
            await this.nodeConsoleAdapter.deleteByNodeId(node.id);
            const queryBuilder = new data_access_layer_1.SequelizeQueryBuilder({});
            queryBuilder.where('id', node.id);
            return await this.nodeDao.delete(queryBuilder.build());
        }
        catch (error) {
            this.logger.error(`Failed to apply delete state to node ${node.id} of nodePoolId ${node.nodePoolId} due to ${(0, node_util_1.inspect)(error)}`);
            throw error;
        }
    }
    async applyNodeToCachedState(node) {
        const masterIntegration = await this.userRepository.getProjectIntegrationById(node.projectIntegrationId);
        if (masterIntegration.masterIntegrationName !== `amazonKeys` &&
            masterIntegration.masterIntegrationName !== `gcloudKey`) {
            await Marshaller_1.Marshaller.callMarshaller({ action: NodeActions.CACHE, nodeId: node.id });
            const updatedNode = await this.applyNodeState(node, NodeStatus.cached);
            return updatedNode;
        }
        await this.resetBuildPlaneMessage(node);
        await this.cloudMarshaller.suspendInstance(node);
        return this.applyNodeState(node, NodeStatus.cached);
    }
    async applyNodeWaitingState(node) {
        const masterIntegration = await this.userRepository.getProjectIntegrationById(node.projectIntegrationId);
        if (masterIntegration.masterIntegrationName !== `amazonKeys` &&
            masterIntegration.masterIntegrationName !== `gcloudKey`) {
            await Marshaller_1.Marshaller.callMarshaller({ action: NodeActions.START, nodeId: node.id });
            const updatedNode = await this.applyNodeState(node, NodeStatus.waiting);
            return updatedNode;
        }
        await this.cloudMarshaller.resumeInstance(node);
        return this.applyNodeState(node, NodeStatus.waiting);
    }
    async createNodeInstance(node) {
        try {
            const instanceCreatedResponse = await this.cloudMarshaller.createInstance(node);
            await this.nodeDao.updateByInstance(node, {
                instanceInfo: instanceCreatedResponse.instanceInfo,
                instanceId: instanceCreatedResponse.instanceId,
                provisionedAt: new Date(),
                statusCode: metadata_1.WAITING,
                statusLastUpdatedAt: new Date(),
                providerMetadataPropertyBag: {
                    ...node.providerMetadataPropertyBag
                }
            });
            this.logger.trace(`Successfully provision node for ${node.friendlyName}`);
        }
        catch (error) {
            this.logger.error(`Failed to create instance for node id ${node.id} due to error ${(0, node_util_1.inspect)(error)}`);
            await this.nodeDao.updateByInstance(node, { statusCode: metadata_1.FAILED });
        }
    }
    async applyNodeState(node, status) {
        try {
            this.logger.trace(`Updating the node ${node.id}:${node.friendlyName} with status: ${status}`);
            const updateResponse = await this.nodeDao.updateByInstance(node, {
                statusCode: status,
                statusLastUpdatedAt: new Date()
            });
            return updateResponse;
        }
        catch (error) {
            this.logger.warn(`Failed to applyNodeState to ${status} for node id ${node.id} due to err ${(0, node_util_1.inspect)(error)} `);
        }
        return node;
    }
    async getNodeAccessToken(node) {
        return this.accessClient.getNodeAccessToken(node);
    }
    async updateNodeBootStrapToken(node, token, txn) {
        return this.nodeDao.updateByInstance(node, { systemPropertyBag: { token: token } }, { transaction: txn });
    }
    async sendWWWSignal(node) {
        const signalPayload = {
            eventType: `node.created`,
            rooms: `v3/projects/${node.projectId}`,
            payload: { id: node.id }
        };
        return this.publisher.publishMessage(`www.signals`, signalPayload);
    }
    async resetBuildPlaneMessage(node) {
        try {
            await this.buildPlaneMessageService.resetBuildPlane(node);
        }
        catch (error) {
            this.logger.error(`Failed to reset build plane message for ${node.id} ${(0, node_util_1.inspect)(error)}`);
        }
    }
    async stopNode(node) {
        try {
            this.logger.info(`Stopping and revoking access for node ${node.id} `);
            await Promise.all([
                this.accessClient.revokeToken(node.systemPropertyBag.token),
                this.nodeDao.updateByInstance(node, {
                    statusCode: metadata_1.STOPPED
                })
            ]);
        }
        catch (error) {
            this.logger.error(`Failed to stop node ${node.id} due to ${error.message}`);
        }
    }
}
exports.NodeService = NodeService;
//# sourceMappingURL=NodeService.js.map