"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const amqp_client_1 = require("@jfrog-pipelines/amqp-client");
const internal_clients_1 = require("@jfrog-pipelines/internal-clients");
const logger_1 = require("@jfrog-pipelines/logger");
const configuration_1 = require("@jfrog-pipelines/configuration");
const NodeDao_1 = require("../NodeDao");
const NodeService_1 = require("../NodeService");
const NodeStatsService_1 = require("../../nodeStats/NodeStatsService");
const NodeConsoleService_1 = require("../../nodeConsole/NodeConsoleService");
const UserRepository_1 = require("../../../acl/userService/UserRepository");
const Marshaller_1 = require("./../marshaller/Marshaller");
jest.mock('../Marshaller.ts');
jest.mock('./../marshaller/Marshaller');
jest.mock('@jfrog-pipelines/data-access-layer', () => {
    const actualDataAccessLayer = jest.requireActual('@jfrog-pipelines/data-access-layer');
    return {
        ...actualDataAccessLayer,
        SequelizeTransactionManager: jest.fn().mockReturnValue({
            getTransaction: jest.fn().mockResolvedValue({ commit: jest.fn(), rollback: jest.fn() })
        })
    };
});
jest.mock('@jfrog-pipelines/configuration', () => {
    const actualTracker = jest.requireActual('@jfrog-pipelines/configuration');
    return {
        ...actualTracker,
        config: {
            apiToken: 'sdfgsd',
            instanceType: 't3a.medium',
            getCustom: jest.fn(),
            customCertsEnabled: false,
            isSaas: true
        },
        configure: jest.fn()
    };
});
jest.mock('@jfrog/access-nodejs-client');
jest.mock('@jfrog-pipelines/amqp-client');
jest.mock('@jfrog-pipelines/logger', () => {
    const actualTracker = jest.requireActual('@jfrog-pipelines/logger');
    return {
        ...actualTracker,
        LoggerFactory: {
            getLogger: jest.fn().mockReturnValue({
                info: jest.fn(),
                error: jest.fn(),
                debug: jest.fn(),
                trace: jest.fn(),
                warn: jest.fn()
            }),
            configure: jest.fn()
        }
    };
});
const client = new amqp_client_1.AmqpConnectionManagerProxy('testcode', 'testversion', { amqpUrl: '', heartbeatIntervalInSeconds: 30 }, logger_1.LoggerFactory.getLogger('NodeServiceTest'));
const publisher = new amqp_client_1.AmqpPublisher(client.getConnectionManager(), '', amqp_client_1.publisherOptions, logger_1.LoggerFactory.getLogger('NodeServiceTest'));
const publishMessageMock = jest.spyOn(amqp_client_1.AmqpPublisher.prototype, 'publishMessage').mockResolvedValueOnce({});
jest.spyOn(configuration_1.ConfigFactory, 'config', 'get').mockImplementation(() => {
    return { rtRouterJoinKey: 'abcd1234' };
});
jest.spyOn(internal_clients_1.AccessClientFactory, 'getClient').mockReturnValue({
    getNodeAccessToken: jest.fn().mockReturnValue('token'),
    revokeToken: jest.fn().mockImplementation(jest.fn())
});
jest
    .spyOn(NodeDao_1.NodeDao.prototype, 'updateByInstance')
    .mockImplementation(async (model, update) => {
    return { ...model, ...update };
});
const projectIntegration = {
    formJSONValues: [{ label: 'key', value: 'value' }],
    masterIntegrationName: 'amazonKeys'
};
jest.spyOn(UserRepository_1.UserRepository.prototype, 'getProjectIntegrationById').mockResolvedValue(projectIntegration);
describe('Test NodeService #createNode', () => {
    afterEach(() => {
        publishMessageMock.mockClear();
        jest.clearAllMocks();
    });
    it('should be able to create node based on node create attributes', async () => {
        const nodeCreateAttributes = {
            friendlyName: 'test',
            projectId: 1,
            nodePoolId: 1,
            systemPropertyBag: {
                token: 'token'
            }
        };
        const dao = new NodeDao_1.NodeDao();
        const expectedValue = {
            friendlyName: 'test',
            projectId: 1,
            nodePoolId: 1,
            systemPropertyBag: { token: 'token' }
        };
        const createMock = jest.spyOn(NodeDao_1.NodeDao.prototype, 'create').mockResolvedValueOnce(expectedValue);
        jest.spyOn(Marshaller_1.Marshaller.prototype, 'createInstance').mockResolvedValueOnce({ instanceId: 'test', instanceInfo: {} });
        const service = new NodeService_1.NodeService(publisher);
        const gotResponse = await service.createNode(nodeCreateAttributes);
        expect(dao.create).toHaveBeenCalled();
        expect(gotResponse).toStrictEqual(expectedValue);
        expect(publisher.publishMessage).toHaveBeenCalled();
        createMock.mockClear();
    });
});
describe('Test NodeService #createBetaNode', () => {
    afterEach(() => {
        publishMessageMock.mockClear();
    });
    it('should be able to create beta node based on node create attributes', async () => {
        const nodeCreateAttributes = {
            friendlyName: 'test',
            projectId: 1,
            nodePoolId: 1
        };
        const dao = new NodeDao_1.NodeDao();
        const expectedValue = { friendlyName: 'test', projectId: 1, nodePoolId: 1 };
        const createMock = jest.spyOn(NodeDao_1.NodeDao.prototype, 'create').mockResolvedValueOnce(expectedValue);
        const service = new NodeService_1.NodeService(publisher);
        const gotResponse = await service.createBetaNode(nodeCreateAttributes, 'imageName:tag');
        expect(dao.create).toHaveBeenCalled();
        expect(gotResponse).toBe(expectedValue);
        expect(publisher.publishMessage).toHaveBeenCalled();
        createMock.mockClear();
    });
});
describe('Test NodeService #getRunningNodes', () => {
    afterEach(() => {
        publishMessageMock.mockClear();
        jest.clearAllMocks();
    });
    it('should return nodes with active status', async () => {
        const dao = new NodeDao_1.NodeDao();
        const expectedResponse = [
            { friendlyName: 'test', projectId: 1, nodePoolId: 1, statusCode: NodeService_1.NodeStatus.success }
        ];
        const findAllMock = jest.spyOn(NodeDao_1.NodeDao.prototype, 'findAll').mockResolvedValueOnce(expectedResponse);
        const service = new NodeService_1.NodeService(publisher);
        const gotResponse = await service.getRunningNodes(1);
        expect(dao.findAll).toHaveBeenCalled();
        expect(gotResponse).toBe(expectedResponse);
        findAllMock.mockClear();
    });
    it('should return null if no active nodes found', async () => {
        const dao = new NodeDao_1.NodeDao();
        const findAllMock = jest.spyOn(NodeDao_1.NodeDao.prototype, 'findAll').mockImplementationOnce(() => Promise.resolve([]));
        const service = new NodeService_1.NodeService(publisher);
        const gotResponse = await service.getRunningNodes(1);
        expect(dao.findAll).toHaveBeenCalled();
        expect(gotResponse).toStrictEqual([]);
        findAllMock.mockClear();
    });
});
describe('Test NodeService #getWaitingNodes', () => {
    afterEach(() => {
        publishMessageMock.mockClear();
        jest.clearAllMocks();
    });
    it('should return nodes with active status', async () => {
        const dao = new NodeDao_1.NodeDao();
        const expectedResponse = [
            { friendlyName: 'test', projectId: 1, nodePoolId: 1, statusCode: NodeService_1.NodeStatus.waiting }
        ];
        const findAllMock = jest.spyOn(NodeDao_1.NodeDao.prototype, 'findAll').mockResolvedValueOnce(expectedResponse);
        const service = new NodeService_1.NodeService(publisher);
        const gotResponse = await service.getWaitingNodes(1);
        expect(dao.findAll).toHaveBeenCalled();
        expect(gotResponse).toBe(expectedResponse);
        findAllMock.mockClear();
    });
    it('should return null if no active nodes found', async () => {
        const dao = new NodeDao_1.NodeDao();
        const findAllMock = jest.spyOn(NodeDao_1.NodeDao.prototype, 'findAll').mockImplementationOnce(() => Promise.resolve([]));
        const service = new NodeService_1.NodeService(publisher);
        const gotResponse = await service.getWaitingNodes(1);
        expect(dao.findAll).toHaveBeenCalled();
        expect(gotResponse).toStrictEqual([]);
        findAllMock.mockClear();
    });
});
describe('Test NodeService #applyNodeDeletedState', () => {
    afterEach(() => {
        publishMessageMock.mockClear();
        jest.clearAllMocks();
    });
    it('should change the node status to stopped', async () => {
        const node = {
            id: 1,
            friendlyName: 'test',
            projectId: 1,
            nodePoolId: 1,
            statusCode: NodeService_1.NodeStatus.stopped,
            instanceId: 'testinstanceid',
            systemPropertyBag: {
                token: 'token'
            }
        };
        jest.spyOn(NodeDao_1.NodeDao.prototype, 'delete').mockResolvedValue(1);
        jest.spyOn(Marshaller_1.Marshaller.prototype, 'deleteInstance').mockImplementation(jest.fn());
        jest.spyOn(NodeStatsService_1.NodeStatsService.prototype, 'deleteByNodeId').mockImplementation(jest.fn());
        jest.spyOn(NodeConsoleService_1.NodeConsoleService.prototype, 'deleteByNodeId').mockImplementation(jest.fn());
        const service = new NodeService_1.NodeService(publisher);
        await service.applyNodeDeletedState(node);
        expect(NodeDao_1.NodeDao.prototype.delete).toHaveBeenCalledTimes(1);
        expect(Marshaller_1.Marshaller.prototype.deleteInstance).toHaveBeenCalledTimes(1);
        expect(NodeStatsService_1.NodeStatsService.prototype.deleteByNodeId).toHaveBeenCalledTimes(1);
        expect(NodeConsoleService_1.NodeConsoleService.prototype.deleteByNodeId).toHaveBeenCalledTimes(1);
    });
});
describe('Test NodeService #applyNodeToCachedState', () => {
    afterEach(() => {
        publishMessageMock.mockClear();
        jest.clearAllMocks();
    });
    it('should change node status to cached', async () => {
        const node = { friendlyName: 'test', projectId: 1, nodePoolId: 1, statusCode: NodeService_1.NodeStatus.cached };
        const dao = new NodeDao_1.NodeDao();
        const service = new NodeService_1.NodeService(publisher);
        const expectedResponse = await service.applyNodeToCachedState(node);
        expect(dao.updateByInstance).toHaveBeenCalled();
        expect(expectedResponse.statusCode).toBe(NodeService_1.NodeStatus.cached);
    });
});
describe('Test NodeService #applyNodeWaitingState', () => {
    afterEach(() => {
        publishMessageMock.mockClear();
        jest.clearAllMocks();
    });
    it('should change node status from cached to waiting', async () => {
        const node = { friendlyName: 'test', projectId: 1, nodePoolId: 1, statusCode: NodeService_1.NodeStatus.waiting };
        const dao = new NodeDao_1.NodeDao();
        const service = new NodeService_1.NodeService(publisher);
        const expectedResponse = await service.applyNodeWaitingState(node);
        expect(dao.updateByInstance).toHaveBeenCalled();
        expect(expectedResponse.statusCode).toBe(NodeService_1.NodeStatus.waiting);
    });
});
//# sourceMappingURL=NodeService.test.js.map