"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CloudServiceClient = void 0;
const cloud_1 = require("@jfrog-pipelines/cloud");
const configuration_1 = require("@jfrog-pipelines/configuration");
const microservice_1 = require("@jfrog-pipelines/microservice");
const UserRepository_1 = require("../../../acl/userService/UserRepository");
const SystemServiceRepository_1 = require("../../../acl/systemService/SystemServiceRepository");
const AwsRequestTransformer_1 = require("./AwsRequestTransformer");
const GcloudRequestTransformer_1 = require("./GcloudRequestTransformer");
const providerType = {
    amazonKeys: 'aws',
    gcloudKey: 'gce',
    azureKeys: 'azure',
    kubernetesConfig: 'k8s'
};
class CloudServiceClient {
    constructor() {
        this.userRepository = new UserRepository_1.UserRepository();
        this.systemServiceRepository = new SystemServiceRepository_1.SystemServiceRepository();
    }
    async getNodeProvider(node, region, zone) {
        const projectIntegration = await this.userRepository.getProjectIntegrationById(node.projectIntegrationId);
        const integrationValues = this.transformFormJSONValues(projectIntegration.formJSONValues);
        const providerName = providerType[projectIntegration.masterIntegrationName];
        if (!providerName) {
            throw new Error(`Invalid provider name: ${providerName}`);
        }
        integrationValues.region = region;
        if (providerName === 'gce') {
            integrationValues.region = zone;
        }
        const saasEnvironment = configuration_1.ConfigFactory.config.getCustom('runtime.saasEnvironment');
        const customerName = configuration_1.ConfigFactory.config.getCustom('runtime.saasCustomerName');
        const isSass = configuration_1.ConfigFactory.config.isSaaS;
        const instanceVariables = new cloud_1.InstanceVariables(isSass, customerName, saasEnvironment);
        return cloud_1.CloudFactory.getProvider(providerName, integrationValues, instanceVariables);
    }
    async getProviderRequestTransformer(providerType) {
        const runMode = configuration_1.ConfigFactory.config.runMode;
        const nodePollerIntervalMS = configuration_1.ConfigFactory.config.nodePollerIntervalMS;
        const customCertsEnabled = configuration_1.ConfigFactory.config.customCertsEnabled;
        const noVerifySsl = configuration_1.ConfigFactory.config.noVerifySsl;
        let externalApiUrl = await this.systemServiceRepository.getExternalURL();
        if (externalApiUrl.trim() == '') {
            externalApiUrl = configuration_1.ConfigFactory.config.externalApiUrl;
        }
        const customCertsData = this.getCustomCertData();
        switch (providerType.constructor) {
            case cloud_1.AwsProvider: {
                return new AwsRequestTransformer_1.AwsRequestTransformer(runMode, externalApiUrl, nodePollerIntervalMS, noVerifySsl, customCertsEnabled, customCertsData);
            }
            case cloud_1.GcloudProvider: {
                return new GcloudRequestTransformer_1.GcloudRequestTransformer(runMode, externalApiUrl, nodePollerIntervalMS, noVerifySsl, customCertsEnabled, customCertsData);
            }
            default: {
                throw new Error('Invalid Cloud Provider');
            }
        }
    }
    transformFormJSONValues(formValues) {
        const formKeyValues = {};
        for (const values of formValues) {
            formKeyValues[values.label] = values.value;
        }
        return formKeyValues;
    }
    getCustomCertData() {
        const certInitialzer = microservice_1.applicationContext.get(microservice_1.CERTIFICATE_INITIALIZER);
        return certInitialzer?.customCertsData;
    }
}
exports.CloudServiceClient = CloudServiceClient;
//# sourceMappingURL=CloudServiceClient.js.map