"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Marshaller = void 0;
const node_util_1 = require("node:util");
const NodePoolDao_1 = require("../../nodePool/NodePoolDao");
const CloudServiceClient_1 = require("./CloudServiceClient");
class Marshaller {
    constructor(ledger) {
        this.ledger = ledger;
        this.cloudServiceClient = new CloudServiceClient_1.CloudServiceClient();
        this.nodePoolDao = new NodePoolDao_1.NodePoolDao();
    }
    async createInstance(node) {
        const nodePool = await this.nodePoolDao.findById(node.nodePoolId);
        if (!nodePool) {
            throw new Error(`Cannot find node pool by id ${node.nodePoolId}`);
        }
        this.validateNodeConfig(node);
        const region = node.providerMetadataPropertyBag.region;
        const cloudProvider = await this.cloudServiceClient.getNodeProvider(node, region, nodePool.providerMetadataPropertyBag?.zone);
        const requestTransformer = await this.cloudServiceClient.getProviderRequestTransformer(cloudProvider);
        const instanceRequestPayload = await requestTransformer.getNodeCreateRequest(nodePool, node);
        try {
            const instance = await cloudProvider.createInstance(instanceRequestPayload);
            const instanceReponse = this.getInstanceResponse(instance, node, nodePool);
            await this.ledger.push(`Instance creation response :\n ${(0, node_util_1.inspect)(instanceReponse.instanceInfo)}`, node.id);
            return instanceReponse;
        }
        catch (error) {
            const failedMessage = `Failed to create instance for nodepool ${nodePool.name} due to error ${error.message}`;
            await this.ledger.push(failedMessage, node.id);
            throw error;
        }
    }
    async deleteInstance(node) {
        try {
            const region = node.providerMetadataPropertyBag?.region;
            const zone = node.providerMetadataPropertyBag?.zone;
            const cloudProvider = await this.cloudServiceClient.getNodeProvider(node, region, zone);
            const requestTransformer = await this.cloudServiceClient.getProviderRequestTransformer(cloudProvider);
            const instanceRequestPayload = requestTransformer.getNodeDeleteRequest(node);
            await cloudProvider.terminateInstance(instanceRequestPayload);
        }
        catch (error) {
            const failedMessage = `Failed to delete node ${node.id} due to error ${error.message}`;
            await this.ledger.push(failedMessage, node.id);
            throw error;
        }
    }
    async suspendInstance(node) {
        try {
            const region = node.providerMetadataPropertyBag?.region;
            const zone = node.providerMetadataPropertyBag?.zone;
            const cloudProvider = await this.cloudServiceClient.getNodeProvider(node, region, zone);
            const requestTransformer = await this.cloudServiceClient.getProviderRequestTransformer(cloudProvider);
            const instanceRequestPayload = requestTransformer.getNodeTransitionRequest(node);
            await cloudProvider.suspendInstance(instanceRequestPayload);
            await this.ledger.push(`Node has been suspended`, node.id);
        }
        catch (error) {
            const failedMessage = `Failed to suspend node ${node.id} due to error ${error.message}`;
            await this.ledger.push(failedMessage, node.id);
            throw error;
        }
    }
    async resumeInstance(node) {
        try {
            const region = node.providerMetadataPropertyBag?.region;
            const zone = node.providerMetadataPropertyBag?.zone;
            const cloudProvider = await this.cloudServiceClient.getNodeProvider(node, region, zone);
            const requestTransformer = await this.cloudServiceClient.getProviderRequestTransformer(cloudProvider);
            const instanceRequestPayload = requestTransformer.getNodeTransitionRequest(node);
            await cloudProvider.startInstance(instanceRequestPayload);
            await this.ledger.push(`Node has been resumed`, node.id);
        }
        catch (error) {
            const failedMessage = `Failed to resume node ${node.id} due to error ${error.message}`;
            await this.ledger.push(failedMessage, node.id);
            throw error;
        }
    }
    validateNodeConfig(node) {
        const region = node.providerMetadataPropertyBag.region;
        const imageId = node.providerMetadataPropertyBag.imageId;
        if (region === '' || imageId == '') {
            throw new Error(`No image id or region specified`);
        }
    }
    getInstanceResponse(instance, node, nodepool) {
        const awsInstanceResponse = instance.getInstance();
        const gcpInstanceResponse = instance.getGcpNode();
        if (awsInstanceResponse) {
            return {
                instanceInfo: awsInstanceResponse,
                instanceId: awsInstanceResponse.InstanceId
            };
        }
        if (gcpInstanceResponse) {
            return {
                instanceInfo: {
                    imageName: node.providerMetadataPropertyBag?.imageMetadata?.imageName,
                    projectName: node.providerMetadataPropertyBag?.imageMetadata?.projectName,
                    subnetName: nodepool.providerMetadataPropertyBag?.subnetName,
                    networkTags: nodepool.providerMetadataPropertyBag?.networkTags,
                    diskSizeinGB: nodepool.diskSizeinGB,
                    assignPublicIp: nodepool.providerMetadataPropertyBag?.assignPublicIp,
                    instanceId: node.friendlyName
                },
                instanceId: node.friendlyName
            };
        }
        throw new Error('Failed getting response');
    }
}
exports.Marshaller = Marshaller;
//# sourceMappingURL=Marshaller.js.map