"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const cloud_1 = require("@jfrog-pipelines/cloud");
const Marshaller_1 = require("../Marshaller");
const NodePoolDao_1 = require("../../../nodePool/NodePoolDao");
const CloudServiceClient_1 = require("../CloudServiceClient");
const BuildPlaneImageService_1 = require("../../../buildPlaneImage/BuildPlaneImageService");
const UserRepository_1 = require("../../../../acl/userService/UserRepository");
const AwsRequestTransformer_1 = require("../AwsRequestTransformer");
const NodeConsoleService_1 = require("../../../nodeConsole/NodeConsoleService");
jest.mock('@jfrog-pipelines/configuration', () => {
    return {
        ConfigFactory: {
            config: {
                getCustom: (key) => {
                    if (key === 'runtime.saasEnvironment') {
                        return 'development';
                    }
                    if (key === 'runtime.saasCustomerName') {
                        return 'customer';
                    }
                    if (key === 'runtime.isSaas') {
                        return 'true';
                    }
                }
            }
        }
    };
});
jest.mock('@jfrog-pipelines/logger', () => {
    const actualTracker = jest.requireActual('@jfrog-pipelines/logger');
    return {
        ...actualTracker,
        LoggerFactory: {
            getLogger: jest.fn().mockReturnValue({
                info: jest.fn(),
                error: jest.fn(),
                debug: jest.fn(),
                trace: jest.fn(),
                warn: jest.fn()
            }),
            configure: jest.fn(),
            getEventLogger: jest.fn().mockReturnValue({
                info: jest.fn(),
                error: jest.fn(),
                debug: jest.fn(),
                trace: jest.fn(),
                warn: jest.fn()
            })
        }
    };
});
const defaultImageRegion = 'us-east-1';
describe('Marshaller', () => {
    const nodePollerInterval = 500;
    const ledger = new NodeConsoleService_1.NodeConsoleService();
    jest.spyOn(NodeConsoleService_1.NodeConsoleService.prototype, 'push').mockImplementation(jest.fn());
    jest.spyOn(BuildPlaneImageService_1.BuildPlaneImageService.prototype, 'getBuildPlaneImageById').mockResolvedValue({
        id: 1,
        region: defaultImageRegion
    });
    const marshaller = new Marshaller_1.Marshaller(ledger);
    describe('createInstance', () => {
        it('should create an instance and update the ledger', async () => {
            const node = {
                nodePoolId: 1,
                systemPropertyBag: {},
                providerMetadataPropertyBag: {
                    region: 'us-east-1',
                    imageId: 'image-id-1',
                    instanceSize: 't3a.medium'
                }
            };
            const nodePool = {
                name: 'poolName',
                id: 1,
                architecture: 'x86_64',
                operatingSystem: 'Ubuntu_20.04',
                providerMetadataPropertyBag: {}
            };
            const awsCredential = new cloud_1.AwsCredential('access', 'secret', node.providerMetadataPropertyBag.region);
            jest.spyOn(NodePoolDao_1.NodePoolDao.prototype, 'findById').mockResolvedValue(nodePool);
            jest.spyOn(CloudServiceClient_1.CloudServiceClient.prototype, 'getNodeProvider').mockImplementationOnce(jest.fn().mockImplementationOnce(() => {
                return new cloud_1.AwsProvider(awsCredential, new cloud_1.InstanceVariables(true, 'cust', 'saas'));
            }));
            jest.spyOn(CloudServiceClient_1.CloudServiceClient.prototype, 'getProviderRequestTransformer').mockImplementationOnce(jest.fn().mockImplementationOnce(() => {
                return new AwsRequestTransformer_1.AwsRequestTransformer('runMode', 'url', nodePollerInterval, true, false, 'customCerts');
            }));
            jest.spyOn(AwsRequestTransformer_1.AwsRequestTransformer.prototype, 'getNodeCreateRequest');
            jest.spyOn(UserRepository_1.UserRepository.prototype, 'getProjectById').mockResolvedValueOnce({
                id: 1
            });
            jest
                .spyOn(UserRepository_1.UserRepository.prototype, 'getUnifiedUiAndWwwUrl')
                .mockResolvedValueOnce({ rtUiUrl: 'testrt', wwwUrl: 'testwww' });
            const createInstanceResponse = new cloud_1.CreateInstanceResponse({ InstanceId: 'instanceid' });
            jest.spyOn(cloud_1.AwsProvider.prototype, 'createInstance').mockResolvedValueOnce(createInstanceResponse);
            await marshaller.createInstance(node);
            expect(NodePoolDao_1.NodePoolDao.prototype.findById).toHaveBeenCalledWith(1);
            expect(CloudServiceClient_1.CloudServiceClient.prototype.getNodeProvider).toHaveBeenCalledWith(node, defaultImageRegion, undefined);
            expect(CloudServiceClient_1.CloudServiceClient.prototype.getProviderRequestTransformer).toHaveBeenCalledTimes(1);
            expect(AwsRequestTransformer_1.AwsRequestTransformer.prototype.getNodeCreateRequest).toHaveBeenCalledTimes(1);
            expect(ledger.push).toHaveBeenCalledTimes(1);
        });
        it('should throw an error if node pool is not found', async () => {
            const node = {
                nodePoolId: 1,
                providerMetadataPropertyBag: {
                    region: 'us-east-1',
                    imageId: 'image-id-1'
                }
            };
            jest.spyOn(NodePoolDao_1.NodePoolDao.prototype, 'findById').mockResolvedValueOnce(null);
            await expect(marshaller.createInstance(node)).rejects.toThrow('Cannot find node pool by id 1');
            expect(NodePoolDao_1.NodePoolDao.prototype.findById).toHaveBeenCalledWith(1);
        });
        it('should handle error while creating an instance', async () => {
            const node = {
                id: 1,
                nodePoolId: 1,
                providerMetadataPropertyBag: {
                    region: 'us-east-1',
                    imageId: 'image-id-1'
                }
            };
            const nodePool = {
                name: 'poolName',
                id: 1,
                architecture: 'x86_64',
                operatingSystem: 'Ubuntu_20.04',
                providerMetadataPropertyBag: {}
            };
            const cloudProviderMock = {
                createInstance: jest.fn().mockImplementationOnce(() => {
                    throw new Error('Instance creation failed');
                }),
                terminateInstance: jest.fn(),
                suspendInstance: jest.fn(),
                startInstance: jest.fn(),
                listInstanceTypes: jest.fn(),
                getInstanceIds: jest.fn(),
                getInstanceRegions: jest.fn()
            };
            jest.spyOn(NodePoolDao_1.NodePoolDao.prototype, 'findById').mockResolvedValue(nodePool);
            jest.spyOn(CloudServiceClient_1.CloudServiceClient.prototype, 'getNodeProvider').mockImplementationOnce(jest.fn().mockImplementationOnce(() => {
                return cloudProviderMock;
            }));
            jest.spyOn(CloudServiceClient_1.CloudServiceClient.prototype, 'getProviderRequestTransformer').mockImplementationOnce(jest.fn().mockImplementationOnce(() => {
                return {
                    getNodeCreateRequest: jest.fn()
                };
            }));
            await expect(marshaller.createInstance(node)).rejects.toThrow('Instance creation failed');
            expect(cloudProviderMock.createInstance).toHaveBeenCalledTimes(1);
            expect(ledger.push).toHaveBeenCalledTimes(1);
            expect(ledger.push).toHaveBeenNthCalledWith(1, `Failed to create instance for nodepool poolName due to error Instance creation failed`, 1);
        });
    });
    describe('deleteInstance', () => {
        it('should delete an instance', async () => {
            const node = {
                id: 1,
                providerMetadataPropertyBag: {
                    region: 'us-east-1',
                    imageId: 'image-id-1'
                }
            };
            const cloudProviderMock = {
                createInstance: jest.fn(),
                terminateInstance: jest.fn(),
                suspendInstance: jest.fn(),
                startInstance: jest.fn(),
                listInstanceTypes: jest.fn(),
                getInstanceIds: jest.fn(),
                getInstanceRegions: jest.fn()
            };
            jest.spyOn(CloudServiceClient_1.CloudServiceClient.prototype, 'getNodeProvider').mockImplementationOnce(jest.fn().mockImplementationOnce(() => {
                return cloudProviderMock;
            }));
            jest.spyOn(CloudServiceClient_1.CloudServiceClient.prototype, 'getProviderRequestTransformer').mockImplementationOnce(jest.fn().mockImplementationOnce(() => {
                return new AwsRequestTransformer_1.AwsRequestTransformer('runMode', 'url', nodePollerInterval, true, false, 'customCerts');
            }));
            jest.spyOn(AwsRequestTransformer_1.AwsRequestTransformer.prototype, 'getNodeDeleteRequest').mockImplementationOnce(jest.fn());
            await marshaller.deleteInstance(node);
            expect(CloudServiceClient_1.CloudServiceClient.prototype.getNodeProvider).toHaveBeenCalledWith(node, defaultImageRegion, undefined);
            expect(CloudServiceClient_1.CloudServiceClient.prototype.getProviderRequestTransformer).toHaveBeenCalledTimes(1);
            expect(AwsRequestTransformer_1.AwsRequestTransformer.prototype.getNodeDeleteRequest).toHaveBeenCalledTimes(1);
            expect(cloudProviderMock.terminateInstance).toHaveBeenCalledTimes(1);
        });
        it('should handle error while deleting an instance', async () => {
            const node = {
                id: 1,
                providerMetadataPropertyBag: {
                    region: 'us-east-1',
                    imageId: 'image-id-1'
                }
            };
            const cloudProviderMock = {
                createInstance: jest.fn(),
                terminateInstance: jest.fn().mockImplementationOnce(() => {
                    throw new Error('Instance deletion failed');
                }),
                suspendInstance: jest.fn(),
                startInstance: jest.fn(),
                listInstanceTypes: jest.fn(),
                getInstanceIds: jest.fn(),
                getInstanceRegions: jest.fn()
            };
            jest.spyOn(CloudServiceClient_1.CloudServiceClient.prototype, 'getNodeProvider').mockImplementationOnce(jest.fn().mockImplementationOnce(() => {
                return cloudProviderMock;
            }));
            jest.spyOn(CloudServiceClient_1.CloudServiceClient.prototype, 'getProviderRequestTransformer').mockImplementationOnce(jest.fn().mockImplementationOnce(() => {
                return new AwsRequestTransformer_1.AwsRequestTransformer('runMode', 'url', nodePollerInterval, true, false, 'customCerts');
            }));
            jest.spyOn(AwsRequestTransformer_1.AwsRequestTransformer.prototype, 'getNodeDeleteRequest').mockImplementationOnce(jest.fn());
            await expect(marshaller.deleteInstance(node)).rejects.toThrow('Instance deletion failed');
            expect(CloudServiceClient_1.CloudServiceClient.prototype.getNodeProvider).toHaveBeenCalledWith(node, defaultImageRegion, undefined);
            expect(CloudServiceClient_1.CloudServiceClient.prototype.getProviderRequestTransformer).toHaveBeenCalledTimes(1);
            expect(AwsRequestTransformer_1.AwsRequestTransformer.prototype.getNodeDeleteRequest).toHaveBeenCalledTimes(1);
            expect(cloudProviderMock.terminateInstance).toHaveBeenCalledTimes(1);
            expect(ledger.push).toHaveBeenCalledTimes(1);
            expect(ledger.push).toHaveBeenCalledWith(`Failed to delete node 1 due to error Instance deletion failed`, 1);
        });
        it('should suspend an instance', async () => {
            const node = {
                id: 1,
                providerMetadataPropertyBag: {
                    region: 'us-east-1',
                    imageId: 'image-id-1'
                }
            };
            const cloudProviderMock = {
                createInstance: jest.fn(),
                terminateInstance: jest.fn(),
                suspendInstance: jest.fn(),
                startInstance: jest.fn(),
                listInstanceTypes: jest.fn(),
                getInstanceIds: jest.fn(),
                getInstanceRegions: jest.fn()
            };
            jest.spyOn(CloudServiceClient_1.CloudServiceClient.prototype, 'getNodeProvider').mockImplementationOnce(jest.fn().mockImplementationOnce(() => {
                return cloudProviderMock;
            }));
            jest.spyOn(CloudServiceClient_1.CloudServiceClient.prototype, 'getProviderRequestTransformer').mockImplementationOnce(jest.fn().mockImplementationOnce(() => {
                return new AwsRequestTransformer_1.AwsRequestTransformer('runMode', 'url', nodePollerInterval, true, false, 'customCerts');
            }));
            jest.spyOn(AwsRequestTransformer_1.AwsRequestTransformer.prototype, 'getNodeTransitionRequest').mockImplementationOnce(jest.fn());
            await marshaller.suspendInstance(node);
            expect(CloudServiceClient_1.CloudServiceClient.prototype.getNodeProvider).toHaveBeenCalledWith(node, defaultImageRegion, undefined);
            expect(CloudServiceClient_1.CloudServiceClient.prototype.getProviderRequestTransformer).toHaveBeenCalledTimes(1);
            expect(AwsRequestTransformer_1.AwsRequestTransformer.prototype.getNodeTransitionRequest).toHaveBeenCalledTimes(1);
            expect(cloudProviderMock.suspendInstance).toHaveBeenCalledTimes(1);
        });
        it('should handle error while suspending an instance', async () => {
            const node = {
                id: 1,
                providerMetadataPropertyBag: {
                    region: 'us-east-1',
                    imageId: 'image-id-1'
                }
            };
            const cloudProviderMock = {
                createInstance: jest.fn(),
                terminateInstance: jest.fn(),
                suspendInstance: jest.fn().mockImplementationOnce(() => {
                    throw new Error('Instance suspension failed');
                }),
                startInstance: jest.fn(),
                listInstanceTypes: jest.fn(),
                getInstanceIds: jest.fn(),
                getInstanceRegions: jest.fn()
            };
            jest.spyOn(CloudServiceClient_1.CloudServiceClient.prototype, 'getNodeProvider').mockImplementationOnce(jest.fn().mockImplementationOnce(() => {
                return cloudProviderMock;
            }));
            jest.spyOn(CloudServiceClient_1.CloudServiceClient.prototype, 'getProviderRequestTransformer').mockImplementationOnce(jest.fn().mockImplementationOnce(() => {
                return new AwsRequestTransformer_1.AwsRequestTransformer('runMode', 'url', nodePollerInterval, true, false, 'customCerts');
            }));
            jest.spyOn(AwsRequestTransformer_1.AwsRequestTransformer.prototype, 'getNodeTransitionRequest').mockImplementationOnce(jest.fn());
            await expect(marshaller.suspendInstance(node)).rejects.toThrow('Instance suspension failed');
            expect(CloudServiceClient_1.CloudServiceClient.prototype.getNodeProvider).toHaveBeenCalledWith(node, defaultImageRegion, undefined);
            expect(CloudServiceClient_1.CloudServiceClient.prototype.getProviderRequestTransformer).toHaveBeenCalledTimes(1);
            expect(AwsRequestTransformer_1.AwsRequestTransformer.prototype.getNodeTransitionRequest).toHaveBeenCalledTimes(1);
            expect(cloudProviderMock.suspendInstance).toHaveBeenCalledTimes(1);
            expect(ledger.push).toHaveBeenCalledTimes(1);
            expect(ledger.push).toHaveBeenCalledWith(`Failed to suspend node 1 due to error Instance suspension failed`, 1);
        });
    });
    describe('resumeInstance', () => {
        it('should resume an instance', async () => {
            const node = {
                id: 1,
                providerMetadataPropertyBag: {
                    region: 'us-east-1',
                    imageId: 'image-id-1'
                }
            };
            const cloudProviderMock = {
                createInstance: jest.fn(),
                terminateInstance: jest.fn(),
                suspendInstance: jest.fn(),
                startInstance: jest.fn(),
                listInstanceTypes: jest.fn(),
                getInstanceIds: jest.fn(),
                getInstanceRegions: jest.fn()
            };
            jest.spyOn(CloudServiceClient_1.CloudServiceClient.prototype, 'getNodeProvider').mockImplementationOnce(jest.fn().mockImplementationOnce(() => {
                return cloudProviderMock;
            }));
            jest.spyOn(CloudServiceClient_1.CloudServiceClient.prototype, 'getProviderRequestTransformer').mockImplementationOnce(jest.fn().mockImplementationOnce(() => {
                return new AwsRequestTransformer_1.AwsRequestTransformer('runMode', 'url', nodePollerInterval, true, false, 'customCerts');
            }));
            jest.spyOn(AwsRequestTransformer_1.AwsRequestTransformer.prototype, 'getNodeTransitionRequest').mockImplementationOnce(jest.fn());
            await marshaller.resumeInstance(node);
            expect(CloudServiceClient_1.CloudServiceClient.prototype.getNodeProvider).toHaveBeenCalledWith(node, defaultImageRegion, undefined);
            expect(CloudServiceClient_1.CloudServiceClient.prototype.getProviderRequestTransformer).toHaveBeenCalledTimes(1);
            expect(AwsRequestTransformer_1.AwsRequestTransformer.prototype.getNodeTransitionRequest).toHaveBeenCalledTimes(1);
            expect(cloudProviderMock.startInstance).toHaveBeenCalledTimes(1);
        });
        it('should handle error while resuming an instance', async () => {
            const node = {
                id: 1,
                providerMetadataPropertyBag: {
                    region: 'us-east-1',
                    imageId: 'image-id-1'
                }
            };
            const cloudProviderMock = {
                createInstance: jest.fn(),
                terminateInstance: jest.fn(),
                suspendInstance: jest.fn(),
                startInstance: jest.fn().mockImplementationOnce(() => {
                    throw new Error('Instance resumption failed');
                }),
                listInstanceTypes: jest.fn(),
                getInstanceIds: jest.fn(),
                getInstanceRegions: jest.fn()
            };
            jest.spyOn(CloudServiceClient_1.CloudServiceClient.prototype, 'getNodeProvider').mockImplementationOnce(jest.fn().mockImplementationOnce(() => {
                return cloudProviderMock;
            }));
            jest.spyOn(CloudServiceClient_1.CloudServiceClient.prototype, 'getProviderRequestTransformer').mockImplementationOnce(jest.fn().mockImplementationOnce(() => {
                return new AwsRequestTransformer_1.AwsRequestTransformer('runMode', 'url', nodePollerInterval, true, false, 'customCerts');
            }));
            jest.spyOn(AwsRequestTransformer_1.AwsRequestTransformer.prototype, 'getNodeTransitionRequest').mockImplementationOnce(jest.fn());
            await expect(marshaller.resumeInstance(node)).rejects.toThrow('Instance resumption failed');
            expect(CloudServiceClient_1.CloudServiceClient.prototype.getNodeProvider).toHaveBeenCalledWith(node, defaultImageRegion, undefined);
            expect(CloudServiceClient_1.CloudServiceClient.prototype.getProviderRequestTransformer).toHaveBeenCalledTimes(1);
            expect(AwsRequestTransformer_1.AwsRequestTransformer.prototype.getNodeTransitionRequest).toHaveBeenCalledTimes(1);
            expect(cloudProviderMock.startInstance).toHaveBeenCalledTimes(1);
            expect(ledger.push).toHaveBeenCalledTimes(1);
            expect(ledger.push).toHaveBeenCalledWith(`Failed to resume node 1 due to error Instance resumption failed`, 1);
        });
    });
});
//# sourceMappingURL=Marshaller.test.js.map